@@ -24,10 +24,13 @@ import (
 
 	"k8s.io/kops/pkg/apis/kops"
 	"k8s.io/kops/pkg/bootstrap"
+	"k8s.io/kops/pkg/kopscontrollerclient"
 	"k8s.io/kops/pkg/wellknownports"
 	"k8s.io/kops/upup/pkg/fi"
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
 	"k8s.io/kops/upup/pkg/fi/cloudup/gce/tpm/gcetpmsigner"
+	"k8s.io/kops/upup/pkg/fi/cloudup/hetzner"
+	"k8s.io/kops/upup/pkg/fi/cloudup/openstack"
 	"k8s.io/kops/upup/pkg/fi/nodeup/nodetasks"
 )
 
@@ -36,7 +39,7 @@ type BootstrapClientBuilder struct {
 	*NodeupModelContext
 }
 
-func (b BootstrapClientBuilder) Build(c *fi.ModelBuilderContext) error {
+func (b BootstrapClientBuilder) Build(c *fi.NodeupModelBuilderContext) error {
 	if b.IsMaster || !b.UseKopsControllerForNodeBootstrap() {
 		return nil
 	}
@@ -50,6 +53,10 @@ func (b BootstrapClientBuilder) Build(c *fi.ModelBuilderContext) error {
 		authenticator, err = gcetpmsigner.NewTPMAuthenticator()
 		// We don't use the custom resolver here in gossip mode (though we could);
 		// instead we use this as a check that protokube has now started.
+	case kops.CloudProviderHetzner:
+		authenticator, err = hetzner.NewHetznerAuthenticator()
+	case kops.CloudProviderOpenstack:
+		authenticator, err = openstack.NewOpenstackAuthenticator()
 
 	default:
 		return fmt.Errorf("unsupported cloud provider for authenticator %q", b.CloudProvider)
@@ -65,7 +72,7 @@ func (b BootstrapClientBuilder) Build(c *fi.ModelBuilderContext) error {
 		Path:   "/",
 	}
 
-	bootstrapClient := &nodetasks.KopsBootstrapClient{
+	bootstrapClient := &kopscontrollerclient.Client{
 		Authenticator: authenticator,
 		CAs:           []byte(b.NodeupConfig.CAs[fi.CertificateIDCA]),
 		BaseURL:       baseURL,
@@ -86,4 +93,4 @@ func (b BootstrapClientBuilder) Build(c *fi.ModelBuilderContext) error {
 	return nil
 }
 
-var _ fi.ModelBuilder = &BootstrapClientBuilder{}
+var _ fi.NodeupModelBuilder = &BootstrapClientBuilder{}
