@@ -23,11 +23,9 @@ import (
 	v1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/klog/v2"
-	"k8s.io/kops/pkg/dns"
 	"k8s.io/kops/pkg/flagbuilder"
 	"k8s.io/kops/pkg/k8scodecs"
 	"k8s.io/kops/pkg/kubemanifest"
-	"k8s.io/kops/pkg/model/components"
 	"k8s.io/kops/pkg/rbac"
 	"k8s.io/kops/upup/pkg/fi"
 	"k8s.io/kops/upup/pkg/fi/nodeup/nodetasks"
@@ -39,11 +37,11 @@ type KubeProxyBuilder struct {
 	*NodeupModelContext
 }
 
-var _ fi.ModelBuilder = &KubeProxyBuilder{}
+var _ fi.NodeupModelBuilder = &KubeProxyBuilder{}
 
 // Build is responsible for building the kube-proxy manifest
 // @TODO we should probably change this to a daemonset in the future and follow the kubeadm path
-func (b *KubeProxyBuilder) Build(c *fi.ModelBuilderContext) error {
+func (b *KubeProxyBuilder) Build(c *fi.NodeupModelBuilderContext) error {
 	if b.Cluster.Spec.KubeProxy.Enabled != nil && !*b.Cluster.Spec.KubeProxy.Enabled {
 		klog.V(2).Infof("Kube-proxy is disabled, will not create configuration for it.")
 		return nil
@@ -52,7 +50,7 @@ func (b *KubeProxyBuilder) Build(c *fi.ModelBuilderContext) error {
 	if b.IsMaster {
 		// If this is a master that is not isolated, run it as a normal node also (start kube-proxy etc)
 		// This lets e.g. daemonset pods communicate with other pods in the system
-		if fi.BoolValue(b.Cluster.Spec.IsolateMasters) {
+		if fi.ValueOf(b.Cluster.Spec.Networking.IsolateControlPlane) {
 			klog.V(2).Infof("Running on Master with IsolateMaster=true; skipping kube-proxy installation")
 			return nil
 		}
@@ -127,7 +125,7 @@ func (b *KubeProxyBuilder) buildPod() (*v1.Pod, error) {
 			// which would mean that DNS can't rely on API to come up
 			c.Master = "https://127.0.0.1"
 		} else {
-			c.Master = "https://" + b.Cluster.Spec.MasterInternalName
+			c.Master = "https://" + b.Cluster.APIInternalName()
 		}
 	}
 
@@ -167,9 +165,6 @@ func (b *KubeProxyBuilder) buildPod() (*v1.Pod, error) {
 	if b.Architecture != architectures.ArchitectureAmd64 {
 		image = strings.Replace(image, "-amd64", "-"+string(b.Architecture), 1)
 	}
-	if components.IsBaseURL(b.Cluster.Spec.KubernetesVersion) && b.IsKubernetesLT("1.25") {
-		image = strings.Replace(image, "registry.k8s.io", "k8s.gcr.io", 1)
-	}
 
 	container := &v1.Container{
 		Name:  "kube-proxy",
@@ -179,7 +174,7 @@ func (b *KubeProxyBuilder) buildPod() (*v1.Pod, error) {
 			Limits:   resourceLimits,
 		},
 		SecurityContext: &v1.SecurityContext{
-			Privileged: fi.Bool(true),
+			Privileged: fi.PtrTo(true),
 		},
 	}
 
@@ -228,7 +223,7 @@ func (b *KubeProxyBuilder) buildPod() (*v1.Pod, error) {
 		sslCertsHost.VolumeMount.MountPath = "/etc/ssl/certs"
 	}
 
-	if dns.IsGossipHostname(b.Cluster.Name) {
+	if b.Cluster.IsGossip() {
 		// Map /etc/hosts from host, so that we see the updates that are made by protokube
 		kubemanifest.AddHostPathMapping(pod, container, "etchosts", "/etc/hosts")
 	}
