@@ -36,10 +36,10 @@ const (
 	debianPackageName  = "unattended-upgrades"
 )
 
-var _ fi.ModelBuilder = &UpdateServiceBuilder{}
+var _ fi.NodeupModelBuilder = &UpdateServiceBuilder{}
 
 // Build is responsible for configuring automatic updates based on the OS.
-func (b *UpdateServiceBuilder) Build(c *fi.ModelBuilderContext) error {
+func (b *UpdateServiceBuilder) Build(c *fi.NodeupModelBuilderContext) error {
 	if b.Distribution == distributions.DistributionFlatcar {
 		b.buildFlatcarSystemdService(c)
 	} else if b.Distribution.IsDebianFamily() {
@@ -49,12 +49,7 @@ func (b *UpdateServiceBuilder) Build(c *fi.ModelBuilderContext) error {
 	return nil
 }
 
-func (b *UpdateServiceBuilder) buildFlatcarSystemdService(c *fi.ModelBuilderContext) {
-	if b.NodeupConfig.UpdatePolicy != kops.UpdatePolicyExternal {
-		klog.Infof("UpdatePolicy requests automatic updates; skipping creation of systemd unit %q", flatcarServiceName)
-		return
-	}
-
+func (b *UpdateServiceBuilder) buildFlatcarSystemdService(c *fi.NodeupModelBuilderContext) {
 	for _, spec := range b.NodeupConfig.Hooks {
 		for _, hook := range spec {
 			if hook.Name == flatcarServiceName || hook.Name == flatcarServiceName+".service" {
@@ -85,7 +80,7 @@ func (b *UpdateServiceBuilder) buildFlatcarSystemdService(c *fi.ModelBuilderCont
 	c.AddTask(service)
 }
 
-func (b *UpdateServiceBuilder) buildDebianPackage(c *fi.ModelBuilderContext) {
+func (b *UpdateServiceBuilder) buildDebianPackage(c *fi.NodeupModelBuilderContext) {
 	contents := ""
 	if b.NodeupConfig.UpdatePolicy == kops.UpdatePolicyExternal {
 		klog.Infof("UpdatePolicy requests automatic updates; skipping installation of package %q", debianPackageName)
