@@ -27,11 +27,23 @@ func UseKopsControllerForNodeBootstrap(cluster *kops.Cluster) bool {
 		return true
 	case kops.CloudProviderGCE:
 		return cluster.IsKubernetesGTE("1.22")
+	case kops.CloudProviderHetzner:
+		return true
+	case kops.CloudProviderOpenstack:
+		return true
 	default:
 		return false
 	}
 }
 
+// UseKopsControllerForNodeConfig checks if nodeup should use kops-controller to get nodeup.Config.
+func UseKopsControllerForNodeConfig(cluster *kops.Cluster) bool {
+	if cluster.IsGossip() {
+		return false
+	}
+	return UseKopsControllerForNodeBootstrap(cluster)
+}
+
 // UseCiliumEtcd is true if we are using the Cilium etcd cluster.
 func UseCiliumEtcd(cluster *kops.Cluster) bool {
 	if cluster.Spec.Networking.Cilium == nil {
