@@ -35,12 +35,11 @@ import (
 func awsValidateCluster(c *kops.Cluster) field.ErrorList {
 	allErrs := field.ErrorList{}
 
-	if c.Spec.API != nil {
-		if c.Spec.API.LoadBalancer != nil {
-			allErrs = append(allErrs, awsValidateAdditionalSecurityGroups(field.NewPath("spec", "api", "loadBalancer", "additionalSecurityGroups"), c.Spec.API.LoadBalancer.AdditionalSecurityGroups)...)
-			allErrs = append(allErrs, awsValidateSSLPolicy(field.NewPath("spec", "api", "loadBalancer", "sslPolicy"), c.Spec.API.LoadBalancer)...)
-			allErrs = append(allErrs, awsValidateLoadBalancerSubnets(field.NewPath("spec", "api", "loadBalancer", "subnets"), c.Spec)...)
-		}
+	if c.Spec.API.LoadBalancer != nil {
+		allErrs = append(allErrs, awsValidateAdditionalSecurityGroups(field.NewPath("spec", "api", "loadBalancer", "additionalSecurityGroups"), c.Spec.API.LoadBalancer.AdditionalSecurityGroups)...)
+		allErrs = append(allErrs, awsValidateSSLPolicy(field.NewPath("spec", "api", "loadBalancer", "sslPolicy"), c.Spec.API.LoadBalancer)...)
+		allErrs = append(allErrs, awsValidateLoadBalancerSubnets(field.NewPath("spec", "api", "loadBalancer", "subnets"), c.Spec)...)
+		allErrs = append(allErrs, awsValidateTopologyDNS(field.NewPath("spec", "api", "loadBalancer", "type"), c)...)
 	}
 
 	allErrs = append(allErrs, awsValidateExternalCloudControllerManager(c)...)
@@ -49,15 +48,15 @@ func awsValidateCluster(c *kops.Cluster) field.ErrorList {
 		allErrs = append(allErrs, awsValidateIAMAuthenticator(field.NewPath("spec", "authentication", "aws"), c.Spec.Authentication.AWS)...)
 	}
 
-	for i, subnet := range c.Spec.Subnets {
-		f := field.NewPath("spec", "subnets").Index(i)
+	for i, subnet := range c.Spec.Networking.Subnets {
+		f := field.NewPath("spec", "networking", "subnets").Index(i)
 		if subnet.AdditionalRoutes != nil {
-			if len(subnet.ProviderID) > 0 {
+			if len(subnet.ID) > 0 {
 				allErrs = append(allErrs, field.Invalid(f, subnet, "additional routes cannot be added if the subnet is shared"))
 			} else if subnet.Type != kops.SubnetTypePrivate {
 				allErrs = append(allErrs, field.Invalid(f, subnet, "additional routes can only be added on private subnets"))
 			}
-			allErrs = append(allErrs, awsValidateAdditionalRoutes(f.Child("additionalRoutes"), subnet.AdditionalRoutes, c.Spec.NetworkCIDR)...)
+			allErrs = append(allErrs, awsValidateAdditionalRoutes(f.Child("additionalRoutes"), subnet.AdditionalRoutes, c.Spec.Networking.NetworkCIDR)...)
 		}
 	}
 
@@ -73,7 +72,7 @@ func awsValidateExternalCloudControllerManager(cluster *kops.Cluster) (allErrs f
 	fldPath := field.NewPath("spec", "externalCloudControllerManager")
 	if !hasAWSEBSCSIDriver(c) {
 		allErrs = append(allErrs, field.Forbidden(fldPath,
-			"AWS external CCM cannot be used without enabling spec.cloudConfig.AWSEBSCSIDriver."))
+			"AWS external CCM cannot be used without enabling spec.cloudProvider.aws.ebsCSIDriverSpec."))
 	}
 	return allErrs
 }
@@ -127,7 +126,7 @@ func awsValidateInstanceMetadata(fieldPath *field.Path, instanceMetadata *kops.I
 	}
 
 	if instanceMetadata.HTTPPutResponseHopLimit != nil {
-		httpPutResponseHopLimit := fi.Int64Value(instanceMetadata.HTTPPutResponseHopLimit)
+		httpPutResponseHopLimit := fi.ValueOf(instanceMetadata.HTTPPutResponseHopLimit)
 		if httpPutResponseHopLimit < 1 || httpPutResponseHopLimit > 64 {
 			allErrs = append(allErrs, field.Invalid(fieldPath.Child("httpPutResponseHopLimit"), instanceMetadata.HTTPPutResponseHopLimit,
 				"HTTPPutResponseLimit must be a value between 1 and 64"))
@@ -170,20 +169,20 @@ func awsValidateInstanceTypeAndImage(instanceTypeFieldPath *field.Path, imageFie
 		return append(allErrs, field.Invalid(imageFieldPath, image,
 			fmt.Sprintf("specified image %q is invalid: %s", image, err)))
 	}
-	imageArch := fi.StringValue(imageInfo.Architecture)
+	imageArch := fi.ValueOf(imageInfo.Architecture)
 
 	// Spotinst uses the instance type field to keep a "," separated list of instance types
 	for _, instanceType := range strings.Split(instanceTypes, ",") {
 		machineInfo, err := cloud.DescribeInstanceType(instanceType)
 		if err != nil {
-			allErrs = append(allErrs, field.Invalid(instanceTypeFieldPath, instanceTypes, fmt.Sprintf("machine type specified is invalid: %q", instanceType)))
+			allErrs = append(allErrs, field.Invalid(instanceTypeFieldPath, instanceTypes, fmt.Sprintf("machine type %q is invalid: %v", instanceType, err)))
 			continue
 		}
 
 		found := false
 		if machineInfo != nil && machineInfo.ProcessorInfo != nil {
 			for _, machineArch := range machineInfo.ProcessorInfo.SupportedArchitectures {
-				if imageArch == fi.StringValue(machineArch) {
+				if imageArch == fi.ValueOf(machineArch) {
 					found = true
 				}
 			}
@@ -194,7 +193,7 @@ func awsValidateInstanceTypeAndImage(instanceTypeFieldPath *field.Path, imageFie
 				machineArch = fi.StringSliceValue(machineInfo.ProcessorInfo.SupportedArchitectures)
 			}
 			allErrs = append(allErrs, field.Invalid(instanceTypeFieldPath, instanceTypes,
-				fmt.Sprintf("machine type architecture does not match image architecture: %q - %q", strings.Join(machineArch, ","), imageArch)))
+				fmt.Sprintf("machine type architecture %q does not match image architecture %q", strings.Join(machineArch, ","), imageArch)))
 		}
 	}
 
@@ -251,19 +250,19 @@ func awsValidateMixedInstancesPolicy(path *field.Path, spec *kops.MixedInstances
 	}
 
 	if spec.OnDemandBase != nil {
-		if fi.Int64Value(spec.OnDemandBase) < 0 {
+		if fi.ValueOf(spec.OnDemandBase) < 0 {
 			errs = append(errs, field.Invalid(path.Child("onDemandBase"), spec.OnDemandBase, "cannot be less than zero"))
 		}
-		if fi.Int64Value(spec.OnDemandBase) > int64(fi.Int32Value(ig.Spec.MaxSize)) {
+		if fi.ValueOf(spec.OnDemandBase) > int64(fi.ValueOf(ig.Spec.MaxSize)) {
 			errs = append(errs, field.Invalid(path.Child("onDemandBase"), spec.OnDemandBase, "cannot be greater than max size"))
 		}
 	}
 
 	if spec.OnDemandAboveBase != nil {
-		if fi.Int64Value(spec.OnDemandAboveBase) < 0 {
+		if fi.ValueOf(spec.OnDemandAboveBase) < 0 {
 			errs = append(errs, field.Invalid(path.Child("onDemandAboveBase"), spec.OnDemandAboveBase, "cannot be less than 0"))
 		}
-		if fi.Int64Value(spec.OnDemandAboveBase) > 100 {
+		if fi.ValueOf(spec.OnDemandAboveBase) > 100 {
 			errs = append(errs, field.Invalid(path.Child("onDemandAboveBase"), spec.OnDemandAboveBase, "cannot be greater than 100"))
 		}
 	}
@@ -273,6 +272,16 @@ func awsValidateMixedInstancesPolicy(path *field.Path, spec *kops.MixedInstances
 	return errs
 }
 
+func awsValidateTopologyDNS(fieldPath *field.Path, c *kops.Cluster) field.ErrorList {
+	allErrs := field.ErrorList{}
+
+	if c.UsesNoneDNS() && c.Spec.API.LoadBalancer != nil && c.Spec.API.LoadBalancer.Class != kops.LoadBalancerClassNetwork {
+		allErrs = append(allErrs, field.Forbidden(fieldPath, "topology.dns.type=none requires Network Load Balancer"))
+	}
+
+	return allErrs
+}
+
 func awsValidateSSLPolicy(fieldPath *field.Path, spec *kops.LoadBalancerAccessSpec) field.ErrorList {
 	allErrs := field.ErrorList{}
 
@@ -298,7 +307,7 @@ func awsValidateLoadBalancerSubnets(fieldPath *field.Path, spec kops.ClusterSpec
 		if subnet.Name == "" {
 			allErrs = append(allErrs, field.Required(fieldPath.Index(i).Child("name"), "subnet name can't be empty"))
 		} else {
-			for _, cs := range spec.Subnets {
+			for _, cs := range spec.Networking.Subnets {
 				if subnet.Name == cs.Name {
 					clusterSubnet = &cs
 					break
@@ -368,12 +377,12 @@ func awsValidateIAMAuthenticator(fieldPath *field.Path, spec *kops.AWSAuthentica
 }
 
 func hasAWSEBSCSIDriver(c kops.ClusterSpec) bool {
-	// AWSEBSCSIDriver will have a default value, so if this is all false, it will be populated on next pass
-	if c.CloudConfig == nil || c.CloudConfig.AWSEBSCSIDriver == nil || c.CloudConfig.AWSEBSCSIDriver.Enabled == nil {
+	// EBSCSIDriverSpec will have a default value, so if this is all false, it will be populated on next pass
+	if c.CloudProvider.AWS.EBSCSIDriver == nil || c.CloudProvider.AWS.EBSCSIDriver.Enabled == nil {
 		return true
 	}
 
-	return *c.CloudConfig.AWSEBSCSIDriver.Enabled
+	return *c.CloudProvider.AWS.EBSCSIDriver.Enabled
 }
 
 func awsValidateAdditionalRoutes(fieldPath *field.Path, routes []kops.RouteSpec, cidr string) field.ErrorList {
@@ -381,7 +390,7 @@ func awsValidateAdditionalRoutes(fieldPath *field.Path, routes []kops.RouteSpec,
 
 	_, clusterNet, errClusterNet := net.ParseCIDR(cidr)
 	if errClusterNet != nil {
-		allErrs = append(allErrs, field.Invalid(field.NewPath("spec", "networkCIDR"), cidr, "Invalid cluster cidr"))
+		allErrs = append(allErrs, field.Invalid(field.NewPath("spec", "networking", "networkCIDR"), cidr, "Invalid cluster cidr"))
 	} else {
 		for i, r := range routes {
 			f := fieldPath.Index(i)
