@@ -137,9 +137,9 @@ type SpotInstanceGroupModelBuilder struct {
 	SecurityLifecycle      fi.Lifecycle
 }
 
-var _ fi.ModelBuilder = &SpotInstanceGroupModelBuilder{}
+var _ fi.CloudupModelBuilder = &SpotInstanceGroupModelBuilder{}
 
-func (b *SpotInstanceGroupModelBuilder) Build(c *fi.ModelBuilderContext) error {
+func (b *SpotInstanceGroupModelBuilder) Build(c *fi.CloudupModelBuilderContext) error {
 	var nodeSpotInstanceGroups []*kops.InstanceGroup
 	var err error
 
@@ -157,7 +157,7 @@ func (b *SpotInstanceGroupModelBuilder) Build(c *fi.ModelBuilderContext) error {
 		switch ig.Spec.Role {
 
 		// Create both Master and Bastion instance groups as Elastigroups.
-		case kops.InstanceGroupRoleMaster, kops.InstanceGroupRoleBastion:
+		case kops.InstanceGroupRoleControlPlane, kops.InstanceGroupRoleBastion:
 			err = b.buildElastigroup(c, ig)
 
 		// Create Node instance groups as Elastigroups or a single Ocean with
@@ -187,14 +187,14 @@ func (b *SpotInstanceGroupModelBuilder) Build(c *fi.ModelBuilderContext) error {
 	return nil
 }
 
-func (b *SpotInstanceGroupModelBuilder) buildElastigroup(c *fi.ModelBuilderContext, ig *kops.InstanceGroup) (err error) {
+func (b *SpotInstanceGroupModelBuilder) buildElastigroup(c *fi.CloudupModelBuilderContext, ig *kops.InstanceGroup) (err error) {
 	klog.V(4).Infof("Building instance group as Elastigroup: %q", b.AutoscalingGroupName(ig))
 	group := &spotinsttasks.Elastigroup{
 		Lifecycle:            b.Lifecycle,
-		Name:                 fi.String(b.AutoscalingGroupName(ig)),
-		Region:               fi.String(b.Region),
-		ImageID:              fi.String(ig.Spec.Image),
-		OnDemandInstanceType: fi.String(strings.Split(ig.Spec.MachineType, ",")[0]),
+		Name:                 fi.PtrTo(b.AutoscalingGroupName(ig)),
+		Region:               fi.PtrTo(b.Region),
+		ImageID:              fi.PtrTo(ig.Spec.Image),
+		OnDemandInstanceType: fi.PtrTo(strings.Split(ig.Spec.MachineType, ",")[0]),
 		SpotInstanceTypes:    strings.Split(ig.Spec.MachineType, ","),
 	}
 
@@ -214,7 +214,7 @@ func (b *SpotInstanceGroupModelBuilder) buildElastigroup(c *fi.ModelBuilderConte
 			}
 
 		case SpotInstanceGroupLabelOrientation:
-			group.Orientation = fi.String(v)
+			group.Orientation = fi.PtrTo(v)
 
 		case SpotInstanceGroupLabelUtilizeReservedInstances:
 			group.UtilizeReservedInstances, err = parseBool(v)
@@ -241,7 +241,7 @@ func (b *SpotInstanceGroupModelBuilder) buildElastigroup(c *fi.ModelBuilderConte
 			}
 
 		case SpotInstanceGroupLabelHealthCheckType:
-			group.HealthCheckType = fi.String(strings.ToUpper(v))
+			group.HealthCheckType = fi.PtrTo(strings.ToUpper(v))
 		}
 	}
 
@@ -264,7 +264,7 @@ func (b *SpotInstanceGroupModelBuilder) buildElastigroup(c *fi.ModelBuilderConte
 
 	// Tenancy.
 	if ig.Spec.Tenancy != "" {
-		group.Tenancy = fi.String(ig.Spec.Tenancy)
+		group.Tenancy = fi.PtrTo(ig.Spec.Tenancy)
 	}
 
 	// Security groups.
@@ -324,21 +324,21 @@ func (b *SpotInstanceGroupModelBuilder) buildElastigroup(c *fi.ModelBuilderConte
 		group.AutoScalerOpts.Taints = nil
 	}
 
-	klog.V(4).Infof("Adding task: Elastigroup/%s", fi.StringValue(group.Name))
+	klog.V(4).Infof("Adding task: Elastigroup/%s", fi.ValueOf(group.Name))
 	c.AddTask(group)
 
 	return nil
 }
 
-func (b *SpotInstanceGroupModelBuilder) buildOcean(c *fi.ModelBuilderContext, igs ...*kops.InstanceGroup) (err error) {
+func (b *SpotInstanceGroupModelBuilder) buildOcean(c *fi.CloudupModelBuilderContext, igs ...*kops.InstanceGroup) (err error) {
 	klog.V(4).Infof("Building instance group as Ocean: %q", "nodes."+b.ClusterName())
 	ocean := &spotinsttasks.Ocean{
 		Lifecycle: b.Lifecycle,
-		Name:      fi.String("nodes." + b.ClusterName()),
+		Name:      fi.PtrTo("nodes." + b.ClusterName()),
 	}
 
 	if featureflag.SpotinstOceanTemplate.Enabled() {
-		ocean.UseAsTemplateOnly = fi.Bool(true)
+		ocean.UseAsTemplateOnly = fi.PtrTo(true)
 	}
 
 	if len(igs) == 0 {
@@ -352,7 +352,7 @@ func (b *SpotInstanceGroupModelBuilder) buildOcean(c *fi.ModelBuilderContext, ig
 				if err != nil {
 					continue
 				}
-				if fi.BoolValue(defaultLaunchSpec) {
+				if fi.ValueOf(defaultLaunchSpec) {
 					if ig != nil {
 						return fmt.Errorf("unable to detect default launch spec: "+
 							"multiple instance groups labeled with `%s: \"true\"`",
@@ -371,7 +371,7 @@ func (b *SpotInstanceGroupModelBuilder) buildOcean(c *fi.ModelBuilderContext, ig
 	klog.V(4).Infof("Detected default launch spec: %q", b.AutoscalingGroupName(ig))
 
 	// Image.
-	ocean.ImageID = fi.String(ig.Spec.Image)
+	ocean.ImageID = fi.PtrTo(ig.Spec.Image)
 
 	// Strategy and instance types.
 	for k, v := range ig.ObjectMeta.Labels {
@@ -452,10 +452,10 @@ func (b *SpotInstanceGroupModelBuilder) buildOcean(c *fi.ModelBuilderContext, ig
 		ocean.AutoScalerOpts.Headroom = nil
 	}
 
-	if !fi.BoolValue(ocean.UseAsTemplateOnly) {
+	if !fi.ValueOf(ocean.UseAsTemplateOnly) {
 		// Capacity.
-		ocean.MinSize = fi.Int64(0)
-		ocean.MaxSize = fi.Int64(0)
+		ocean.MinSize = fi.PtrTo(int64(0))
+		ocean.MaxSize = fi.PtrTo(int64(0))
 
 		// User data.
 		ocean.UserData, err = b.BootstrapScriptBuilder.ResourceNodeUp(c, ig)
@@ -499,19 +499,19 @@ func (b *SpotInstanceGroupModelBuilder) buildOcean(c *fi.ModelBuilderContext, ig
 		}
 	}
 
-	klog.V(4).Infof("Adding task: Ocean/%s", fi.StringValue(ocean.Name))
+	klog.V(4).Infof("Adding task: Ocean/%s", fi.ValueOf(ocean.Name))
 	c.AddTask(ocean)
 
 	return nil
 }
 
-func (b *SpotInstanceGroupModelBuilder) buildLaunchSpec(c *fi.ModelBuilderContext,
+func (b *SpotInstanceGroupModelBuilder) buildLaunchSpec(c *fi.CloudupModelBuilderContext,
 	ig, igOcean *kops.InstanceGroup, ocean *spotinsttasks.Ocean) (err error) {
 	klog.V(4).Infof("Building instance group as LaunchSpec: %q", b.AutoscalingGroupName(ig))
 	launchSpec := &spotinsttasks.LaunchSpec{
-		Name:      fi.String(b.AutoscalingGroupName(ig)),
+		Name:      fi.PtrTo(b.AutoscalingGroupName(ig)),
 		Lifecycle: b.Lifecycle,
-		ImageID:   fi.String(ig.Spec.Image),
+		ImageID:   fi.PtrTo(ig.Spec.Image),
 		Ocean:     ocean, // link to Ocean
 	}
 
@@ -545,12 +545,12 @@ func (b *SpotInstanceGroupModelBuilder) buildLaunchSpec(c *fi.ModelBuilderContex
 
 	// Capacity.
 	minSize, maxSize := b.buildCapacity(ig)
-	if fi.BoolValue(ocean.UseAsTemplateOnly) {
+	if fi.ValueOf(ocean.UseAsTemplateOnly) {
 		launchSpec.MinSize = minSize
 		launchSpec.MaxSize = maxSize
 	} else {
-		ocean.MinSize = fi.Int64(fi.Int64Value(ocean.MinSize) + fi.Int64Value(minSize))
-		ocean.MaxSize = fi.Int64(fi.Int64Value(ocean.MaxSize) + fi.Int64Value(maxSize))
+		ocean.MinSize = fi.PtrTo(fi.ValueOf(ocean.MinSize) + fi.ValueOf(minSize))
+		ocean.MaxSize = fi.PtrTo(fi.ValueOf(ocean.MaxSize) + fi.ValueOf(maxSize))
 	}
 
 	// User data.
@@ -621,13 +621,13 @@ func (b *SpotInstanceGroupModelBuilder) buildLaunchSpec(c *fi.ModelBuilderContex
 		}
 	}
 
-	klog.V(4).Infof("Adding task: LaunchSpec/%s", fi.StringValue(launchSpec.Name))
+	klog.V(4).Infof("Adding task: LaunchSpec/%s", fi.ValueOf(launchSpec.Name))
 	c.AddTask(launchSpec)
 
 	return nil
 }
 
-func (b *SpotInstanceGroupModelBuilder) buildSecurityGroups(c *fi.ModelBuilderContext,
+func (b *SpotInstanceGroupModelBuilder) buildSecurityGroups(c *fi.CloudupModelBuilderContext,
 	ig *kops.InstanceGroup) ([]*awstasks.SecurityGroup, error) {
 	securityGroups := []*awstasks.SecurityGroup{
 		b.LinkToSecurityGroup(ig.Spec.Role),
@@ -636,9 +636,9 @@ func (b *SpotInstanceGroupModelBuilder) buildSecurityGroups(c *fi.ModelBuilderCo
 	for _, id := range ig.Spec.AdditionalSecurityGroups {
 		sg := &awstasks.SecurityGroup{
 			Lifecycle: b.SecurityLifecycle,
-			ID:        fi.String(id),
-			Name:      fi.String(id),
-			Shared:    fi.Bool(true),
+			ID:        fi.PtrTo(id),
+			Name:      fi.PtrTo(id),
+			Shared:    fi.PtrTo(true),
 		}
 		if err := c.EnsureTask(sg); err != nil {
 			return nil, err
@@ -668,8 +668,8 @@ func (b *SpotInstanceGroupModelBuilder) buildSubnets(ig *kops.InstanceGroup) ([]
 
 func (b *SpotInstanceGroupModelBuilder) buildPublicIPOpts(ig *kops.InstanceGroup) (*bool, error) {
 	subnetMap := make(map[string]*kops.ClusterSubnetSpec)
-	for i := range b.Cluster.Spec.Subnets {
-		subnet := &b.Cluster.Spec.Subnets[i]
+	for i := range b.Cluster.Spec.Networking.Subnets {
+		subnet := &b.Cluster.Spec.Networking.Subnets[i]
 		subnetMap[subnet.Name] = subnet
 	}
 
@@ -703,7 +703,7 @@ func (b *SpotInstanceGroupModelBuilder) buildPublicIPOpts(ig *kops.InstanceGroup
 		return nil, fmt.Errorf("unknown subnet type %q", subnetType)
 	}
 
-	return fi.Bool(associatePublicIP), nil
+	return fi.PtrTo(associatePublicIP), nil
 }
 
 func (b *SpotInstanceGroupModelBuilder) buildRootVolumeOpts(ig *kops.InstanceGroup) (*spotinsttasks.RootVolumeOpts, error) {
@@ -711,21 +711,21 @@ func (b *SpotInstanceGroupModelBuilder) buildRootVolumeOpts(ig *kops.InstanceGro
 
 	// Optimization.
 	{
-		if fi.BoolValue(ig.Spec.RootVolumeOptimization) {
+		if fi.ValueOf(ig.Spec.RootVolumeOptimization) {
 			opts.Optimization = ig.Spec.RootVolumeOptimization
 		}
 	}
 
 	// Encryption.
 	{
-		if fi.BoolValue(ig.Spec.RootVolumeEncryption) {
+		if fi.ValueOf(ig.Spec.RootVolumeEncryption) {
 			opts.Encryption = ig.Spec.RootVolumeEncryption
 		}
 	}
 
 	// Size.
 	{
-		size := fi.Int32Value(ig.Spec.RootVolumeSize)
+		size := fi.ValueOf(ig.Spec.RootVolumeSize)
 		if size == 0 {
 			var err error
 			size, err = defaults.DefaultInstanceGroupVolumeSize(ig.Spec.Role)
@@ -733,31 +733,31 @@ func (b *SpotInstanceGroupModelBuilder) buildRootVolumeOpts(ig *kops.InstanceGro
 				return nil, err
 			}
 		}
-		opts.Size = fi.Int64(int64(size))
+		opts.Size = fi.PtrTo(int64(size))
 	}
 
 	// Type.
 	{
-		typ := fi.StringValue(ig.Spec.RootVolumeType)
+		typ := fi.ValueOf(ig.Spec.RootVolumeType)
 		if typ == "" {
 			typ = "gp2"
 		}
-		opts.Type = fi.String(typ)
+		opts.Type = fi.PtrTo(typ)
 	}
 
 	// IOPS.
 	{
-		iops := fi.Int32Value(ig.Spec.RootVolumeIOPS)
+		iops := fi.ValueOf(ig.Spec.RootVolumeIOPS)
 		if iops > 0 {
-			opts.IOPS = fi.Int64(int64(iops))
+			opts.IOPS = fi.PtrTo(int64(iops))
 		}
 	}
 
 	// Throughput.
 	{
-		throughput := fi.Int32Value(ig.Spec.RootVolumeThroughput)
+		throughput := fi.ValueOf(ig.Spec.RootVolumeThroughput)
 		if throughput > 0 {
-			opts.Throughput = fi.Int64(int64(throughput))
+			opts.Throughput = fi.PtrTo(int64(throughput))
 		}
 	}
 
@@ -767,7 +767,7 @@ func (b *SpotInstanceGroupModelBuilder) buildRootVolumeOpts(ig *kops.InstanceGro
 func (b *SpotInstanceGroupModelBuilder) buildCapacity(ig *kops.InstanceGroup) (*int64, *int64) {
 	minSize := int32(1)
 	if ig.Spec.MinSize != nil {
-		minSize = fi.Int32Value(ig.Spec.MinSize)
+		minSize = fi.ValueOf(ig.Spec.MinSize)
 	} else if ig.Spec.Role == kops.InstanceGroupRoleNode {
 		minSize = 2
 	}
@@ -779,10 +779,10 @@ func (b *SpotInstanceGroupModelBuilder) buildCapacity(ig *kops.InstanceGroup) (*
 		maxSize = 2
 	}
 
-	return fi.Int64(int64(minSize)), fi.Int64(int64(maxSize))
+	return fi.PtrTo(int64(minSize)), fi.PtrTo(int64(maxSize))
 }
 
-func (b *SpotInstanceGroupModelBuilder) buildLoadBalancers(c *fi.ModelBuilderContext,
+func (b *SpotInstanceGroupModelBuilder) buildLoadBalancers(c *fi.CloudupModelBuilderContext,
 	ig *kops.InstanceGroup) ([]*awstasks.ClassicLoadBalancer, []*awstasks.TargetGroup, error) {
 	var loadBalancers []*awstasks.ClassicLoadBalancer
 	var targetGroups []*awstasks.TargetGroup
@@ -807,20 +807,20 @@ func (b *SpotInstanceGroupModelBuilder) buildLoadBalancers(c *fi.ModelBuilderCon
 			lb := &awstasks.ClassicLoadBalancer{
 				Name:             extLB.LoadBalancerName,
 				LoadBalancerName: extLB.LoadBalancerName,
-				Shared:           fi.Bool(true),
+				Shared:           fi.PtrTo(true),
 			}
 			loadBalancers = append(loadBalancers, lb)
 			c.EnsureTask(lb)
 		}
 		if extLB.TargetGroupARN != nil {
-			targetGroupName, err := awsup.GetTargetGroupNameFromARN(fi.StringValue(extLB.TargetGroupARN))
+			targetGroupName, err := awsup.GetTargetGroupNameFromARN(fi.ValueOf(extLB.TargetGroupARN))
 			if err != nil {
 				return nil, nil, err
 			}
 			tg := &awstasks.TargetGroup{
-				Name:   fi.String(ig.Name + "-" + targetGroupName),
+				Name:   fi.PtrTo(ig.Name + "-" + targetGroupName),
 				ARN:    extLB.TargetGroupARN,
-				Shared: fi.Bool(true),
+				Shared: fi.PtrTo(true),
 			}
 			targetGroups = append(targetGroups, tg)
 			c.AddTask(tg)
@@ -840,11 +840,11 @@ func (b *SpotInstanceGroupModelBuilder) buildTags(ig *kops.InstanceGroup) (map[s
 
 func (b *SpotInstanceGroupModelBuilder) buildAutoScalerOpts(clusterID string, ig *kops.InstanceGroup) (*spotinsttasks.AutoScalerOpts, error) {
 	opts := &spotinsttasks.AutoScalerOpts{
-		ClusterID: fi.String(clusterID),
+		ClusterID: fi.PtrTo(clusterID),
 	}
 
 	switch ig.Spec.Role {
-	case kops.InstanceGroupRoleMaster:
+	case kops.InstanceGroupRoleControlPlane:
 		return opts, nil
 
 	case kops.InstanceGroupRoleBastion:
@@ -852,8 +852,8 @@ func (b *SpotInstanceGroupModelBuilder) buildAutoScalerOpts(clusterID string, ig
 	}
 
 	// Enable the auto scaler for Node instance groups.
-	opts.Enabled = fi.Bool(true)
-	opts.AutoConfig = fi.Bool(true)
+	opts.Enabled = fi.PtrTo(true)
+	opts.AutoConfig = fi.PtrTo(true)
 
 	// Parse instance group labels.
 	var defaultNodeLabels bool
@@ -865,7 +865,7 @@ func (b *SpotInstanceGroupModelBuilder) buildAutoScalerOpts(clusterID string, ig
 				if err != nil {
 					return nil, err
 				}
-				opts.Enabled = fi.Bool(!fi.BoolValue(v))
+				opts.Enabled = fi.PtrTo(!fi.ValueOf(v))
 			}
 
 		case SpotInstanceGroupLabelAutoScalerDefaultNodeLabels:
@@ -874,7 +874,7 @@ func (b *SpotInstanceGroupModelBuilder) buildAutoScalerOpts(clusterID string, ig
 				if err != nil {
 					return nil, err
 				}
-				defaultNodeLabels = fi.BoolValue(v)
+				defaultNodeLabels = fi.ValueOf(v)
 			}
 
 		case SpotInstanceGroupLabelAutoScalerCooldown:
@@ -883,7 +883,7 @@ func (b *SpotInstanceGroupModelBuilder) buildAutoScalerOpts(clusterID string, ig
 				if err != nil {
 					return nil, err
 				}
-				opts.Cooldown = fi.Int(int(fi.Int64Value(v)))
+				opts.Cooldown = fi.PtrTo(int(fi.ValueOf(v)))
 			}
 
 		case SpotInstanceGroupLabelAutoScalerAutoConfig:
@@ -901,7 +901,7 @@ func (b *SpotInstanceGroupModelBuilder) buildAutoScalerOpts(clusterID string, ig
 				if err != nil {
 					return nil, err
 				}
-				opts.AutoHeadroomPercentage = fi.Int(int(fi.Int64Value(v)))
+				opts.AutoHeadroomPercentage = fi.PtrTo(int(fi.ValueOf(v)))
 			}
 
 		case SpotInstanceGroupLabelAutoScalerHeadroomCPUPerUnit:
@@ -913,7 +913,7 @@ func (b *SpotInstanceGroupModelBuilder) buildAutoScalerOpts(clusterID string, ig
 				if opts.Headroom == nil {
 					opts.Headroom = new(spotinsttasks.AutoScalerHeadroomOpts)
 				}
-				opts.Headroom.CPUPerUnit = fi.Int(int(fi.Int64Value(v)))
+				opts.Headroom.CPUPerUnit = fi.PtrTo(int(fi.ValueOf(v)))
 			}
 
 		case SpotInstanceGroupLabelAutoScalerHeadroomGPUPerUnit:
@@ -925,7 +925,7 @@ func (b *SpotInstanceGroupModelBuilder) buildAutoScalerOpts(clusterID string, ig
 				if opts.Headroom == nil {
 					opts.Headroom = new(spotinsttasks.AutoScalerHeadroomOpts)
 				}
-				opts.Headroom.GPUPerUnit = fi.Int(int(fi.Int64Value(v)))
+				opts.Headroom.GPUPerUnit = fi.PtrTo(int(fi.ValueOf(v)))
 			}
 
 		case SpotInstanceGroupLabelAutoScalerHeadroomMemPerUnit:
@@ -937,7 +937,7 @@ func (b *SpotInstanceGroupModelBuilder) buildAutoScalerOpts(clusterID string, ig
 				if opts.Headroom == nil {
 					opts.Headroom = new(spotinsttasks.AutoScalerHeadroomOpts)
 				}
-				opts.Headroom.MemPerUnit = fi.Int(int(fi.Int64Value(v)))
+				opts.Headroom.MemPerUnit = fi.PtrTo(int(fi.ValueOf(v)))
 			}
 
 		case SpotInstanceGroupLabelAutoScalerHeadroomNumOfUnits:
@@ -949,7 +949,7 @@ func (b *SpotInstanceGroupModelBuilder) buildAutoScalerOpts(clusterID string, ig
 				if opts.Headroom == nil {
 					opts.Headroom = new(spotinsttasks.AutoScalerHeadroomOpts)
 				}
-				opts.Headroom.NumOfUnits = fi.Int(int(fi.Int64Value(v)))
+				opts.Headroom.NumOfUnits = fi.PtrTo(int(fi.ValueOf(v)))
 			}
 
 		case SpotInstanceGroupLabelAutoScalerScaleDownMaxPercentage:
@@ -973,7 +973,7 @@ func (b *SpotInstanceGroupModelBuilder) buildAutoScalerOpts(clusterID string, ig
 				if opts.Down == nil {
 					opts.Down = new(spotinsttasks.AutoScalerDownOpts)
 				}
-				opts.Down.EvaluationPeriods = fi.Int(int(fi.Int64Value(v)))
+				opts.Down.EvaluationPeriods = fi.PtrTo(int(fi.ValueOf(v)))
 			}
 
 		case SpotInstanceGroupLabelAutoScalerResourceLimitsMaxVCPU:
@@ -985,7 +985,7 @@ func (b *SpotInstanceGroupModelBuilder) buildAutoScalerOpts(clusterID string, ig
 				if opts.ResourceLimits == nil {
 					opts.ResourceLimits = new(spotinsttasks.AutoScalerResourceLimitsOpts)
 				}
-				opts.ResourceLimits.MaxVCPU = fi.Int(int(fi.Int64Value(v)))
+				opts.ResourceLimits.MaxVCPU = fi.PtrTo(int(fi.ValueOf(v)))
 			}
 
 		case SpotInstanceGroupLabelAutoScalerResourceLimitsMaxMemory:
@@ -997,7 +997,7 @@ func (b *SpotInstanceGroupModelBuilder) buildAutoScalerOpts(clusterID string, ig
 				if opts.ResourceLimits == nil {
 					opts.ResourceLimits = new(spotinsttasks.AutoScalerResourceLimitsOpts)
 				}
-				opts.ResourceLimits.MaxMemory = fi.Int(int(fi.Int64Value(v)))
+				opts.ResourceLimits.MaxMemory = fi.PtrTo(int(fi.ValueOf(v)))
 			}
 		}
 	}
@@ -1005,10 +1005,10 @@ func (b *SpotInstanceGroupModelBuilder) buildAutoScalerOpts(clusterID string, ig
 	// Configure Elastigroup defaults to avoid state drifts.
 	if !featureflag.SpotinstOcean.Enabled() {
 		if opts.Cooldown == nil {
-			opts.Cooldown = fi.Int(300)
+			opts.Cooldown = fi.PtrTo(300)
 		}
 		if opts.Down != nil && opts.Down.EvaluationPeriods == nil {
-			opts.Down.EvaluationPeriods = fi.Int(5)
+			opts.Down.EvaluationPeriods = fi.PtrTo(5)
 		}
 	}
 
@@ -1110,7 +1110,7 @@ func defaultSpotPercentage(ig *kops.InstanceGroup) *float64 {
 	var percentage float64
 
 	switch ig.Spec.Role {
-	case kops.InstanceGroupRoleMaster, kops.InstanceGroupRoleBastion:
+	case kops.InstanceGroupRoleControlPlane, kops.InstanceGroupRoleBastion:
 		percentage = 0
 	case kops.InstanceGroupRoleNode:
 		percentage = 100
