@@ -30,26 +30,25 @@ type AWSEBSCSIDriverOptionsBuilder struct {
 var _ loader.OptionsBuilder = &AWSEBSCSIDriverOptionsBuilder{}
 
 func (b *AWSEBSCSIDriverOptionsBuilder) BuildOptions(o interface{}) error {
-	clusterSpec := o.(*kops.ClusterSpec)
-	if clusterSpec.GetCloudProvider() != kops.CloudProviderAWS {
+	aws := o.(*kops.ClusterSpec).CloudProvider.AWS
+	if aws == nil {
 		return nil
 	}
 
-	cc := clusterSpec.CloudConfig
-	if cc.AWSEBSCSIDriver == nil {
-		cc.AWSEBSCSIDriver = &kops.AWSEBSCSIDriver{
-			Enabled: fi.Bool(b.IsKubernetesGTE("1.22")),
+	if aws.EBSCSIDriver == nil {
+		aws.EBSCSIDriver = &kops.EBSCSIDriverSpec{
+			Enabled: fi.PtrTo(b.IsKubernetesGTE("1.22")),
 		}
 	}
-	c := cc.AWSEBSCSIDriver
+	c := aws.EBSCSIDriver
 
-	if !fi.BoolValue(c.Enabled) {
+	if !fi.ValueOf(c.Enabled) {
 		return nil
 	}
 
 	if c.Version == nil {
-		version := "v1.12.0"
-		c.Version = fi.String(version)
+		version := "v1.14.1"
+		c.Version = &version
 	}
 
 	return nil
