@@ -45,22 +45,22 @@ func (b *ContainerdOptionsBuilder) BuildOptions(o interface{}) error {
 
 	if clusterSpec.ContainerRuntime == "containerd" {
 		// Set version based on Kubernetes version
-		if fi.StringValue(containerd.Version) == "" {
+		if fi.ValueOf(containerd.Version) == "" {
 			if b.IsKubernetesGTE("1.23") {
-				containerd.Version = fi.String("1.6.10")
+				containerd.Version = fi.PtrTo("1.6.18")
 				containerd.Runc = &kops.Runc{
-					Version: fi.String("1.1.4"),
+					Version: fi.PtrTo("1.1.4"),
 				}
 			} else {
-				containerd.Version = fi.String("1.4.13")
+				containerd.Version = fi.PtrTo("1.4.13")
 			}
 		}
 		// Set default log level to INFO
-		containerd.LogLevel = fi.String("info")
+		containerd.LogLevel = fi.PtrTo("info")
 
 	} else if clusterSpec.ContainerRuntime == "docker" {
 		// Docker version should always be available
-		dockerVersion := fi.StringValue(clusterSpec.Docker.Version)
+		dockerVersion := fi.ValueOf(clusterSpec.Docker.Version)
 		if dockerVersion == "" {
 			return fmt.Errorf("docker version is required")
 		} else {
@@ -75,18 +75,18 @@ func (b *ContainerdOptionsBuilder) BuildOptions(o interface{}) error {
 			}
 		}
 		// Set default log level to INFO
-		containerd.LogLevel = fi.String("info")
+		containerd.LogLevel = fi.PtrTo("info")
 		// Build config file for containerd running in Docker mode
 		config, _ := toml.Load("")
 		config.SetPath([]string{"disabled_plugins"}, []string{"cri"})
-		containerd.ConfigOverride = fi.String(config.String())
+		containerd.ConfigOverride = fi.PtrTo(config.String())
 
 	} else {
 		// Unknown container runtime, should not install containerd
 		containerd.SkipInstall = true
 	}
 
-	if containerd.NvidiaGPU != nil && fi.BoolValue(containerd.NvidiaGPU.Enabled) && containerd.NvidiaGPU.DriverPackage == "" {
+	if containerd.NvidiaGPU != nil && fi.ValueOf(containerd.NvidiaGPU.Enabled) && containerd.NvidiaGPU.DriverPackage == "" {
 		containerd.NvidiaGPU.DriverPackage = kops.NvidiaDefaultDriverPackage
 	}
 
