@@ -36,11 +36,11 @@ type KubeApiserverBuilder struct {
 	AssetBuilder *assets.AssetBuilder
 }
 
-var _ fi.ModelBuilder = &KubeApiserverBuilder{}
+var _ fi.CloudupModelBuilder = &KubeApiserverBuilder{}
 
 // Build creates the tasks relating to kube-apiserver
 // Currently we only build the kube-apiserver-healthcheck sidecar
-func (b *KubeApiserverBuilder) Build(c *fi.ModelBuilderContext) error {
+func (b *KubeApiserverBuilder) Build(c *fi.CloudupModelBuilderContext) error {
 	manifest, err := b.buildManifest()
 	if err != nil {
 		return err
@@ -57,14 +57,14 @@ func (b *KubeApiserverBuilder) Build(c *fi.ModelBuilderContext) error {
 	c.AddTask(&fitasks.ManagedFile{
 		Contents:  fi.NewBytesResource(manifestYAML),
 		Lifecycle: b.Lifecycle,
-		Location:  fi.String(location),
-		Name:      fi.String("manifests-static-" + key),
+		Location:  fi.PtrTo(location),
+		Name:      fi.PtrTo("manifests-static-" + key),
 	})
 
 	b.AssetBuilder.StaticManifests = append(b.AssetBuilder.StaticManifests, &assets.StaticManifest{
 		Key:   key,
 		Path:  location,
-		Roles: []kops.InstanceGroupRole{kops.InstanceGroupRoleMaster, kops.InstanceGroupRoleAPIServer},
+		Roles: []kops.InstanceGroupRole{kops.InstanceGroupRoleControlPlane, kops.InstanceGroupRoleAPIServer},
 	})
 	return nil
 }
@@ -79,7 +79,7 @@ kind: Pod
 spec:
   containers:
   - name: healthcheck
-    image: registry.k8s.io/kops/kube-apiserver-healthcheck:1.25.3
+    image: registry.k8s.io/kops/kube-apiserver-healthcheck:1.26.0
     livenessProbe:
       httpGet:
         # The sidecar serves a healthcheck on the same port,
