@@ -33,10 +33,10 @@ type GCEModelContext struct {
 
 // LinkToNetwork returns the GCE Network object the cluster is located in
 func (c *GCEModelContext) LinkToNetwork() (*gcetasks.Network, error) {
-	if c.Cluster.Spec.NetworkID == "" {
+	if c.Cluster.Spec.Networking.NetworkID == "" {
 		return &gcetasks.Network{Name: s(c.SafeTruncatedClusterName())}, nil
 	}
-	name, project, err := gce.ParseNameAndProjectFromNetworkID(c.Cluster.Spec.NetworkID)
+	name, project, err := gce.ParseNameAndProjectFromNetworkID(c.Cluster.Spec.Networking.NetworkID)
 	if err != nil {
 		return nil, err
 	}
@@ -59,7 +59,7 @@ func (c *GCEModelContext) NameForIPAliasRange(key string) string {
 
 // LinkToSubnet returns a link to the GCE subnet object
 func (c *GCEModelContext) LinkToSubnet(subnet *kops.ClusterSubnetSpec) *gcetasks.Subnet {
-	name := subnet.ProviderID
+	name := subnet.ID
 	if name == "" {
 		name = gce.ClusterSuffixedName(subnet.Name, c.Cluster.ObjectMeta.Name, 63)
 	}
@@ -133,22 +133,22 @@ func (c *GCEModelContext) NameForFirewallRule(id string) string {
 }
 
 func (c *GCEModelContext) NetworkingIsIPAlias() bool {
-	return c.Cluster.Spec.Networking != nil && c.Cluster.Spec.Networking.GCE != nil
+	return c.Cluster.Spec.Networking.GCE != nil
 }
 
 func (c *GCEModelContext) NetworkingIsGCERoutes() bool {
-	return c.Cluster.Spec.Networking != nil && c.Cluster.Spec.Networking.Kubenet != nil
+	return c.Cluster.Spec.Networking.Kubenet != nil
 }
 
 // LinkToServiceAccount returns a link to the GCE ServiceAccount object for VMs in the given role
 func (c *GCEModelContext) LinkToServiceAccount(ig *kops.InstanceGroup) *gcetasks.ServiceAccount {
-	if c.Cluster.Spec.CloudConfig.GCEServiceAccount != "" {
+	if c.Cluster.Spec.CloudProvider.GCE.ServiceAccount != "" {
 		// This is a legacy setting because the nodes & control-plane run under the same serviceaccount
-		klog.Warningf("using legacy spec.cloudConfig.gceServiceAccount=%q setting", c.Cluster.Spec.CloudConfig.GCEServiceAccount)
+		klog.Warningf("using legacy spec.cloudProvider.gce.serviceAccount=%q setting", c.Cluster.Spec.CloudProvider.GCE.ServiceAccount)
 		return &gcetasks.ServiceAccount{
 			Name:   s("shared"),
-			Email:  &c.Cluster.Spec.CloudConfig.GCEServiceAccount,
-			Shared: fi.Bool(true),
+			Email:  &c.Cluster.Spec.CloudProvider.GCE.ServiceAccount,
+			Shared: fi.PtrTo(true),
 		}
 	}
 
@@ -156,7 +156,7 @@ func (c *GCEModelContext) LinkToServiceAccount(ig *kops.InstanceGroup) *gcetasks
 
 	name := ""
 	switch role {
-	case kops.InstanceGroupRoleAPIServer, kops.InstanceGroupRoleMaster:
+	case kops.InstanceGroupRoleAPIServer, kops.InstanceGroupRoleControlPlane:
 		name = gce.ControlPlane
 
 	case kops.InstanceGroupRoleBastion:
