@@ -22,7 +22,7 @@ import (
 	"strings"
 
 	"k8s.io/kops/pkg/apis/kops"
-	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
+	"k8s.io/kops/pkg/truncate"
 )
 
 // MaxLengthIAMRoleName defines the max length of an IAMRole name
@@ -52,7 +52,6 @@ func (b *IAMModelContext) IAMNameForServiceAccountRole(role Subject) (string, er
 	if !ok {
 		return "", fmt.Errorf("role %v does not have ServiceAccount", role)
 	}
-
 	name := IAMNameForServiceAccountRole(serviceAccount.Name, serviceAccount.Namespace, b.ClusterName())
 	return name, nil
 }
@@ -61,8 +60,9 @@ func (b *IAMModelContext) IAMNameForServiceAccountRole(role Subject) (string, er
 func (b *IAMModelContext) ClusterName() string {
 	return b.Cluster.ObjectMeta.Name
 }
+
 func IAMNameForServiceAccountRole(name, namespace, clusterName string) string {
 	role := name + "." + strings.ReplaceAll(namespace, "*", "wildcard") + ".sa." + clusterName
-	role = awsup.TruncateString(role, awsup.TruncateStringOptions{MaxLength: MaxLengthIAMRoleName, AlwaysAddHash: false})
+	role = truncate.TruncateString(role, truncate.TruncateStringOptions{MaxLength: MaxLengthIAMRoleName, AlwaysAddHash: false})
 	return role
 }
