@@ -22,7 +22,7 @@ import (
 	"strings"
 
 	"github.com/aws/aws-sdk-go/service/ec2"
-
+	"github.com/scaleway/scaleway-sdk-go/api/instance/v1"
 	"k8s.io/kops/pkg/apis/kops"
 	"k8s.io/kops/pkg/apis/kops/model"
 	"k8s.io/kops/upup/pkg/fi"
@@ -38,6 +38,8 @@ import (
 	"k8s.io/kops/upup/pkg/fi/cloudup/hetznertasks"
 	"k8s.io/kops/upup/pkg/fi/cloudup/openstack"
 	"k8s.io/kops/upup/pkg/fi/cloudup/openstacktasks"
+	"k8s.io/kops/upup/pkg/fi/cloudup/scaleway"
+	"k8s.io/kops/upup/pkg/fi/cloudup/scalewaytasks"
 )
 
 const (
@@ -55,16 +57,16 @@ type MasterVolumeBuilder struct {
 	Lifecycle fi.Lifecycle
 }
 
-var _ fi.ModelBuilder = &MasterVolumeBuilder{}
+var _ fi.CloudupModelBuilder = &MasterVolumeBuilder{}
 
-func (b *MasterVolumeBuilder) Build(c *fi.ModelBuilderContext) error {
+func (b *MasterVolumeBuilder) Build(c *fi.CloudupModelBuilderContext) error {
 	for _, etcd := range b.Cluster.Spec.EtcdClusters {
 		for _, m := range etcd.Members {
-			// EBS volume for each member of the each etcd cluster
+			// EBS volume for each member of each etcd cluster
 			prefix := m.Name + ".etcd-" + etcd.Name
 			name := prefix + "." + b.ClusterName()
 
-			igName := fi.StringValue(m.InstanceGroup)
+			igName := fi.ValueOf(m.InstanceGroup)
 			if igName == "" {
 				return fmt.Errorf("InstanceGroup not set on etcd %s/%s", m.Name, etcd.Name)
 			}
@@ -85,7 +87,7 @@ func (b *MasterVolumeBuilder) Build(c *fi.ModelBuilderContext) error {
 			}
 			zone := zones[0]
 
-			volumeSize := fi.Int32Value(m.VolumeSize)
+			volumeSize := fi.ValueOf(m.VolumeSize)
 			if volumeSize == 0 {
 				volumeSize = DefaultEtcdVolumeSize
 			}
@@ -114,7 +116,12 @@ func (b *MasterVolumeBuilder) Build(c *fi.ModelBuilderContext) error {
 					return err
 				}
 			case kops.CloudProviderAzure:
-				b.addAzureVolume(c, name, volumeSize, zone, etcd, m, allMembers)
+				err = b.addAzureVolume(c, name, volumeSize, zone, etcd, m, allMembers)
+				if err != nil {
+					return err
+				}
+			case kops.CloudProviderScaleway:
+				b.addScalewayVolume(c, name, volumeSize, zone, etcd, m, allMembers)
 			default:
 				return fmt.Errorf("unknown cloudprovider %q", b.Cluster.Spec.GetCloudProvider())
 			}
@@ -123,14 +130,14 @@ func (b *MasterVolumeBuilder) Build(c *fi.ModelBuilderContext) error {
 	return nil
 }
 
-func (b *MasterVolumeBuilder) addAWSVolume(c *fi.ModelBuilderContext, name string, volumeSize int32, zone string, etcd kops.EtcdClusterSpec, m kops.EtcdMemberSpec, allMembers []string) error {
+func (b *MasterVolumeBuilder) addAWSVolume(c *fi.CloudupModelBuilderContext, name string, volumeSize int32, zone string, etcd kops.EtcdClusterSpec, m kops.EtcdMemberSpec, allMembers []string) error {
 	// https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html
-	volumeType := fi.StringValue(m.VolumeType)
+	volumeType := fi.ValueOf(m.VolumeType)
 	if volumeType == "" {
 		volumeType = DefaultAWSEtcdVolumeType
 	}
-	volumeIops := fi.Int32Value(m.VolumeIOPS)
-	volumeThroughput := fi.Int32Value(m.VolumeThroughput)
+	volumeIops := fi.ValueOf(m.VolumeIOPS)
+	volumeThroughput := fi.ValueOf(m.VolumeThroughput)
 	switch volumeType {
 	case ec2.VolumeTypeIo1, ec2.VolumeTypeIo2:
 		if volumeIops < 100 {
@@ -160,31 +167,32 @@ func (b *MasterVolumeBuilder) addAWSVolume(c *fi.ModelBuilderContext, name strin
 	// tags[awsup.TagClusterName] = b.C.cluster.Name
 	// This is the configuration of the etcd cluster
 	tags[awsup.TagNameEtcdClusterPrefix+etcd.Name] = m.Name + "/" + strings.Join(allMembers, ",")
-	// This says "only mount on a master"
+	// This says "only mount on a control plane node"
+	tags[awsup.TagNameRolePrefix+"control-plane"] = "1"
 	tags[awsup.TagNameRolePrefix+"master"] = "1"
 
 	// We always add an owned tags (these can't be shared)
 	tags["kubernetes.io/cluster/"+b.Cluster.ObjectMeta.Name] = "owned"
 
-	encrypted := fi.BoolValue(m.EncryptedVolume)
+	encrypted := fi.ValueOf(m.EncryptedVolume)
 
 	t := &awstasks.EBSVolume{
-		Name:      fi.String(name),
+		Name:      fi.PtrTo(name),
 		Lifecycle: b.Lifecycle,
 
-		AvailabilityZone: fi.String(zone),
-		SizeGB:           fi.Int64(int64(volumeSize)),
-		VolumeType:       fi.String(volumeType),
+		AvailabilityZone: fi.PtrTo(zone),
+		SizeGB:           fi.PtrTo(int64(volumeSize)),
+		VolumeType:       fi.PtrTo(volumeType),
 		KmsKeyId:         m.KmsKeyID,
-		Encrypted:        fi.Bool(encrypted),
+		Encrypted:        fi.PtrTo(encrypted),
 		Tags:             tags,
 	}
 	switch volumeType {
 	case ec2.VolumeTypeGp3:
-		t.VolumeThroughput = fi.Int64(int64(volumeThroughput))
+		t.VolumeThroughput = fi.PtrTo(int64(volumeThroughput))
 		fallthrough
 	case ec2.VolumeTypeIo1, ec2.VolumeTypeIo2:
-		t.VolumeIops = fi.Int64(int64(volumeIops))
+		t.VolumeIops = fi.PtrTo(int64(volumeIops))
 	}
 
 	c.AddTask(t)
@@ -215,7 +223,7 @@ func validateAWSVolume(name, volumeType string, volumeSize, volumeIops, volumeTh
 	return nil
 }
 
-func (b *MasterVolumeBuilder) addDOVolume(c *fi.ModelBuilderContext, name string, volumeSize int32, zone string, etcd kops.EtcdClusterSpec, m kops.EtcdMemberSpec, allMembers []string) {
+func (b *MasterVolumeBuilder) addDOVolume(c *fi.CloudupModelBuilderContext, name string, volumeSize int32, zone string, etcd kops.EtcdClusterSpec, m kops.EtcdMemberSpec, allMembers []string) {
 	// required that names start with a lower case and only contains letters, numbers and hyphens
 	name = "kops-" + do.SafeClusterName(name)
 
@@ -232,18 +240,18 @@ func (b *MasterVolumeBuilder) addDOVolume(c *fi.ModelBuilderContext, name string
 	tags[do.TagKubernetesClusterNamePrefix] = do.SafeClusterName(b.Cluster.ObjectMeta.Name)
 
 	t := &dotasks.Volume{
-		Name:      fi.String(name),
+		Name:      fi.PtrTo(name),
 		Lifecycle: b.Lifecycle,
-		SizeGB:    fi.Int64(int64(volumeSize)),
-		Region:    fi.String(zone),
+		SizeGB:    fi.PtrTo(int64(volumeSize)),
+		Region:    fi.PtrTo(zone),
 		Tags:      tags,
 	}
 
 	c.AddTask(t)
 }
 
-func (b *MasterVolumeBuilder) addGCEVolume(c *fi.ModelBuilderContext, prefix string, volumeSize int32, zone string, etcd kops.EtcdClusterSpec, m kops.EtcdMemberSpec, allMembers []string) {
-	volumeType := fi.StringValue(m.VolumeType)
+func (b *MasterVolumeBuilder) addGCEVolume(c *fi.CloudupModelBuilderContext, prefix string, volumeSize int32, zone string, etcd kops.EtcdClusterSpec, m kops.EtcdMemberSpec, allMembers []string) {
+	volumeType := fi.ValueOf(m.VolumeType)
 	if volumeType == "" {
 		volumeType = DefaultGCEEtcdVolumeType
 	}
@@ -278,26 +286,26 @@ func (b *MasterVolumeBuilder) addGCEVolume(c *fi.ModelBuilderContext, prefix str
 	name := gce.ClusterSuffixedName(prefix, b.Cluster.ObjectMeta.Name, 63)
 
 	t := &gcetasks.Disk{
-		Name:      fi.String(name),
+		Name:      fi.PtrTo(name),
 		Lifecycle: b.Lifecycle,
 
-		Zone:       fi.String(zone),
-		SizeGB:     fi.Int64(int64(volumeSize)),
-		VolumeType: fi.String(volumeType),
+		Zone:       fi.PtrTo(zone),
+		SizeGB:     fi.PtrTo(int64(volumeSize)),
+		VolumeType: fi.PtrTo(volumeType),
 		Labels:     tags,
 	}
 
 	c.AddTask(t)
 }
 
-func (b *MasterVolumeBuilder) addHetznerVolume(c *fi.ModelBuilderContext, name string, volumeSize int32, zone string, etcd kops.EtcdClusterSpec, m kops.EtcdMemberSpec, allMembers []string) {
+func (b *MasterVolumeBuilder) addHetznerVolume(c *fi.CloudupModelBuilderContext, name string, volumeSize int32, zone string, etcd kops.EtcdClusterSpec, m kops.EtcdMemberSpec, allMembers []string) {
 	tags := make(map[string]string)
 	tags[hetzner.TagKubernetesClusterName] = b.Cluster.ObjectMeta.Name
-	tags[hetzner.TagKubernetesInstanceGroup] = fi.StringValue(m.InstanceGroup)
+	tags[hetzner.TagKubernetesInstanceGroup] = fi.ValueOf(m.InstanceGroup)
 	tags[hetzner.TagKubernetesVolumeRole] = etcd.Name
 
 	t := &hetznertasks.Volume{
-		Name:      fi.String(name),
+		Name:      fi.PtrTo(name),
 		Lifecycle: b.Lifecycle,
 		Size:      int(volumeSize),
 		Location:  zone,
@@ -308,8 +316,8 @@ func (b *MasterVolumeBuilder) addHetznerVolume(c *fi.ModelBuilderContext, name s
 	return
 }
 
-func (b *MasterVolumeBuilder) addOpenstackVolume(c *fi.ModelBuilderContext, name string, volumeSize int32, zone string, etcd kops.EtcdClusterSpec, m kops.EtcdMemberSpec, allMembers []string) error {
-	volumeType := fi.StringValue(m.VolumeType)
+func (b *MasterVolumeBuilder) addOpenstackVolume(c *fi.CloudupModelBuilderContext, name string, volumeSize int32, zone string, etcd kops.EtcdClusterSpec, m kops.EtcdMemberSpec, allMembers []string) error {
+	volumeType := fi.ValueOf(m.VolumeType)
 
 	// The tags are how protokube knows to mount the volume and use it for etcd
 	tags := make(map[string]string)
@@ -319,18 +327,19 @@ func (b *MasterVolumeBuilder) addOpenstackVolume(c *fi.ModelBuilderContext, name
 	}
 	// This is the configuration of the etcd cluster
 	tags[openstack.TagNameEtcdClusterPrefix+etcd.Name] = m.Name + "/" + strings.Join(allMembers, ",")
-	// This says "only mount on a master"
+	// This says "only mount on a control plane node"
+	tags[openstack.TagNameRolePrefix+openstack.TagRoleControlPlane] = "1"
 	tags[openstack.TagNameRolePrefix+"master"] = "1"
 
 	// override zone
 	if b.Cluster.Spec.CloudProvider.Openstack.BlockStorage != nil && b.Cluster.Spec.CloudProvider.Openstack.BlockStorage.OverrideAZ != nil {
-		zone = fi.StringValue(b.Cluster.Spec.CloudProvider.Openstack.BlockStorage.OverrideAZ)
+		zone = fi.ValueOf(b.Cluster.Spec.CloudProvider.Openstack.BlockStorage.OverrideAZ)
 	}
 	t := &openstacktasks.Volume{
-		Name:             fi.String(name),
-		AvailabilityZone: fi.String(zone),
-		VolumeType:       fi.String(volumeType),
-		SizeGB:           fi.Int64(int64(volumeSize)),
+		Name:             fi.PtrTo(name),
+		AvailabilityZone: fi.PtrTo(zone),
+		VolumeType:       fi.PtrTo(volumeType),
+		SizeGB:           fi.PtrTo(int64(volumeSize)),
 		Tags:             tags,
 		Lifecycle:        b.Lifecycle,
 	}
@@ -340,41 +349,71 @@ func (b *MasterVolumeBuilder) addOpenstackVolume(c *fi.ModelBuilderContext, name
 }
 
 func (b *MasterVolumeBuilder) addAzureVolume(
-	c *fi.ModelBuilderContext,
+	c *fi.CloudupModelBuilderContext,
 	name string,
 	volumeSize int32,
 	zone string,
 	etcd kops.EtcdClusterSpec,
 	m kops.EtcdMemberSpec,
 	allMembers []string,
-) {
+) error {
 	// The tags are use by Protokube to mount the volume and use it for etcd.
 	tags := map[string]*string{
 		// This is the configuration of the etcd cluster.
-		azure.TagNameEtcdClusterPrefix + etcd.Name: fi.String(m.Name + "/" + strings.Join(allMembers, ",")),
-		// This says "only mount on a master".
-		azure.TagNameRolePrefix + azure.TagRoleMaster: fi.String("1"),
+		azure.TagNameEtcdClusterPrefix + etcd.Name: fi.PtrTo(m.Name + "/" + strings.Join(allMembers, ",")),
+		// This says "only mount on a control plane node".
+		azure.TagNameRolePrefix + azure.TagRoleControlPlane: fi.PtrTo("1"),
+		azure.TagNameRolePrefix + azure.TagRoleMaster:       fi.PtrTo("1"),
 		// We always add an owned tags (these can't be shared).
 		// Use dash (_) as a splitter. Other CSPs use slash (/), but slash is not
 		// allowed as a tag key in Azure.
-		"kubernetes.io_cluster_" + b.Cluster.ObjectMeta.Name: fi.String("owned"),
+		"kubernetes.io_cluster_" + b.Cluster.ObjectMeta.Name: fi.PtrTo("owned"),
 	}
 
 	// Apply all user defined labels on the volumes.
 	for k, v := range b.Cluster.Spec.CloudLabels {
-		tags[k] = fi.String(v)
+		tags[k] = fi.PtrTo(v)
 	}
 
-	// TODO(kenji): Respect zone and m.EncryptedVolume.
+	zoneNumber, err := azure.ZoneToAvailabilityZoneNumber(zone)
+	if err != nil {
+		return err
+	}
+
+	// TODO(kenji): Respect m.EncryptedVolume.
 	t := &azuretasks.Disk{
-		Name:      fi.String(name),
+		Name:      fi.PtrTo(name),
 		Lifecycle: b.Lifecycle,
 		// We cannot use AzureModelContext.LinkToResourceGroup() here because of cyclic dependency.
 		ResourceGroup: &azuretasks.ResourceGroup{
-			Name: fi.String(b.Cluster.AzureResourceGroupName()),
+			Name: fi.PtrTo(b.Cluster.AzureResourceGroupName()),
 		},
-		SizeGB: fi.Int32(volumeSize),
+		SizeGB: fi.PtrTo(volumeSize),
 		Tags:   tags,
+		Zones:  &[]string{zoneNumber},
 	}
 	c.AddTask(t)
+
+	return nil
+}
+
+func (b *MasterVolumeBuilder) addScalewayVolume(c *fi.CloudupModelBuilderContext, name string, volumeSize int32, zone string, etcd kops.EtcdClusterSpec, m kops.EtcdMemberSpec, allMembers []string) {
+	tags := []string{
+		fmt.Sprintf("%s=%s", scaleway.TagClusterName, b.Cluster.ObjectMeta.Name),
+		fmt.Sprintf("%s=%s", scaleway.TagNameEtcdClusterPrefix, etcd.Name),
+		fmt.Sprintf("%s=%s", scaleway.TagNameRolePrefix, scaleway.TagRoleMaster),
+		fmt.Sprintf("%s=%s", scaleway.TagInstanceGroup, fi.ValueOf(m.InstanceGroup)),
+	}
+
+	t := &scalewaytasks.Volume{
+		Name:      fi.PtrTo(name),
+		Lifecycle: b.Lifecycle,
+		Size:      fi.PtrTo(int64(volumeSize) * 1e9),
+		Zone:      &zone,
+		Tags:      tags,
+		Type:      fi.PtrTo(string(instance.VolumeVolumeTypeBSSD)),
+	}
+	c.AddTask(t)
+
+	return
 }
