@@ -24,7 +24,6 @@ import (
 	"k8s.io/kops/upup/pkg/fi/cloudup/openstacktasks"
 
 	"k8s.io/klog/v2"
-	"k8s.io/kops/pkg/dns"
 	"k8s.io/kops/pkg/wellknownports"
 	"k8s.io/utils/net"
 
@@ -47,18 +46,18 @@ type FirewallModelBuilder struct {
 	Rules     map[string]*openstacktasks.SecurityGroupRule
 }
 
-var _ fi.ModelBuilder = &FirewallModelBuilder{}
+var _ fi.CloudupModelBuilder = &FirewallModelBuilder{}
 
 func (b *FirewallModelBuilder) usesOctavia() bool {
 	if b.Cluster.Spec.CloudProvider.Openstack.Loadbalancer != nil {
-		return fi.BoolValue(b.Cluster.Spec.CloudProvider.Openstack.Loadbalancer.UseOctavia)
+		return fi.ValueOf(b.Cluster.Spec.CloudProvider.Openstack.Loadbalancer.UseOctavia)
 	}
 	return false
 }
 
 func (b *FirewallModelBuilder) getOctaviaProvider() string {
 	if b.Cluster.Spec.CloudProvider.Openstack.Loadbalancer != nil {
-		return fi.StringValue(b.Cluster.Spec.CloudProvider.Openstack.Loadbalancer.Provider)
+		return fi.ValueOf(b.Cluster.Spec.CloudProvider.Openstack.Loadbalancer.Provider)
 	}
 	return ""
 }
@@ -68,7 +67,7 @@ func (b *FirewallModelBuilder) getOctaviaProvider() string {
 //	Example
 //	Create an Ingress rule on source allowing traffic from dest with the options in the SecurityGroupRule
 //	Create an Egress rule on source allowing traffic to dest with the options in the SecurityGroupRule
-func (b *FirewallModelBuilder) addDirectionalGroupRule(c *fi.ModelBuilderContext, source, dest *openstacktasks.SecurityGroup, sgr *openstacktasks.SecurityGroupRule) {
+func (b *FirewallModelBuilder) addDirectionalGroupRule(c *fi.CloudupModelBuilderContext, source, dest *openstacktasks.SecurityGroup, sgr *openstacktasks.SecurityGroupRule) {
 	t := &openstacktasks.SecurityGroupRule{
 		Direction:      sgr.Direction,
 		EtherType:      sgr.EtherType,
@@ -79,16 +78,16 @@ func (b *FirewallModelBuilder) addDirectionalGroupRule(c *fi.ModelBuilderContext
 		RemoteGroup:    dest,
 		RemoteIPPrefix: sgr.RemoteIPPrefix,
 		SecGroup:       source,
-		Delete:         fi.Bool(false),
+		Delete:         fi.PtrTo(false),
 	}
 
-	klog.V(8).Infof("Adding rule %v", fi.StringValue(t.GetName()))
-	b.Rules[fi.StringValue(t.GetName())] = t
+	klog.V(8).Infof("Adding rule %v", fi.ValueOf(t.GetName()))
+	b.Rules[fi.ValueOf(t.GetName())] = t
 }
 
 // addSSHRules - sets the ssh rules based on the presence of a bastion
-func (b *FirewallModelBuilder) addSSHRules(c *fi.ModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
-	masterName := b.SecurityGroupName(kops.InstanceGroupRoleMaster)
+func (b *FirewallModelBuilder) addSSHRules(c *fi.CloudupModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
+	masterName := b.SecurityGroupName(kops.InstanceGroupRoleControlPlane)
 	nodeName := b.SecurityGroupName(kops.InstanceGroupRoleNode)
 	bastionName := b.SecurityGroupName(kops.InstanceGroupRoleBastion)
 	masterSG := sgMap[masterName]
@@ -147,11 +146,9 @@ func (b *FirewallModelBuilder) addSSHRules(c *fi.ModelBuilderContext, sgMap map[
 }
 
 // addETCDRules - Add ETCD access rules based on which CNI might need to access __ETCD_ENDPOINTS__
-func (b *FirewallModelBuilder) addETCDRules(c *fi.ModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
-	masterName := b.SecurityGroupName(kops.InstanceGroupRoleMaster)
-	nodeName := b.SecurityGroupName(kops.InstanceGroupRoleNode)
+func (b *FirewallModelBuilder) addETCDRules(c *fi.CloudupModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
+	masterName := b.SecurityGroupName(kops.InstanceGroupRoleControlPlane)
 	masterSG := sgMap[masterName]
-	nodeSG := sgMap[nodeName]
 
 	// ETCD Peer Discovery
 	etcdRule := &openstacktasks.SecurityGroupRule{
@@ -184,26 +181,11 @@ func (b *FirewallModelBuilder) addETCDRules(c *fi.ModelBuilderContext, sgMap map
 		}
 		b.addDirectionalGroupRule(c, masterSG, masterSG, etcdMgmrRule)
 	}
-
-	if b.Cluster.Spec.Networking.Calico != nil {
-
-		etcdCNIRule := &openstacktasks.SecurityGroupRule{
-			Lifecycle:    b.Lifecycle,
-			Direction:    s(string(rules.DirIngress)),
-			Protocol:     s(string(rules.ProtocolTCP)),
-			EtherType:    s(IPV4),
-			PortRangeMin: i(4001),
-			PortRangeMax: i(4001),
-		}
-		// Master access from other masters covered above
-		// Allow nodes to reach ETCD endpoints
-		b.addDirectionalGroupRule(c, masterSG, nodeSG, etcdCNIRule)
-	}
 	return nil
 }
 
 // addNodePortRules - Add node port rules to nodes give the NodePortRange
-func (b *FirewallModelBuilder) addNodePortRules(c *fi.ModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
+func (b *FirewallModelBuilder) addNodePortRules(c *fi.CloudupModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
 	nodeName := b.SecurityGroupName(kops.InstanceGroupRoleNode)
 	nodeSG := sgMap[nodeName]
 
@@ -231,10 +213,10 @@ func (b *FirewallModelBuilder) addNodePortRules(c *fi.ModelBuilderContext, sgMap
 }
 
 // addHTTPSRules - Add rules to 443 access given the presence of a loadbalancer or not
-func (b *FirewallModelBuilder) addHTTPSRules(c *fi.ModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup, useVIPACL bool) error {
-	masterName := b.SecurityGroupName(kops.InstanceGroupRoleMaster)
+func (b *FirewallModelBuilder) addHTTPSRules(c *fi.CloudupModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup, useVIPACL bool) error {
+	masterName := b.SecurityGroupName(kops.InstanceGroupRoleControlPlane)
 	nodeName := b.SecurityGroupName(kops.InstanceGroupRoleNode)
-	lbSGName := b.Cluster.Spec.MasterPublicName
+	lbSGName := b.APIResourceName()
 	lbSG := sgMap[lbSGName]
 	masterSG := sgMap[masterName]
 	nodeSG := sgMap[nodeName]
@@ -255,7 +237,7 @@ func (b *FirewallModelBuilder) addHTTPSRules(c *fi.ModelBuilderContext, sgMap ma
 	if b.UseLoadBalancerForAPI() {
 		if !useVIPACL {
 			// Allow API Access to the lb sg
-			for _, apiAccess := range b.Cluster.Spec.KubernetesAPIAccess {
+			for _, apiAccess := range b.Cluster.Spec.API.Access {
 				etherType := IPV4
 				if !net.IsIPv4CIDRString(apiAccess) {
 					etherType = IPV6
@@ -277,7 +259,7 @@ func (b *FirewallModelBuilder) addHTTPSRules(c *fi.ModelBuilderContext, sgMap ma
 		// FIXME: Octavia port traffic appears to be denied though its port is in lbSG
 		if b.usesOctavia() {
 			if b.getOctaviaProvider() == "ovn" {
-				for _, apiAccess := range b.Cluster.Spec.KubernetesAPIAccess {
+				for _, apiAccess := range b.Cluster.Spec.API.Access {
 					etherType := IPV4
 					if !net.IsIPv4CIDRString(apiAccess) {
 						etherType = IPV6
@@ -300,14 +282,14 @@ func (b *FirewallModelBuilder) addHTTPSRules(c *fi.ModelBuilderContext, sgMap ma
 					EtherType:      s(IPV4),
 					PortRangeMin:   i(443),
 					PortRangeMax:   i(443),
-					RemoteIPPrefix: s(b.Cluster.Spec.NetworkCIDR),
+					RemoteIPPrefix: s(b.Cluster.Spec.Networking.NetworkCIDR),
 				})
 			}
 		}
 
 	} else {
 		// Allow the masters to receive connections from KubernetesAPIAccess
-		for _, apiAccess := range b.Cluster.Spec.KubernetesAPIAccess {
+		for _, apiAccess := range b.Cluster.Spec.API.Access {
 			etherType := IPV4
 			if !net.IsIPv4CIDRString(apiAccess) {
 				etherType = IPV6
@@ -328,9 +310,9 @@ func (b *FirewallModelBuilder) addHTTPSRules(c *fi.ModelBuilderContext, sgMap ma
 }
 
 // addKubeletRules - Add rules to 10250 port
-func (b *FirewallModelBuilder) addKubeletRules(c *fi.ModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
+func (b *FirewallModelBuilder) addKubeletRules(c *fi.CloudupModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
 	// TODO: This is the default port for kubelet and may be overridden
-	masterName := b.SecurityGroupName(kops.InstanceGroupRoleMaster)
+	masterName := b.SecurityGroupName(kops.InstanceGroupRoleControlPlane)
 	nodeName := b.SecurityGroupName(kops.InstanceGroupRoleNode)
 	masterSG := sgMap[masterName]
 	nodeSG := sgMap[nodeName]
@@ -353,8 +335,8 @@ func (b *FirewallModelBuilder) addKubeletRules(c *fi.ModelBuilderContext, sgMap
 }
 
 // addNodeExporterAndOccmRules - Allow 9100 TCP port from nodesg, allow 10258 from nodes to master - expose occm metrics
-func (b *FirewallModelBuilder) addNodeExporterAndOccmRules(c *fi.ModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
-	masterName := b.SecurityGroupName(kops.InstanceGroupRoleMaster)
+func (b *FirewallModelBuilder) addNodeExporterAndOccmRules(c *fi.CloudupModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
+	masterName := b.SecurityGroupName(kops.InstanceGroupRoleControlPlane)
 	nodeName := b.SecurityGroupName(kops.InstanceGroupRoleNode)
 	masterSG := sgMap[masterName]
 	nodeSG := sgMap[nodeName]
@@ -383,8 +365,8 @@ func (b *FirewallModelBuilder) addNodeExporterAndOccmRules(c *fi.ModelBuilderCon
 }
 
 // addDNSRules - Add DNS rules for internal DNS queries
-func (b *FirewallModelBuilder) addDNSRules(c *fi.ModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
-	masterName := b.SecurityGroupName(kops.InstanceGroupRoleMaster)
+func (b *FirewallModelBuilder) addDNSRules(c *fi.CloudupModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
+	masterName := b.SecurityGroupName(kops.InstanceGroupRoleControlPlane)
 	nodeName := b.SecurityGroupName(kops.InstanceGroupRoleNode)
 	masterSG := sgMap[masterName]
 	nodeSG := sgMap[nodeName]
@@ -405,54 +387,49 @@ func (b *FirewallModelBuilder) addDNSRules(c *fi.ModelBuilderContext, sgMap map[
 }
 
 // addCNIRules - Add ports required for different CNI implementations
-func (b *FirewallModelBuilder) addCNIRules(c *fi.ModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
+func (b *FirewallModelBuilder) addCNIRules(c *fi.CloudupModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
 	udpPorts := []int{}
 	tcpPorts := []int{}
 	protocols := []string{}
 
-	// allow cadvisor
-	tcpPorts = append(tcpPorts, 4194)
+	if b.Cluster.Spec.Networking.Kopeio != nil {
+		// VXLAN over UDP
+		// https://tools.ietf.org/html/rfc7348
+		udpPorts = append(udpPorts, 4789)
+	}
 
-	if b.Cluster.Spec.Networking != nil {
-		if b.Cluster.Spec.Networking.Kopeio != nil {
-			// VXLAN over UDP
-			// https://tools.ietf.org/html/rfc7348
-			udpPorts = append(udpPorts, 4789)
-		}
+	if b.Cluster.Spec.Networking.Cilium != nil {
+		udpPorts = append(udpPorts, 8472)
+		tcpPorts = append(tcpPorts, 4240)
+	}
 
-		if b.Cluster.Spec.Networking.Cilium != nil {
+	if b.Cluster.Spec.Networking.Weave != nil {
+		udpPorts = append(udpPorts, 6783)
+		tcpPorts = append(tcpPorts, 6783)
+		udpPorts = append(udpPorts, 6784)
+	}
+
+	if b.Cluster.Spec.Networking.Flannel != nil {
+		switch b.Cluster.Spec.Networking.Flannel.Backend {
+		case "", "udp":
+			udpPorts = append(udpPorts, 8285)
+		case "vxlan":
 			udpPorts = append(udpPorts, 8472)
-			tcpPorts = append(tcpPorts, 4240)
-		}
-
-		if b.Cluster.Spec.Networking.Weave != nil {
-			udpPorts = append(udpPorts, 6783)
-			tcpPorts = append(tcpPorts, 6783)
-			udpPorts = append(udpPorts, 6784)
-		}
-
-		if b.Cluster.Spec.Networking.Flannel != nil {
-			switch b.Cluster.Spec.Networking.Flannel.Backend {
-			case "", "udp":
-				udpPorts = append(udpPorts, 8285)
-			case "vxlan":
-				udpPorts = append(udpPorts, 8472)
-			default:
-				klog.Warningf("unknown flannel networking backend %q", b.Cluster.Spec.Networking.Flannel.Backend)
-			}
-		}
-
-		if b.Cluster.Spec.Networking.Calico != nil {
-			tcpPorts = append(tcpPorts, 179)
-			protocols = append(protocols, ProtocolIPEncap)
-		}
-
-		if b.Cluster.Spec.Networking.Kuberouter != nil {
-			protocols = append(protocols, ProtocolIPEncap)
+		default:
+			klog.Warningf("unknown flannel networking backend %q", b.Cluster.Spec.Networking.Flannel.Backend)
 		}
 	}
 
-	masterName := b.SecurityGroupName(kops.InstanceGroupRoleMaster)
+	if b.Cluster.Spec.Networking.Calico != nil {
+		tcpPorts = append(tcpPorts, 179)
+		protocols = append(protocols, ProtocolIPEncap)
+	}
+
+	if b.Cluster.Spec.Networking.KubeRouter != nil {
+		protocols = append(protocols, ProtocolIPEncap)
+	}
+
+	masterName := b.SecurityGroupName(kops.InstanceGroupRoleControlPlane)
 	nodeName := b.SecurityGroupName(kops.InstanceGroupRoleNode)
 	masterSG := sgMap[masterName]
 	nodeSG := sgMap[nodeName]
@@ -500,10 +477,28 @@ func (b *FirewallModelBuilder) addCNIRules(c *fi.ModelBuilderContext, sgMap map[
 	return nil
 }
 
+// addKopsControllerRules - Add rules for kops-controller for node bootstrap
+func (b *FirewallModelBuilder) addKopsControllerRules(c *fi.CloudupModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
+	masterName := b.SecurityGroupName(kops.InstanceGroupRoleControlPlane)
+	nodeName := b.SecurityGroupName(kops.InstanceGroupRoleNode)
+	masterSG := sgMap[masterName]
+	nodeSG := sgMap[nodeName]
+	kopsControllerRule := &openstacktasks.SecurityGroupRule{
+		Lifecycle:    b.Lifecycle,
+		Direction:    s(string(rules.DirIngress)),
+		Protocol:     s(string(rules.ProtocolTCP)),
+		EtherType:    s(string(rules.EtherType4)),
+		PortRangeMin: i(wellknownports.KopsControllerPort),
+		PortRangeMax: i(wellknownports.KopsControllerPort),
+	}
+	b.addDirectionalGroupRule(c, masterSG, nodeSG, kopsControllerRule)
+	return nil
+}
+
 // addProtokubeRules - Add rules for protokube if gossip DNS is enabled
-func (b *FirewallModelBuilder) addProtokubeRules(c *fi.ModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
-	if dns.IsGossipHostname(b.ClusterName()) {
-		masterName := b.SecurityGroupName(kops.InstanceGroupRoleMaster)
+func (b *FirewallModelBuilder) addProtokubeRules(c *fi.CloudupModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup) error {
+	if b.Cluster.IsGossip() {
+		masterName := b.SecurityGroupName(kops.InstanceGroupRoleControlPlane)
 		nodeName := b.SecurityGroupName(kops.InstanceGroupRoleNode)
 		masterSG := sgMap[masterName]
 		nodeSG := sgMap[nodeName]
@@ -546,7 +541,7 @@ func (b *FirewallModelBuilder) getExistingRules(sgMap map[string]*openstacktasks
 			return fmt.Errorf("Found multiple security groups with the same name: %v", sgName)
 		}
 		sg := sgs[0]
-		sgt.Name = fi.String(sg.Name)
+		sgt.Name = fi.PtrTo(sg.Name)
 		sgIdMap[sg.ID] = sgt
 	}
 
@@ -561,28 +556,28 @@ func (b *FirewallModelBuilder) getExistingRules(sgMap map[string]*openstacktasks
 		for _, rule := range sgRules {
 
 			t := &openstacktasks.SecurityGroupRule{
-				ID:             fi.String(rule.ID),
-				Direction:      fi.String(rule.Direction),
-				EtherType:      fi.String(rule.EtherType),
-				PortRangeMax:   fi.Int(rule.PortRangeMax),
-				PortRangeMin:   fi.Int(rule.PortRangeMin),
-				Protocol:       fi.String(rule.Protocol),
-				RemoteIPPrefix: fi.String(rule.RemoteIPPrefix),
+				ID:             fi.PtrTo(rule.ID),
+				Direction:      fi.PtrTo(rule.Direction),
+				EtherType:      fi.PtrTo(rule.EtherType),
+				PortRangeMax:   fi.PtrTo(rule.PortRangeMax),
+				PortRangeMin:   fi.PtrTo(rule.PortRangeMin),
+				Protocol:       fi.PtrTo(rule.Protocol),
+				RemoteIPPrefix: fi.PtrTo(rule.RemoteIPPrefix),
 				RemoteGroup:    sgIdMap[rule.RemoteGroupID],
 				Lifecycle:      b.Lifecycle,
 				SecGroup:       sgIdMap[rule.SecGroupID],
-				Delete:         fi.Bool(true),
+				Delete:         fi.PtrTo(true),
 			}
 			klog.V(8).Infof("Adding existing rule %v", t)
-			b.Rules[fi.StringValue(t.GetName())] = t
+			b.Rules[fi.ValueOf(t.GetName())] = t
 		}
 	}
 	return nil
 }
 
-func (b *FirewallModelBuilder) addDefaultEgress(c *fi.ModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup, useVIPACL bool) {
+func (b *FirewallModelBuilder) addDefaultEgress(c *fi.CloudupModelBuilderContext, sgMap map[string]*openstacktasks.SecurityGroup, useVIPACL bool) {
 	for name, sg := range sgMap {
-		if useVIPACL && name == b.Cluster.Spec.MasterPublicName {
+		if useVIPACL && name == b.APIResourceName() {
 			continue
 		}
 		t := &openstacktasks.SecurityGroupRule{
@@ -606,8 +601,8 @@ func (b *FirewallModelBuilder) addDefaultEgress(c *fi.ModelBuilderContext, sgMap
 }
 
 // Build - schedule security groups and security group rule tasks for Openstack
-func (b *FirewallModelBuilder) Build(c *fi.ModelBuilderContext) error {
-	roles := []kops.InstanceGroupRole{kops.InstanceGroupRoleMaster, kops.InstanceGroupRoleNode}
+func (b *FirewallModelBuilder) Build(c *fi.CloudupModelBuilderContext) error {
+	roles := []kops.InstanceGroupRole{kops.InstanceGroupRoleControlPlane, kops.InstanceGroupRoleNode}
 	if b.UsesSSHBastion() {
 		roles = append(roles, kops.InstanceGroupRoleBastion)
 	}
@@ -619,7 +614,7 @@ func (b *FirewallModelBuilder) Build(c *fi.ModelBuilderContext) error {
 		useVIPACL = true
 	}
 	sg := &openstacktasks.SecurityGroup{
-		Name:             s(b.Cluster.Spec.MasterPublicName),
+		Name:             s(b.APIResourceName()),
 		Lifecycle:        b.Lifecycle,
 		RemoveExtraRules: []string{"port=443"},
 	}
@@ -627,7 +622,7 @@ func (b *FirewallModelBuilder) Build(c *fi.ModelBuilderContext) error {
 		sg.RemoveGroup = true
 	}
 	c.AddTask(sg)
-	sgMap[b.Cluster.Spec.MasterPublicName] = sg
+	sgMap[b.APIResourceName()] = sg
 	for _, role := range roles {
 
 		// Create Security Group for Role
@@ -641,7 +636,7 @@ func (b *FirewallModelBuilder) Build(c *fi.ModelBuilderContext) error {
 			sg.RemoveExtraRules = []string{"port=22"}
 		} else if role == kops.InstanceGroupRoleNode {
 			sg.RemoveExtraRules = []string{"port=22", "port=10250"}
-		} else if role == kops.InstanceGroupRoleMaster {
+		} else if role == kops.InstanceGroupRoleControlPlane {
 			sg.RemoveExtraRules = []string{"port=22", "port=443", "port=10250"}
 		}
 		c.AddTask(sg)
@@ -671,6 +666,8 @@ func (b *FirewallModelBuilder) Build(c *fi.ModelBuilderContext) error {
 	b.addNodeExporterAndOccmRules(c, sgMap)
 	// Protokube Rules
 	b.addProtokubeRules(c, sgMap)
+	// Kops-controller Rules
+	b.addKopsControllerRules(c, sgMap)
 	// Allow necessary local traffic
 	b.addCNIRules(c, sgMap)
 	// ETCD Leader Election
