@@ -25,7 +25,6 @@ import (
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/aws/awserr"
 	"github.com/aws/aws-sdk-go/service/autoscaling"
-	"github.com/aws/aws-sdk-go/service/cloudformation"
 	"github.com/aws/aws-sdk-go/service/ec2"
 	"github.com/aws/aws-sdk-go/service/elb"
 	"github.com/aws/aws-sdk-go/service/elbv2"
@@ -51,14 +50,15 @@ const (
 
 type listFn func(fi.Cloud, string) ([]*resources.Resource, error)
 
-func ListResourcesAWS(cloud awsup.AWSCloud, clusterName string) (map[string]*resources.Resource, error) {
+func ListResourcesAWS(cloud awsup.AWSCloud, clusterInfo resources.ClusterInfo) (map[string]*resources.Resource, error) {
+	clusterName := clusterInfo.Name
+	clusterUsesNoneDNS := clusterInfo.UsesNoneDNS
+
 	resourceTrackers := make(map[string]*resources.Resource)
 
 	// These are the functions that are used for looking up
 	// cluster resources by their tags.
 	listFunctions := []listFn{
-		// CloudFormation
-		// ListCloudFormationStacks,
 		// EC2
 		ListAutoScalingGroups,
 		ListInstances,
@@ -78,8 +78,6 @@ func ListResourcesAWS(cloud awsup.AWSCloud, clusterName string) (map[string]*res
 		ListELBV2s,
 		ListTargetGroups,
 
-		// Route 53
-		ListRoute53Records,
 		// IAM
 		ListIAMInstanceProfiles,
 		ListIAMRoles,
@@ -91,6 +89,11 @@ func ListResourcesAWS(cloud awsup.AWSCloud, clusterName string) (map[string]*res
 		ListEventBridgeRules,
 	}
 
+	if !dns.IsGossipClusterName(clusterName) && !clusterUsesNoneDNS {
+		// Route 53
+		listFunctions = append(listFunctions, ListRoute53Records)
+	}
+
 	if featureflag.Spotinst.Enabled() {
 		// Spotinst resources
 		listFunctions = append(listFunctions, ListSpotinstResources)
@@ -324,56 +327,6 @@ func DeleteInstance(cloud fi.Cloud, t *resources.Resource) error {
 	return nil
 }
 
-func DeleteCloudFormationStack(cloud fi.Cloud, t *resources.Resource) error {
-	c := cloud.(awsup.AWSCloud)
-
-	id := t.ID
-	klog.V(2).Infof("deleting CloudFormation stack %q %q", t.Name, id)
-
-	request := &cloudformation.DeleteStackInput{}
-	request.StackName = &t.Name
-
-	_, err := c.CloudFormation().DeleteStack(request)
-	if err != nil {
-		return fmt.Errorf("error deleting CloudFormation stack %q: %v", id, err)
-	}
-	return nil
-}
-
-func DumpCloudFormationStack(op *resources.DumpOperation, r *resources.Resource) error {
-	data := make(map[string]interface{})
-	data["id"] = r.ID
-	data["type"] = r.Type
-	data["raw"] = r.Obj
-	op.Dump.Resources = append(op.Dump.Resources, data)
-	return nil
-}
-
-func ListCloudFormationStacks(cloud fi.Cloud, clusterName string) ([]*resources.Resource, error) {
-	var resourceTrackers []*resources.Resource
-	request := &cloudformation.ListStacksInput{}
-	c := cloud.(awsup.AWSCloud)
-	response, err := c.CloudFormation().ListStacks(request)
-	if err != nil {
-		return nil, fmt.Errorf("unable to list CloudFormation stacks: %v", err)
-	}
-	for _, stack := range response.StackSummaries {
-		if *stack.StackName == clusterName {
-			resourceTracker := &resources.Resource{
-				Name:    *stack.StackName,
-				ID:      *stack.StackId,
-				Type:    "cloud-formation",
-				Deleter: DeleteCloudFormationStack,
-				Dumper:  DumpCloudFormationStack,
-				Obj:     stack,
-			}
-			resourceTrackers = append(resourceTrackers, resourceTracker)
-		}
-	}
-
-	return resourceTrackers, nil
-}
-
 func ListInstances(cloud fi.Cloud, clusterName string) ([]*resources.Resource, error) {
 	c := cloud.(awsup.AWSCloud)
 
@@ -497,13 +450,11 @@ func guessSSHUser(image *ec2.Image) string {
 	switch owner {
 	case awsup.WellKnownAccountAmazonLinux2, awsup.WellKnownAccountRedhat:
 		return "ec2-user"
-	case awsup.WellKnownAccountCentOS:
-		return "centos"
-	case awsup.WellKnownAccountDebian, awsup.WellKnownAccountDebian9, awsup.WellKnownAccountKopeio:
+	case awsup.WellKnownAccountDebian:
 		return "admin"
 	case awsup.WellKnownAccountUbuntu:
 		return "ubuntu"
-	case awsup.WellKnownAccountCoreOS, awsup.WellKnownAccountFlatcar:
+	case awsup.WellKnownAccountFlatcar:
 		return "core"
 	}
 
@@ -536,13 +487,28 @@ func DumpInstance(op *resources.DumpOperation, r *resources.Resource) error {
 			}
 		}
 	}
+	if len(i.PublicAddresses) == 0 {
+		if ec2Instance.Ipv6Address != nil {
+			i.PrivateAddresses = append(i.PrivateAddresses, *ec2Instance.Ipv6Address)
+		} else if ec2Instance.PrivateIpAddress != nil {
+			i.PrivateAddresses = append(i.PrivateAddresses, *ec2Instance.PrivateIpAddress)
+		}
+	}
+	isControlPlane := false
 	for _, tag := range ec2Instance.Tags {
 		key := aws.StringValue(tag.Key)
 		if !strings.HasPrefix(key, awsup.TagNameRolePrefix) {
 			continue
 		}
 		role := strings.TrimPrefix(key, awsup.TagNameRolePrefix)
-		i.Roles = append(i.Roles, role)
+		if role == "master" || role == "control-plane" {
+			isControlPlane = true
+		} else {
+			i.Roles = append(i.Roles, role)
+		}
+	}
+	if isControlPlane {
+		i.Roles = append(i.Roles, "control-plane")
 	}
 
 	imageID := aws.StringValue(ec2Instance.ImageId)
@@ -603,7 +569,7 @@ func ListVolumes(cloud fi.Cloud, clusterName string) ([]*resources.Resource, err
 		}
 
 		var blocks []string
-		// blocks = append(blocks, "vpc:" + aws.StringValue(rt.VpcId))
+		// blocks = append(blocks, "vpc:" + aws.ValueOf(rt.VpcId))
 
 		resourceTracker.Blocks = blocks
 
@@ -1393,7 +1359,7 @@ func DeleteAutoScalingGroupLaunchTemplate(cloud fi.Cloud, r *resources.Resource)
 	klog.V(2).Infof("Deleting EC2 LaunchTemplate %q", r.ID)
 
 	if _, err := c.EC2().DeleteLaunchTemplate(&ec2.DeleteLaunchTemplateInput{
-		LaunchTemplateId: fi.String(r.ID),
+		LaunchTemplateId: fi.PtrTo(r.ID),
 	}); err != nil {
 		return fmt.Errorf("error deleting ec2 LaunchTemplate %q: %v", r.ID, err)
 	}
@@ -1808,10 +1774,6 @@ func deleteRoute53Records(cloud fi.Cloud, zone *route53.HostedZone, resourceTrac
 func ListRoute53Records(cloud fi.Cloud, clusterName string) ([]*resources.Resource, error) {
 	var resourceTrackers []*resources.Resource
 
-	if dns.IsGossipHostname(clusterName) {
-		return resourceTrackers, nil
-	}
-
 	c := cloud.(awsup.AWSCloud)
 
 	// Normalize cluster name, with leading "."
@@ -2009,7 +1971,7 @@ func ListIAMRoles(cloud fi.Cloud, clusterName string) ([]*resources.Resource, er
 
 							continue
 						} else if awsup.AWSErrorCode(err) == iam.ErrCodeNoSuchEntityException {
-							klog.Warningf("could not find instance profile %q. Resource may already have been deleted: %v", name, awserror)
+							klog.Warningf("could not find role %q. Resource may already have been deleted: %v", name, awserror)
 							continue
 						}
 					}
@@ -2017,7 +1979,7 @@ func ListIAMRoles(cloud fi.Cloud, clusterName string) ([]*resources.Resource, er
 					return false
 				}
 				for _, tag := range roleOutput.Role.Tags {
-					if fi.StringValue(tag.Key) == ownershipTag && fi.StringValue(tag.Value) == "owned" {
+					if fi.ValueOf(tag.Key) == ownershipTag && fi.ValueOf(tag.Value) == "owned" {
 						resourceTracker := &resources.Resource{
 							Name:    name,
 							ID:      name,
@@ -2102,7 +2064,7 @@ func ListIAMInstanceProfiles(cloud fi.Cloud, clusterName string) ([]*resources.R
 				return false
 			}
 			for _, tag := range profileOutput.InstanceProfile.Tags {
-				if fi.StringValue(tag.Key) == ownershipTag && fi.StringValue(tag.Value) == "owned" {
+				if fi.ValueOf(tag.Key) == ownershipTag && fi.ValueOf(tag.Value) == "owned" {
 					profiles = append(profiles, p)
 				}
 			}
@@ -2181,7 +2143,7 @@ func ListIAMOIDCProviders(cloud fi.Cloud, clusterName string) ([]*resources.Reso
 
 func DeleteIAMOIDCProvider(cloud fi.Cloud, r *resources.Resource) error {
 	c := cloud.(awsup.AWSCloud)
-	arn := fi.String(r.ID)
+	arn := fi.PtrTo(r.ID)
 	{
 		klog.V(2).Infof("Deleting IAM OIDC Provider %v", arn)
 		request := &iam.DeleteOpenIDConnectProviderInput{
