@@ -42,7 +42,6 @@ import (
 	"k8s.io/kops/pkg/apis/nodeup"
 	"k8s.io/kops/pkg/assets"
 	"k8s.io/kops/pkg/client/simple"
-	"k8s.io/kops/pkg/dns"
 	"k8s.io/kops/pkg/featureflag"
 	"k8s.io/kops/pkg/kubemanifest"
 	"k8s.io/kops/pkg/model"
@@ -57,6 +56,7 @@ import (
 	"k8s.io/kops/pkg/model/hetznermodel"
 	"k8s.io/kops/pkg/model/iam"
 	"k8s.io/kops/pkg/model/openstackmodel"
+	"k8s.io/kops/pkg/model/scalewaymodel"
 	"k8s.io/kops/pkg/templates"
 	"k8s.io/kops/pkg/wellknownports"
 	"k8s.io/kops/upup/models"
@@ -64,11 +64,11 @@ import (
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
 	"k8s.io/kops/upup/pkg/fi/cloudup/azure"
 	"k8s.io/kops/upup/pkg/fi/cloudup/bootstrapchannelbuilder"
-	"k8s.io/kops/upup/pkg/fi/cloudup/cloudformation"
 	"k8s.io/kops/upup/pkg/fi/cloudup/do"
 	"k8s.io/kops/upup/pkg/fi/cloudup/gce"
 	"k8s.io/kops/upup/pkg/fi/cloudup/hetzner"
 	"k8s.io/kops/upup/pkg/fi/cloudup/openstack"
+	"k8s.io/kops/upup/pkg/fi/cloudup/scaleway"
 	"k8s.io/kops/upup/pkg/fi/cloudup/terraform"
 	"k8s.io/kops/upup/pkg/fi/cloudup/terraformWriter"
 	"k8s.io/kops/util/pkg/architectures"
@@ -82,9 +82,9 @@ const (
 	starline = "*********************************************************************************"
 
 	// OldestSupportedKubernetesVersion is the oldest kubernetes version that is supported in kOps.
-	OldestSupportedKubernetesVersion = "1.20.0"
+	OldestSupportedKubernetesVersion = "1.21.0"
 	// OldestRecommendedKubernetesVersion is the oldest kubernetes version that is not deprecated in kOps.
-	OldestRecommendedKubernetesVersion = "1.22.0"
+	OldestRecommendedKubernetesVersion = "1.23.0"
 )
 
 // TerraformCloudProviders is the list of cloud providers with terraform target support
@@ -107,7 +107,7 @@ type ApplyClusterCmd struct {
 	TargetName string
 
 	// Target is the fi.Target we will operate against
-	Target fi.Target
+	Target fi.CloudupTarget
 
 	// OutDir is a local directory in which we place output, can cache files etc
 	OutDir string
@@ -144,7 +144,7 @@ type ApplyClusterCmd struct {
 	GetAssets bool
 
 	// TaskMap is the map of tasks that we built (output)
-	TaskMap map[string]fi.Task
+	TaskMap map[string]fi.CloudupTask
 
 	// ImageAssets are the image assets we use (output).
 	ImageAssets []*assets.ImageAsset
@@ -246,7 +246,7 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 	}
 
 	assetBuilder := assets.NewAssetBuilder(c.Cluster, c.GetAssets)
-	err = c.upgradeSpecs(assetBuilder)
+	err = c.upgradeSpecs(ctx, assetBuilder)
 	if err != nil {
 		return err
 	}
@@ -302,7 +302,7 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 	if cluster.Spec.KubernetesVersion == "" {
 		return fmt.Errorf("KubernetesVersion not set")
 	}
-	if cluster.Spec.DNSZone == "" && !dns.IsGossipHostname(cluster.ObjectMeta.Name) {
+	if cluster.Spec.DNSZone == "" && !cluster.IsGossip() && !cluster.UsesNoneDNS() {
 		return fmt.Errorf("DNSZone not set")
 	}
 
@@ -363,7 +363,7 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 	}
 
 	encryptionConfigSecretHash := ""
-	if fi.BoolValue(c.Cluster.Spec.EncryptionConfig) {
+	if fi.ValueOf(c.Cluster.Spec.EncryptionConfig) {
 		secret, err := secretStore.FindSecret("encryptionconfig")
 		if err != nil {
 			return fmt.Errorf("could not load encryptionconfig secret: %v", err)
@@ -396,8 +396,6 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 		return err
 	}
 
-	checkExisting := true
-
 	project := ""
 
 	var sshPublicKeys [][]byte
@@ -476,6 +474,22 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 				return fmt.Errorf("exactly one 'admin' SSH public key can be specified when running with Openstack; please delete a key using `kops delete secret`")
 			}
 		}
+
+	case kops.CloudProviderScaleway:
+		{
+			if !featureflag.Scaleway.Enabled() {
+				return fmt.Errorf("Scaleway support is currently alpha, and is feature-gated.  export KOPS_FEATURE_FLAGS=Scaleway")
+			}
+
+			if len(sshPublicKeys) == 0 {
+				return fmt.Errorf("SSH public key must be specified when running with Scaleway (create with `kops create secret --name %s sshpublickey admin -i ~/.ssh/id_rsa.pub`)", cluster.ObjectMeta.Name)
+			}
+
+			if len(sshPublicKeys) != 1 {
+				return fmt.Errorf("exactly one 'admin' SSH public key can be specified when running with Scaleway; please delete a key using `kops delete secret`")
+			}
+		}
+
 	default:
 		return fmt.Errorf("unknown CloudProvider %q", cluster.Spec.GetCloudProvider())
 	}
@@ -483,9 +497,7 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 	modelContext.SSHPublicKeys = sshPublicKeys
 	modelContext.Region = cloud.Region()
 
-	if dns.IsGossipHostname(cluster.ObjectMeta.Name) {
-		klog.V(2).Infof("Gossip DNS: skipping DNS validation")
-	} else {
+	if !cluster.IsGossip() && !cluster.UsesNoneDNS() {
 		err = validateDNS(cluster, cloud)
 		if err != nil {
 			return err
@@ -600,8 +612,8 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 				l.Builders = append(l.Builders, awsModelBuilder)
 			}
 
-			nth := c.Cluster.Spec.NodeTerminationHandler
-			if nth != nil && fi.BoolValue(nth.Enabled) && fi.BoolValue(nth.EnableSQSTerminationDraining) {
+			nth := c.Cluster.Spec.CloudProvider.AWS.NodeTerminationHandler
+			if nth.IsQueueMode() {
 				l.Builders = append(l.Builders, &awsmodel.NodeTerminationHandlerBuilder{
 					AWSModelContext: awsModelContext,
 					Lifecycle:       clusterLifecycle,
@@ -672,16 +684,25 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 				&openstackmodel.ServerGroupModelBuilder{OpenstackModelContext: openstackModelContext, BootstrapScriptBuilder: bootstrapScriptBuilder, Lifecycle: clusterLifecycle},
 			)
 
+		case kops.CloudProviderScaleway:
+			scwModelContext := &scalewaymodel.ScwModelContext{
+				KopsModelContext: modelContext,
+			}
+			l.Builders = append(l.Builders,
+				&scalewaymodel.InstanceModelBuilder{ScwModelContext: scwModelContext, BootstrapScriptBuilder: bootstrapScriptBuilder, Lifecycle: clusterLifecycle},
+				&scalewaymodel.SSHKeyModelBuilder{ScwModelContext: scwModelContext, Lifecycle: securityLifecycle},
+			)
+
 		default:
 			return fmt.Errorf("unknown cloudprovider %q", cluster.Spec.GetCloudProvider())
 		}
 	}
-	c.TaskMap, err = l.BuildTasks(c.LifecycleOverrides)
+	c.TaskMap, err = l.BuildTasks(ctx, c.LifecycleOverrides)
 	if err != nil {
 		return fmt.Errorf("error building tasks: %v", err)
 	}
 
-	var target fi.Target
+	var target fi.CloudupTarget
 	shouldPrecreateDNS := true
 
 	switch c.TargetName {
@@ -699,12 +720,13 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 			target = openstack.NewOpenstackAPITarget(cloud.(openstack.OpenstackCloud))
 		case kops.CloudProviderAzure:
 			target = azure.NewAzureAPITarget(cloud.(azure.AzureCloud))
+		case kops.CloudProviderScaleway:
+			target = scaleway.NewScwAPITarget(cloud.(scaleway.ScwCloud))
 		default:
 			return fmt.Errorf("direct configuration not supported with CloudProvider:%q", cluster.Spec.GetCloudProvider())
 		}
 
 	case TargetTerraform:
-		checkExisting = false
 		outDir := c.OutDir
 		var vfsProvider *vfs.TerraformProvider
 		if tfPath, ok := configBase.(vfs.TerraformPath); ok && featureflag.TerraformManagedFiles.Enabled() {
@@ -736,28 +758,12 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 		// Can cause conflicts with terraform management
 		shouldPrecreateDNS = false
 
-	case TargetCloudformation:
-		checkExisting = false
-		outDir := c.OutDir
-		target = cloudformation.NewCloudformationTarget(cloud, project, outDir)
-
-		// Can cause conflicts with cloudformation management
-		shouldPrecreateDNS = false
-
-		fmt.Printf("\n")
-		fmt.Printf("%s\n", starline)
-		fmt.Printf("\n")
-		fmt.Printf("Kops support for CloudFormation is deprecated and will be removed in a future release.\n")
-		fmt.Printf("\n")
-		fmt.Printf("%s\n", starline)
-		fmt.Printf("\n")
-
 	case TargetDryRun:
 		var out io.Writer = os.Stdout
 		if c.GetAssets {
 			out = io.Discard
 		}
-		target = fi.NewDryRunTarget(assetBuilder, out)
+		target = fi.NewCloudupDryRunTarget(assetBuilder, out)
 
 		// Avoid making changes on a dry-run
 		shouldPrecreateDNS = false
@@ -767,18 +773,17 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 	}
 	c.Target = target
 
-	if checkExisting {
+	if target.DefaultCheckExisting() {
 		c.TaskMap, err = l.FindDeletions(cloud, c.LifecycleOverrides)
 		if err != nil {
 			return fmt.Errorf("error finding deletions: %w", err)
 		}
 	}
 
-	context, err := fi.NewContext(target, cluster, cloud, keyStore, secretStore, configBase, checkExisting, c.TaskMap)
+	context, err := fi.NewCloudupContext(ctx, target, cluster, cloud, keyStore, secretStore, configBase, c.TaskMap)
 	if err != nil {
 		return fmt.Errorf("error building context: %v", err)
 	}
-	defer context.Close()
 
 	var options fi.RunTasksOptions
 	if c.RunTasksOptions != nil {
@@ -792,7 +797,7 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 		return fmt.Errorf("error running tasks: %v", err)
 	}
 
-	if dns.IsGossipHostname(cluster.Name) {
+	if cluster.IsGossip() || cluster.UsesNoneDNS() {
 		shouldPrecreateDNS = false
 	}
 
@@ -814,8 +819,8 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 }
 
 // upgradeSpecs ensures that fields are fully populated / defaulted
-func (c *ApplyClusterCmd) upgradeSpecs(assetBuilder *assets.AssetBuilder) error {
-	fullCluster, err := PopulateClusterSpec(c.Clientset, c.Cluster, c.Cloud, assetBuilder)
+func (c *ApplyClusterCmd) upgradeSpecs(ctx context.Context, assetBuilder *assets.AssetBuilder) error {
+	fullCluster, err := PopulateClusterSpec(ctx, c.Clientset, c.Cluster, c.Cloud, assetBuilder)
 	if err != nil {
 		return err
 	}
@@ -1180,7 +1185,7 @@ func newNodeUpConfigBuilder(cluster *kops.Cluster, assetBuilder *assets.AssetBui
 	}
 
 	for _, role := range kops.AllInstanceGroupRoles {
-		isMaster := role == kops.InstanceGroupRoleMaster
+		isMaster := role == kops.InstanceGroupRoleControlPlane
 		isAPIServer := role == kops.InstanceGroupRoleAPIServer
 
 		images[role] = make(map[architectures.Architecture][]*nodeup.Image)
@@ -1269,7 +1274,7 @@ func newNodeUpConfigBuilder(cluster *kops.Cluster, assetBuilder *assets.AssetBui
 		if isMaster {
 			for _, etcdCluster := range cluster.Spec.EtcdClusters {
 				for _, member := range etcdCluster.Members {
-					instanceGroup := fi.StringValue(member.InstanceGroup)
+					instanceGroup := fi.ValueOf(member.InstanceGroup)
 					etcdManifest := fmt.Sprintf("manifests/etcd/%s-%s.yaml", etcdCluster.Name, instanceGroup)
 					etcdManifests[instanceGroup] = append(etcdManifests[instanceGroup], configBase.Join(etcdManifest).Path())
 				}
@@ -1306,8 +1311,8 @@ func (n *nodeUpConfigBuilder) BuildConfig(ig *kops.InstanceGroup, apiserverAddit
 		return nil, nil, fmt.Errorf("cannot determine role for instance group: %v", ig.ObjectMeta.Name)
 	}
 
-	useGossip := dns.IsGossipHostname(cluster.Spec.MasterInternalName)
-	isMaster := role == kops.InstanceGroupRoleMaster
+	useGossip := cluster.IsGossip()
+	isMaster := role == kops.InstanceGroupRoleControlPlane
 	hasAPIServer := isMaster || role == kops.InstanceGroupRoleAPIServer
 
 	config, bootConfig := nodeup.NewConfig(cluster, ig)
@@ -1395,27 +1400,58 @@ func (n *nodeUpConfigBuilder) BuildConfig(ig *kops.InstanceGroup, apiserverAddit
 		}
 	}
 
-	useConfigServer := featureflag.KopsControllerStateStore.Enabled() && (role != kops.InstanceGroupRoleMaster)
-	if useConfigServer {
-		baseURL := url.URL{
-			Scheme: "https",
-			Host:   net.JoinHostPort("kops-controller.internal."+cluster.ObjectMeta.Name, strconv.Itoa(wellknownports.KopsControllerPort)),
-			Path:   "/",
-		}
+	if hasAPIServer {
+		config.ApiserverAdditionalIPs = apiserverAdditionalIPs
+	}
 
+	// Set API server address to an IP from the cluster network CIDR
+	if cluster.UsesNoneDNS() {
+		switch cluster.Spec.GetCloudProvider() {
+		case kops.CloudProviderAWS, kops.CloudProviderHetzner, kops.CloudProviderOpenstack:
+			// Use a private IP address that belongs to the cluster network CIDR (some additional addresses may be FQDNs or public IPs)
+			for _, additionalIP := range apiserverAdditionalIPs {
+				for _, networkCIDR := range append(cluster.Spec.Networking.AdditionalNetworkCIDRs, cluster.Spec.Networking.NetworkCIDR) {
+					_, cidr, err := net.ParseCIDR(networkCIDR)
+					if err != nil {
+						return nil, nil, fmt.Errorf("failed to parse network CIDR %q: %w", networkCIDR, err)
+					}
+					if cidr.Contains(net.ParseIP(additionalIP)) {
+						bootConfig.APIServerIPs = append(bootConfig.APIServerIPs, additionalIP)
+					}
+				}
+			}
+
+		case kops.CloudProviderGCE:
+			// Use any IP address that is found (including public ones)
+			for _, additionalIP := range apiserverAdditionalIPs {
+				bootConfig.APIServerIPs = append(bootConfig.APIServerIPs, additionalIP)
+			}
+		default:
+			return nil, nil, fmt.Errorf("'none' DNS topology is not supported for cloud %q", cluster.Spec.GetCloudProvider())
+		}
+	}
+
+	useConfigServer := apiModel.UseKopsControllerForNodeConfig(cluster) && !ig.HasAPIServer()
+	if useConfigServer {
+		hosts := []string{"kops-controller.internal." + cluster.ObjectMeta.Name}
+		if cluster.UsesNoneDNS() && len(bootConfig.APIServerIPs) > 0 {
+			hosts = bootConfig.APIServerIPs
+		}
 		configServer := &nodeup.ConfigServerOptions{
-			Server:         baseURL.String(),
 			CACertificates: config.CAs[fi.CertificateIDCA],
 		}
-
+		for _, host := range hosts {
+			baseURL := url.URL{
+				Scheme: "https",
+				Host:   net.JoinHostPort(host, strconv.Itoa(wellknownports.KopsControllerPort)),
+				Path:   "/",
+			}
+			configServer.Servers = append(configServer.Servers, baseURL.String())
+		}
 		bootConfig.ConfigServer = configServer
 		delete(config.CAs, fi.CertificateIDCA)
 	} else {
-		bootConfig.ConfigBase = fi.String(n.configBase.Path())
-	}
-
-	if isMaster {
-		config.ApiserverAdditionalIPs = apiserverAdditionalIPs
+		bootConfig.ConfigBase = fi.PtrTo(n.configBase.Path())
 	}
 
 	for _, manifest := range n.assetBuilder.StaticManifests {
@@ -1466,8 +1502,10 @@ func (n *nodeUpConfigBuilder) BuildConfig(ig *kops.InstanceGroup, apiserverAddit
 		config.NvidiaGPU = n.buildNvidiaConfig(ig)
 	}
 
-	if ig.Spec.WarmPool != nil || cluster.Spec.WarmPool != nil {
-		config.WarmPoolImages = n.buildWarmPoolImages(ig)
+	if cluster.Spec.CloudProvider.AWS != nil {
+		if ig.Spec.WarmPool != nil || cluster.Spec.CloudProvider.AWS.WarmPool != nil {
+			config.WarmPoolImages = n.buildWarmPoolImages(ig)
+		}
 	}
 
 	if ig.Spec.Packages != nil {
@@ -1525,7 +1563,7 @@ func (n *nodeUpConfigBuilder) buildContainerdConfig(ig *kops.InstanceGroup) *kop
 
 // buildWarmPoolImages returns a list of container images that should be pre-pulled during instance pre-initialization
 func (n *nodeUpConfigBuilder) buildWarmPoolImages(ig *kops.InstanceGroup) []string {
-	if ig == nil || ig.Spec.Role == kops.InstanceGroupRoleMaster {
+	if ig == nil || ig.Spec.Role == kops.InstanceGroupRoleControlPlane {
 		return nil
 	}
 
