@@ -28,7 +28,6 @@ import (
 
 	"k8s.io/kops/upup/pkg/fi"
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
-	"k8s.io/kops/upup/pkg/fi/cloudup/cloudformation"
 	"k8s.io/kops/upup/pkg/fi/cloudup/terraform"
 	"k8s.io/kops/upup/pkg/fi/cloudup/terraformWriter"
 	"k8s.io/kops/util/pkg/maps"
@@ -97,9 +96,14 @@ type AutoscalingGroup struct {
 	Tags map[string]string
 	// TargetGroups is a list of ALB/NLB target group ARNs to add to the autoscaling group
 	TargetGroups []*TargetGroup
+	// CapacityRebalance makes ASG proactively replace spot instances when ASG receives a rebalance recommendation
+	CapacityRebalance *bool
+	// WarmPool is the WarmPool config for the ASG
+	WarmPool *WarmPool
 }
 
 var _ fi.CompareWithID = &AutoscalingGroup{}
+var _ fi.CloudupTaskNormalize = &AutoscalingGroup{}
 
 // CompareWithID returns the ID of the ASG
 func (e *AutoscalingGroup) CompareWithID() *string {
@@ -107,10 +111,10 @@ func (e *AutoscalingGroup) CompareWithID() *string {
 }
 
 // Find is used to discover the ASG in the cloud provider
-func (e *AutoscalingGroup) Find(c *fi.Context) (*AutoscalingGroup, error) {
-	cloud := c.Cloud.(awsup.AWSCloud)
+func (e *AutoscalingGroup) Find(c *fi.CloudupContext) (*AutoscalingGroup, error) {
+	cloud := c.T.Cloud.(awsup.AWSCloud)
 
-	g, err := findAutoscalingGroup(cloud, fi.StringValue(e.Name))
+	g, err := findAutoscalingGroup(cloud, fi.ValueOf(e.Name))
 	if err != nil {
 		return nil, err
 	}
@@ -127,7 +131,7 @@ func (e *AutoscalingGroup) Find(c *fi.Context) (*AutoscalingGroup, error) {
 
 	// Use 0 as default value when api returns nil (same as model)
 	if g.MaxInstanceLifetime == nil {
-		actual.MaxInstanceLifetime = fi.Int64(0)
+		actual.MaxInstanceLifetime = fi.PtrTo(int64(0))
 	} else {
 		actual.MaxInstanceLifetime = g.MaxInstanceLifetime
 	}
@@ -158,12 +162,12 @@ func (e *AutoscalingGroup) Find(c *fi.Context) (*AutoscalingGroup, error) {
 		for _, lb := range e.LoadBalancers {
 			// All external ELBs have their Shared field set to true. The API ELB does not.
 			// Note that Shared is set by the kops model rather than AWS tags.
-			if !fi.BoolValue(lb.Shared) {
+			if !fi.ValueOf(lb.Shared) {
 				apiLBTask = lb
 			}
 		}
 		if apiLBTask != nil && len(actual.LoadBalancers) > 0 {
-			apiLBDesc, err := c.Cloud.(awsup.AWSCloud).FindELBByNameTag(fi.StringValue(apiLBTask.Name))
+			apiLBDesc, err := c.T.Cloud.(awsup.AWSCloud).FindELBByNameTag(fi.ValueOf(apiLBTask.Name))
 			if err != nil {
 				return nil, err
 			}
@@ -182,15 +186,11 @@ func (e *AutoscalingGroup) Find(c *fi.Context) (*AutoscalingGroup, error) {
 	actual.TargetGroups = []*TargetGroup{}
 	if len(g.TargetGroupARNs) > 0 {
 		for _, tg := range g.TargetGroupARNs {
-			targetGroupName, err := awsup.GetTargetGroupNameFromARN(fi.StringValue(tg))
+			targetGroupName, err := awsup.GetTargetGroupNameFromARN(fi.ValueOf(tg))
 			if err != nil {
 				return nil, err
 			}
-			if targetGroupName != awsup.GetResourceName32(c.Cluster.Name, "tcp") && targetGroupName != awsup.GetResourceName32(c.Cluster.Name, "tls") {
-				actual.TargetGroups = append(actual.TargetGroups, &TargetGroup{ARN: aws.String(*tg), Name: aws.String(targetGroupName)})
-			} else {
-				actual.TargetGroups = append(actual.TargetGroups, &TargetGroup{ARN: aws.String(*tg), Name: aws.String(fi.StringValue(g.AutoScalingGroupName) + "-" + targetGroupName)})
-			}
+			actual.TargetGroups = append(actual.TargetGroups, &TargetGroup{ARN: aws.String(*tg), Name: aws.String(targetGroupName)})
 		}
 	}
 	sort.Stable(OrderTargetGroupsByName(actual.TargetGroups))
@@ -237,7 +237,7 @@ func (e *AutoscalingGroup) Find(c *fi.Context) (*AutoscalingGroup, error) {
 			actual.MixedSpotMaxPrice = mpd.SpotMaxPrice
 			// MixedSpotMaxPrice must be set to "" in order to unset.
 			if mpd.SpotMaxPrice == nil {
-				actual.MixedSpotMaxPrice = fi.String("")
+				actual.MixedSpotMaxPrice = fi.PtrTo("")
 			}
 		}
 
@@ -250,7 +250,7 @@ func (e *AutoscalingGroup) Find(c *fi.Context) (*AutoscalingGroup, error) {
 			}
 
 			for _, n := range g.MixedInstancesPolicy.LaunchTemplate.Overrides {
-				actual.MixedInstanceOverrides = append(actual.MixedInstanceOverrides, fi.StringValue(n.InstanceType))
+				actual.MixedInstanceOverrides = append(actual.MixedInstanceOverrides, fi.ValueOf(n.InstanceType))
 			}
 		}
 	}
@@ -291,14 +291,14 @@ func findAutoscalingGroup(cloud awsup.AWSCloud, name string) (*autoscaling.Group
 			// Check for "Delete in progress" (the only use .Status). We won't be able to update or create while
 			// this is true, but filtering it out here makes the messages slightly clearer.
 			if g.Status != nil {
-				klog.Warningf("Skipping AutoScalingGroup %v: %v", fi.StringValue(g.AutoScalingGroupName), fi.StringValue(g.Status))
+				klog.Warningf("Skipping AutoScalingGroup %v: %v", fi.ValueOf(g.AutoScalingGroupName), fi.ValueOf(g.Status))
 				continue
 			}
 
 			if aws.StringValue(g.AutoScalingGroupName) == name {
 				found = append(found, g)
 			} else {
-				klog.Warningf("Got ASG with unexpected name %q", fi.StringValue(g.AutoScalingGroupName))
+				klog.Warningf("Got ASG with unexpected name %q", fi.ValueOf(g.AutoScalingGroupName))
 			}
 		}
 
@@ -318,21 +318,16 @@ func findAutoscalingGroup(cloud awsup.AWSCloud, name string) (*autoscaling.Group
 	return nil, fmt.Errorf("found multiple AutoscalingGroups with name: %q", name)
 }
 
-func (e *AutoscalingGroup) normalize(c *fi.Context) error {
+func (e *AutoscalingGroup) Normalize(c *fi.CloudupContext) error {
 	sort.Strings(e.Metrics)
+	c.T.Cloud.(awsup.AWSCloud).AddTags(e.Name, e.Tags)
 
 	return nil
 }
 
 // Run is responsible for running the task
-func (e *AutoscalingGroup) Run(c *fi.Context) error {
-	err := e.normalize(c)
-	if err != nil {
-		return err
-	}
-	c.Cloud.(awsup.AWSCloud).AddTags(e.Name, e.Tags)
-
-	return fi.DefaultDeltaRunMethod(e, c)
+func (e *AutoscalingGroup) Run(c *fi.CloudupContext) error {
+	return fi.CloudupDefaultDeltaRunMethod(e, c)
 }
 
 // CheckChanges is responsible for checking for changes??
@@ -350,7 +345,7 @@ func (e *AutoscalingGroup) CheckChanges(a, ex, changes *AutoscalingGroup) error
 func (v *AutoscalingGroup) RenderAWS(t *awsup.AWSAPITarget, a, e, changes *AutoscalingGroup) error {
 	// @step: did we find an autoscaling group?
 	if a == nil {
-		klog.V(2).Infof("Creating autoscaling group with name: %s", fi.StringValue(e.Name))
+		klog.V(2).Infof("Creating autoscaling group with name: %s", fi.ValueOf(e.Name))
 
 		request := &autoscaling.CreateAutoScalingGroupInput{
 			AutoScalingGroupName:             e.Name,
@@ -358,11 +353,12 @@ func (v *AutoscalingGroup) RenderAWS(t *awsup.AWSAPITarget, a, e, changes *Autos
 			MaxSize:                          e.MaxSize,
 			NewInstancesProtectedFromScaleIn: e.InstanceProtection,
 			Tags:                             v.AutoscalingGroupTags(),
-			VPCZoneIdentifier:                fi.String(strings.Join(e.AutoscalingGroupSubnets(), ",")),
+			VPCZoneIdentifier:                fi.PtrTo(strings.Join(e.AutoscalingGroupSubnets(), ",")),
+			CapacityRebalance:                e.CapacityRebalance,
 		}
 
 		//On ASG creation 0 value is forbidden
-		if fi.Int64Value(e.MaxInstanceLifetime) == 0 {
+		if fi.ValueOf(e.MaxInstanceLifetime) == 0 {
 			request.MaxInstanceLifetime = nil
 		} else {
 			request.MaxInstanceLifetime = e.MaxInstanceLifetime
@@ -370,7 +366,7 @@ func (v *AutoscalingGroup) RenderAWS(t *awsup.AWSAPITarget, a, e, changes *Autos
 
 		for _, k := range e.LoadBalancers {
 			if k.LoadBalancerName == nil {
-				lbDesc, err := t.Cloud.FindELBByNameTag(fi.StringValue(k.GetName()))
+				lbDesc, err := t.Cloud.FindELBByNameTag(fi.ValueOf(k.GetName()))
 				if err != nil {
 					return err
 				}
@@ -407,7 +403,7 @@ func (v *AutoscalingGroup) RenderAWS(t *awsup.AWSAPITarget, a, e, changes *Autos
 			p := request.MixedInstancesPolicy.LaunchTemplate
 			for _, x := range e.MixedInstanceOverrides {
 				p.Overrides = append(p.Overrides, &autoscaling.LaunchTemplateOverrides{
-					InstanceType: fi.String(x),
+					InstanceType: fi.PtrTo(x),
 				},
 				)
 			}
@@ -521,7 +517,7 @@ func (v *AutoscalingGroup) RenderAWS(t *awsup.AWSAPITarget, a, e, changes *Autos
 			if changes.MixedInstanceOverrides != nil {
 				p := request.MixedInstancesPolicy.LaunchTemplate
 				for _, x := range changes.MixedInstanceOverrides {
-					p.Overrides = append(p.Overrides, &autoscaling.LaunchTemplateOverrides{InstanceType: fi.String(x)})
+					p.Overrides = append(p.Overrides, &autoscaling.LaunchTemplateOverrides{InstanceType: fi.PtrTo(x)})
 				}
 				changes.MixedInstanceOverrides = nil
 			}
@@ -551,7 +547,7 @@ func (v *AutoscalingGroup) RenderAWS(t *awsup.AWSAPITarget, a, e, changes *Autos
 			request.MaxInstanceLifetime = e.MaxInstanceLifetime
 			changes.MaxInstanceLifetime = nil
 		} else {
-			request.MaxInstanceLifetime = fi.Int64(0)
+			request.MaxInstanceLifetime = fi.PtrTo(int64(0))
 		}
 
 		var updateTagsRequest *autoscaling.CreateOrUpdateTagsInput
@@ -656,12 +652,17 @@ func (v *AutoscalingGroup) RenderAWS(t *awsup.AWSAPITarget, a, e, changes *Autos
 			changes.InstanceProtection = nil
 		}
 
+		if changes.CapacityRebalance != nil {
+			request.CapacityRebalance = e.CapacityRebalance
+			changes.CapacityRebalance = nil
+		}
+
 		empty := &AutoscalingGroup{}
 		if !reflect.DeepEqual(empty, changes) {
 			klog.Warningf("cannot apply changes to AutoScalingGroup: %v", changes)
 		}
 
-		klog.V(2).Infof("Updating autoscaling group %s", fi.StringValue(e.Name))
+		klog.V(2).Infof("Updating autoscaling group %s", fi.ValueOf(e.Name))
 
 		if _, err := t.Cloud.Autoscaling().UpdateAutoScalingGroup(request); err != nil {
 			return fmt.Errorf("error updating AutoscalingGroup: %v", err)
@@ -760,7 +761,7 @@ func (e *AutoscalingGroup) AutoscalingGroupSubnets() []string {
 	var list []string
 
 	for _, x := range e.Subnets {
-		list = append(list, fi.StringValue(x.ID))
+		list = append(list, fi.ValueOf(x.ID))
 	}
 
 	return list
@@ -930,6 +931,11 @@ type terraformMixedInstancesPolicy struct {
 	InstanceDistribution []*terraformAutoscalingInstanceDistribution `cty:"instances_distribution"`
 }
 
+type terraformWarmPool struct {
+	MinSize *int64 `cty:"min_size"`
+	MaxSize *int64 `cty:"max_group_prepared_capacity"`
+}
+
 type terraformAutoscalingGroup struct {
 	Name                    *string                                          `cty:"name"`
 	LaunchConfigurationName *terraformWriter.Literal                         `cty:"launch_configuration"`
@@ -946,6 +952,8 @@ type terraformAutoscalingGroup struct {
 	LoadBalancers           []*terraformWriter.Literal                       `cty:"load_balancers"`
 	TargetGroupARNs         []*terraformWriter.Literal                       `cty:"target_group_arns"`
 	MaxInstanceLifetime     *int64                                           `cty:"max_instance_lifetime"`
+	CapacityRebalance       *bool                                            `cty:"capacity_rebalance"`
+	WarmPool                *terraformWarmPool                               `cty:"warm_pool"`
 }
 
 // RenderTerraform is responsible for rendering the terraform codebase
@@ -958,6 +966,7 @@ func (_ *AutoscalingGroup) RenderTerraform(t *terraform.TerraformTarget, a, e, c
 		EnabledMetrics:      aws.StringSlice(e.Metrics),
 		InstanceProtection:  e.InstanceProtection,
 		MaxInstanceLifetime: e.MaxInstanceLifetime,
+		CapacityRebalance:   e.CapacityRebalance,
 	}
 
 	for _, s := range e.Subnets {
@@ -967,9 +976,9 @@ func (_ *AutoscalingGroup) RenderTerraform(t *terraform.TerraformTarget, a, e, c
 	for _, k := range maps.SortedKeys(e.Tags) {
 		v := e.Tags[k]
 		tf.Tags = append(tf.Tags, &terraformASGTag{
-			Key:               fi.String(k),
-			Value:             fi.String(v),
-			PropagateAtLaunch: fi.Bool(true),
+			Key:               fi.PtrTo(k),
+			Value:             fi.PtrTo(v),
+			PropagateAtLaunch: fi.PtrTo(true),
 		})
 	}
 
@@ -985,7 +994,7 @@ func (_ *AutoscalingGroup) RenderTerraform(t *terraform.TerraformTarget, a, e, c
 
 	if e.UseMixedInstancesPolicy() {
 		// Temporary warning until https://github.com/terraform-providers/terraform-provider-aws/issues/9750 is resolved
-		if e.MixedSpotAllocationStrategy == fi.String("capacity-optimized") {
+		if e.MixedSpotAllocationStrategy == fi.PtrTo("capacity-optimized") {
 			fmt.Print("Terraform does not currently support a capacity optimized strategy - please see https://github.com/terraform-providers/terraform-provider-aws/issues/9750")
 		}
 
@@ -1015,7 +1024,7 @@ func (_ *AutoscalingGroup) RenderTerraform(t *terraform.TerraformTarget, a, e, c
 		}
 
 		for _, x := range e.MixedInstanceOverrides {
-			tf.MixedInstancesPolicy[0].LaunchTemplate[0].Override = append(tf.MixedInstancesPolicy[0].LaunchTemplate[0].Override, &terraformAutoscalingMixedInstancesPolicyLaunchTemplateOverride{InstanceType: fi.String(x)})
+			tf.MixedInstancesPolicy[0].LaunchTemplate[0].Override = append(tf.MixedInstancesPolicy[0].LaunchTemplate[0].Override, &terraformAutoscalingMixedInstancesPolicyLaunchTemplateOverride{InstanceType: fi.PtrTo(x)})
 		}
 	} else if e.LaunchTemplate != nil {
 		tf.LaunchTemplate = &terraformAutoscalingLaunchTemplateSpecification{
@@ -1030,6 +1039,9 @@ func (_ *AutoscalingGroup) RenderTerraform(t *terraform.TerraformTarget, a, e, c
 	for k := range e.Tags {
 		if strings.HasPrefix(k, CloudTagInstanceGroupRolePrefix) {
 			suffix := strings.TrimPrefix(k, CloudTagInstanceGroupRolePrefix)
+			if suffix == "control-plane" {
+				suffix = "master"
+			}
 			if role != "" && role != suffix {
 				return fmt.Errorf("Found multiple role tags: %q vs %q", role, suffix)
 			}
@@ -1060,154 +1072,22 @@ func (_ *AutoscalingGroup) RenderTerraform(t *terraform.TerraformTarget, a, e, c
 	var processes []*string
 	if e.SuspendProcesses != nil {
 		for _, p := range *e.SuspendProcesses {
-			processes = append(processes, fi.String(p))
+			processes = append(processes, fi.PtrTo(p))
 		}
 	}
 	tf.SuspendedProcesses = processes
 
+	if e.WarmPool != nil && *e.WarmPool.Enabled {
+		tf.WarmPool = &terraformWarmPool{
+			MinSize: &e.WarmPool.MinSize,
+			MaxSize: e.WarmPool.MaxSize,
+		}
+	}
+
 	return t.RenderResource("aws_autoscaling_group", *e.Name, tf)
 }
 
 // TerraformLink fills in the property
 func (e *AutoscalingGroup) TerraformLink() *terraformWriter.Literal {
-	return terraformWriter.LiteralProperty("aws_autoscaling_group", fi.StringValue(e.Name), "id")
-}
-
-type cloudformationASGTag struct {
-	Key               *string `json:"Key"`
-	Value             *string `json:"Value"`
-	PropagateAtLaunch *bool   `json:"PropagateAtLaunch"`
-}
-
-type cloudformationASGMetricsCollection struct {
-	Granularity *string   `json:"Granularity"`
-	Metrics     []*string `json:"Metrics"`
-}
-
-type cloudformationAutoscalingLaunchTemplateSpecification struct {
-	// LaunchTemplateId is the IDx of the template to use.
-	LaunchTemplateId *cloudformation.Literal `json:"LaunchTemplateId,omitempty"`
-	// Version is the version number of the template to use.
-	Version *cloudformation.Literal `json:"Version,omitempty"`
-}
-
-type cloudformationAutoscalingLaunchTemplateOverride struct {
-	// InstanceType is the instance to use
-	InstanceType *string `json:"InstanceType,omitempty"`
-}
-
-type cloudformationAutoscalingLaunchTemplate struct {
-	// LaunchTemplateSpecification is the definition for a LT
-	LaunchTemplateSpecification *cloudformationAutoscalingLaunchTemplateSpecification `json:"LaunchTemplateSpecification,omitempty"`
-	// Override the is machine type override
-	Overrides []*cloudformationAutoscalingLaunchTemplateOverride `json:"Overrides,omitempty"`
-}
-
-type cloudformationAutoscalingInstanceDistribution struct {
-	// OnDemandAllocationStrategy
-	OnDemandAllocationStrategy *string `json:"InstancesDistribution,omitempty"`
-	// OnDemandBaseCapacity is the base ondemand requirement
-	OnDemandBaseCapacity *int64 `json:"OnDemandBaseCapacity,omitempty"`
-	// OnDemandPercentageAboveBaseCapacity is the percentage above base for on-demand instances
-	OnDemandPercentageAboveBaseCapacity *int64 `json:"OnDemandPercentageAboveBaseCapacity,omitempty"`
-	// SpotAllocationStrategy is the spot allocation stratergy
-	SpotAllocationStrategy *string `json:"SpotAllocationStrategy,omitempty"`
-	// SpotInstancePool is the number of pools
-	SpotInstancePools *int64 `json:"SpotInstancePools,omitempty"`
-	// SpotMaxPrice is the max bid on spot instance, defaults to demand value
-	SpotMaxPrice *string `json:"SpotMaxPrice,omitempty"`
-}
-
-type cloudformationMixedInstancesPolicy struct {
-	// LaunchTemplate is the launch template spec
-	LaunchTemplate *cloudformationAutoscalingLaunchTemplate `json:"LaunchTemplate,omitempty"`
-	// InstanceDistribution is the distribution strategy
-	InstanceDistribution *cloudformationAutoscalingInstanceDistribution `json:"InstancesDistribution,omitempty"`
-}
-
-type cloudformationAutoscalingGroup struct {
-	Name                    *string                                               `json:"AutoScalingGroupName,omitempty"`
-	LaunchConfigurationName *cloudformation.Literal                               `json:"LaunchConfigurationName,omitempty"`
-	LaunchTemplate          *cloudformationAutoscalingLaunchTemplateSpecification `json:"LaunchTemplate,omitempty"`
-	MaxSize                 *string                                               `json:"MaxSize,omitempty"`
-	MinSize                 *string                                               `json:"MinSize,omitempty"`
-	VPCZoneIdentifier       []*cloudformation.Literal                             `json:"VPCZoneIdentifier,omitempty"`
-	Tags                    []*cloudformationASGTag                               `json:"Tags,omitempty"`
-	MetricsCollection       []*cloudformationASGMetricsCollection                 `json:"MetricsCollection,omitempty"`
-	MixedInstancesPolicy    *cloudformationMixedInstancesPolicy                   `json:"MixedInstancesPolicy,omitempty"`
-	LoadBalancerNames       []*cloudformation.Literal                             `json:"LoadBalancerNames,omitempty"`
-	TargetGroupARNs         []*cloudformation.Literal                             `json:"TargetGroupARNs,omitempty"`
-}
-
-// RenderCloudformation is responsible for generating the cloudformation template
-func (_ *AutoscalingGroup) RenderCloudformation(t *cloudformation.CloudformationTarget, a, e, changes *AutoscalingGroup) error {
-	cf := &cloudformationAutoscalingGroup{
-		Name:    e.Name,
-		MinSize: fi.ToString(e.MinSize),
-		MaxSize: fi.ToString(e.MaxSize),
-		MetricsCollection: []*cloudformationASGMetricsCollection{
-			{
-				Granularity: e.Granularity,
-				Metrics:     aws.StringSlice(e.Metrics),
-			},
-		},
-	}
-
-	if e.UseMixedInstancesPolicy() {
-		cf.MixedInstancesPolicy = &cloudformationMixedInstancesPolicy{
-			LaunchTemplate: &cloudformationAutoscalingLaunchTemplate{
-				LaunchTemplateSpecification: &cloudformationAutoscalingLaunchTemplateSpecification{
-					LaunchTemplateId: e.LaunchTemplate.CloudformationLink(),
-					Version:          e.LaunchTemplate.CloudformationVersion(),
-				},
-			},
-			InstanceDistribution: &cloudformationAutoscalingInstanceDistribution{
-				OnDemandAllocationStrategy:          e.MixedOnDemandAllocationStrategy,
-				OnDemandBaseCapacity:                e.MixedOnDemandBase,
-				OnDemandPercentageAboveBaseCapacity: e.MixedOnDemandAboveBase,
-				SpotAllocationStrategy:              e.MixedSpotAllocationStrategy,
-				SpotInstancePools:                   e.MixedSpotInstancePools,
-				SpotMaxPrice:                        e.MixedSpotMaxPrice,
-			},
-		}
-
-		for _, x := range e.MixedInstanceOverrides {
-			cf.MixedInstancesPolicy.LaunchTemplate.Overrides = append(cf.MixedInstancesPolicy.LaunchTemplate.Overrides, &cloudformationAutoscalingLaunchTemplateOverride{InstanceType: fi.String(x)})
-		}
-	} else if e.LaunchTemplate != nil {
-		cf.LaunchTemplate = &cloudformationAutoscalingLaunchTemplateSpecification{
-			LaunchTemplateId: e.LaunchTemplate.CloudformationLink(),
-			Version:          e.LaunchTemplate.CloudformationVersion(),
-		}
-	} else {
-		return fmt.Errorf("could not find one of launch configuration, mixed instances policy, or launch template")
-	}
-
-	for _, s := range e.Subnets {
-		cf.VPCZoneIdentifier = append(cf.VPCZoneIdentifier, s.CloudformationLink())
-	}
-
-	for _, k := range maps.SortedKeys(e.Tags) {
-		v := e.Tags[k]
-		cf.Tags = append(cf.Tags, &cloudformationASGTag{
-			Key:               fi.String(k),
-			Value:             fi.String(v),
-			PropagateAtLaunch: fi.Bool(true),
-		})
-	}
-
-	for _, k := range e.LoadBalancers {
-		cf.LoadBalancerNames = append(cf.LoadBalancerNames, k.CloudformationLink())
-	}
-
-	for _, tg := range e.TargetGroups {
-		cf.TargetGroupARNs = append(cf.TargetGroupARNs, tg.CloudformationLink())
-	}
-
-	return t.RenderResource("AWS::AutoScaling::AutoScalingGroup", fi.StringValue(e.Name), cf)
-}
-
-// CloudformationLink is adds a reference
-func (e *AutoscalingGroup) CloudformationLink() *cloudformation.Literal {
-	return cloudformation.Ref("AWS::AutoScaling::AutoScalingGroup", fi.StringValue(e.Name))
+	return terraformWriter.LiteralProperty("aws_autoscaling_group", fi.ValueOf(e.Name), "id")
 }
