@@ -26,7 +26,6 @@ import (
 	"k8s.io/kops/pkg/featureflag"
 	"k8s.io/kops/upup/pkg/fi"
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
-	"k8s.io/kops/upup/pkg/fi/cloudup/cloudformation"
 	"k8s.io/kops/upup/pkg/fi/cloudup/terraform"
 	"k8s.io/kops/upup/pkg/fi/cloudup/terraformWriter"
 )
@@ -40,7 +39,7 @@ type VPC struct {
 	CIDR *string
 
 	// AmazonIPv6 is used only for Terraform rendering.
-	// Direct and CloudFormation rendering is handled via the VPCAmazonIPv6CIDRBlock task
+	// Direct rendering is handled via the VPCAmazonIPv6CIDRBlock task
 	AmazonIPv6 *bool
 	IPv6CIDR   *string
 
@@ -59,20 +58,20 @@ type VPC struct {
 }
 
 var (
-	_ fi.CompareWithID     = &VPC{}
-	_ fi.ProducesDeletions = &VPC{}
+	_ fi.CompareWithID            = &VPC{}
+	_ fi.CloudupProducesDeletions = &VPC{}
 )
 
 func (e *VPC) CompareWithID() *string {
 	return e.ID
 }
 
-func (e *VPC) Find(c *fi.Context) (*VPC, error) {
-	cloud := c.Cloud.(awsup.AWSCloud)
+func (e *VPC) Find(c *fi.CloudupContext) (*VPC, error) {
+	cloud := c.T.Cloud.(awsup.AWSCloud)
 
 	request := &ec2.DescribeVpcsInput{}
 
-	if fi.StringValue(e.ID) != "" {
+	if fi.ValueOf(e.ID) != "" {
 		request.VpcIds = []*string{e.ID}
 	} else {
 		request.Filters = cloud.BuildFilters(e.Name)
@@ -168,16 +167,16 @@ func (s *VPC) CheckChanges(a, e, changes *VPC) error {
 	return nil
 }
 
-func (e *VPC) Run(c *fi.Context) error {
-	return fi.DefaultDeltaRunMethod(e, c)
+func (e *VPC) Run(c *fi.CloudupContext) error {
+	return fi.CloudupDefaultDeltaRunMethod(e, c)
 }
 
 func (_ *VPC) RenderAWS(t *awsup.AWSAPITarget, a, e, changes *VPC) error {
-	shared := fi.BoolValue(e.Shared)
+	shared := fi.ValueOf(e.Shared)
 	if shared {
 		// Verify the VPC was found and matches our required settings
 		if a == nil {
-			return fmt.Errorf("VPC with id %q not found", fi.StringValue(e.ID))
+			return fmt.Errorf("VPC with id %q not found", fi.ValueOf(e.ID))
 		}
 
 		if changes != nil && changes.EnableDNSSupport != nil {
@@ -185,7 +184,7 @@ func (_ *VPC) RenderAWS(t *awsup.AWSAPITarget, a, e, changes *VPC) error {
 				klog.Warningf("VPC did not have EnableDNSSupport=true, but ignoring because of VPCSkipEnableDNSSupport feature-flag")
 			} else {
 				// TODO: We could easily just allow kops to fix this...
-				return fmt.Errorf("VPC with id %q was set to be shared, but did not have EnableDNSSupport=true.", fi.StringValue(e.ID))
+				return fmt.Errorf("VPC with id %q was set to be shared, but did not have EnableDNSSupport=true.", fi.ValueOf(e.ID))
 			}
 		}
 	}
@@ -233,16 +232,16 @@ func (_ *VPC) RenderAWS(t *awsup.AWSAPITarget, a, e, changes *VPC) error {
 	return t.AddAWSTags(*e.ID, e.Tags)
 }
 
-func (e *VPC) FindDeletions(c *fi.Context) ([]fi.Deletion, error) {
-	if fi.IsNilOrEmpty(e.ID) || fi.BoolValue(e.Shared) {
+func (e *VPC) FindDeletions(c *fi.CloudupContext) ([]fi.CloudupDeletion, error) {
+	if fi.IsNilOrEmpty(e.ID) || fi.ValueOf(e.Shared) {
 		return nil, nil
 	}
 
-	var removals []fi.Deletion
+	var removals []fi.CloudupDeletion
 	request := &ec2.DescribeVpcsInput{
 		VpcIds: []*string{e.ID},
 	}
-	cloud := c.Cloud.(awsup.AWSCloud)
+	cloud := c.T.Cloud.(awsup.AWSCloud)
 	response, err := cloud.EC2().DescribeVpcs(request)
 	if err != nil {
 		return nil, err
@@ -258,13 +257,13 @@ func (e *VPC) FindDeletions(c *fi.Context) ([]fi.Deletion, error) {
 	for _, association := range vpc.CidrBlockAssociationSet {
 		// We'll only delete CIDR associations that are not the primary association
 		// and that have a state of "associated"
-		if fi.StringValue(association.CidrBlock) == fi.StringValue(vpc.CidrBlock) ||
-			association.CidrBlockState != nil && fi.StringValue(association.CidrBlockState.State) != ec2.VpcCidrBlockStateCodeAssociated {
+		if fi.ValueOf(association.CidrBlock) == fi.ValueOf(vpc.CidrBlock) ||
+			association.CidrBlockState != nil && fi.ValueOf(association.CidrBlockState.State) != ec2.VpcCidrBlockStateCodeAssociated {
 			continue
 		}
 		match := false
 		for _, cidr := range e.AssociateExtraCIDRBlocks {
-			if fi.StringValue(association.CidrBlock) == cidr {
+			if fi.ValueOf(association.CidrBlock) == cidr {
 				match = true
 				break
 			}
@@ -280,6 +279,10 @@ func (e *VPC) FindDeletions(c *fi.Context) ([]fi.Deletion, error) {
 	return removals, nil
 }
 
+type terraformVPCData struct {
+	ID *string `cty:"id"`
+}
+
 type terraformVPC struct {
 	CIDR               *string           `cty:"cidr_block"`
 	EnableDNSHostnames *bool             `cty:"enable_dns_hostnames"`
@@ -293,15 +296,46 @@ func (_ *VPC) RenderTerraform(t *terraform.TerraformTarget, a, e, changes *VPC)
 		return err
 	}
 
-	shared := fi.BoolValue(e.Shared)
+	cidrPrefixLengthCaptureList := terraformWriter.LiteralFunctionExpression("regex",
+		terraformWriter.LiteralFromStringValue(".*/(\\\\d+)"),
+		terraformWriter.LiteralTokens("local", "vpc_ipv6_cidr_block"),
+	)
+	cidrPrefixLengthString := terraformWriter.LiteralIndexExpression(
+		cidrPrefixLengthCaptureList,
+		terraformWriter.LiteralFromIntValue(0),
+	)
+	if err := t.AddOutputVariable("vpc_ipv6_cidr_length", terraformWriter.LiteralNullConditionalExpression(
+		terraformWriter.LiteralTokens("local", "vpc_ipv6_cidr_block"),
+		terraformWriter.LiteralFunctionExpression("tonumber", cidrPrefixLengthString),
+	)); err != nil {
+		return err
+	}
+
+	shared := fi.ValueOf(e.Shared)
 	if shared {
 		// Not terraform owned / managed
 		// We won't apply changes, but our validation (kops update) will still warn
-		return nil
+
+		if err := t.AddOutputVariable("vpc_cidr_block", terraformWriter.LiteralData("aws_vpc", *e.Name, "cidr_block")); err != nil {
+			return err
+		}
+
+		if err := t.AddOutputVariable("vpc_ipv6_cidr_block", terraformWriter.LiteralData("aws_vpc", *e.Name, "ipv6_cidr_block")); err != nil {
+			return err
+		}
+
+		tf := terraformVPCData{
+			ID: e.ID,
+		}
+
+		return t.RenderDataSource("aws_vpc", *e.Name, tf)
 	}
 
 	if err := t.AddOutputVariable("vpc_cidr_block", terraformWriter.LiteralProperty("aws_vpc", *e.Name, "cidr_block")); err != nil {
-		// TODO: Should we try to output vpc_cidr_block for shared vpcs?
+		return err
+	}
+
+	if err := t.AddOutputVariable("vpc_ipv6_cidr_block", terraformWriter.LiteralProperty("aws_vpc", *e.Name, "ipv6_cidr_block")); err != nil {
 		return err
 	}
 
@@ -317,7 +351,7 @@ func (_ *VPC) RenderTerraform(t *terraform.TerraformTarget, a, e, changes *VPC)
 }
 
 func (e *VPC) TerraformLink() *terraformWriter.Literal {
-	shared := fi.BoolValue(e.Shared)
+	shared := fi.ValueOf(e.Shared)
 	if shared {
 		if e.ID == nil {
 			klog.Fatalf("ID must be set, if VPC is shared: %s", e)
@@ -330,54 +364,15 @@ func (e *VPC) TerraformLink() *terraformWriter.Literal {
 	return terraformWriter.LiteralProperty("aws_vpc", *e.Name, "id")
 }
 
-type cloudformationVPC struct {
-	CidrBlock          *string             `json:"CidrBlock,omitempty"`
-	EnableDnsHostnames *bool               `json:"EnableDnsHostnames,omitempty"`
-	EnableDnsSupport   *bool               `json:"EnableDnsSupport,omitempty"`
-	Tags               []cloudformationTag `json:"Tags,omitempty"`
-}
-
-func (_ *VPC) RenderCloudformation(t *cloudformation.CloudformationTarget, a, e, changes *VPC) error {
-	shared := fi.BoolValue(e.Shared)
-	if shared {
-		// Not cloudformation owned / managed
-		// We won't apply changes, but our validation (kops update) will still warn
-		return nil
-	}
-
-	tf := &cloudformationVPC{
-		CidrBlock:          e.CIDR,
-		EnableDnsHostnames: e.EnableDNSHostnames,
-		EnableDnsSupport:   e.EnableDNSSupport,
-		Tags:               buildCloudformationTags(e.Tags),
-	}
-
-	return t.RenderResource("AWS::EC2::VPC", *e.Name, tf)
-}
-
-func (e *VPC) CloudformationLink() *cloudformation.Literal {
-	shared := fi.BoolValue(e.Shared)
-	if shared {
-		if e.ID == nil {
-			klog.Fatalf("ID must be set, if VPC is shared: %s", e)
-		}
-
-		klog.V(4).Infof("reusing existing VPC with id %q", *e.ID)
-		return cloudformation.LiteralString(*e.ID)
-	}
-
-	return cloudformation.Ref("AWS::EC2::VPC", *e.Name)
-}
-
 type deleteVPCCIDRBlock struct {
 	vpcID         *string
 	cidrBlock     *string
 	associationID *string
 }
 
-var _ fi.Deletion = &deleteVPCCIDRBlock{}
+var _ fi.CloudupDeletion = &deleteVPCCIDRBlock{}
 
-func (d *deleteVPCCIDRBlock) Delete(t fi.Target) error {
+func (d *deleteVPCCIDRBlock) Delete(t fi.CloudupTarget) error {
 	awsTarget, ok := t.(*awsup.AWSAPITarget)
 	if !ok {
 		return fmt.Errorf("unexpected target type for deletion: %T", t)
