@@ -37,7 +37,6 @@ import (
 	"github.com/aws/aws-sdk-go/aws/session"
 	"github.com/aws/aws-sdk-go/service/autoscaling"
 	"github.com/aws/aws-sdk-go/service/autoscaling/autoscalingiface"
-	"github.com/aws/aws-sdk-go/service/cloudformation"
 	"github.com/aws/aws-sdk-go/service/ec2"
 	"github.com/aws/aws-sdk-go/service/ec2/ec2iface"
 	"github.com/aws/aws-sdk-go/service/elb"
@@ -98,6 +97,7 @@ const (
 	TagNameEtcdClusterPrefix = "k8s.io/etcd/"
 )
 
+const TagRoleControlPlane = "control-plane"
 const TagRoleMaster = "master"
 
 // TagNameKopsRole is the AWS tag used to identify the role an object plays for a cluster
@@ -110,12 +110,8 @@ const tagNameDetachedInstance = "kops.k8s.io/detached-from-asg"
 
 const (
 	WellKnownAccountAmazonLinux2 = "137112412989"
-	WellKnownAccountCentOS       = "125523088429"
-	WellKnownAccountCoreOS       = "595879546273"
 	WellKnownAccountDebian       = "136693071363"
-	WellKnownAccountDebian9      = "379101102735"
 	WellKnownAccountFlatcar      = "075585003325"
-	WellKnownAccountKopeio       = "383156758163"
 	WellKnownAccountRedhat       = "309956199498"
 	WellKnownAccountUbuntu       = "099720109477"
 )
@@ -127,7 +123,6 @@ const AWSErrCodeInvalidAction = "InvalidAction"
 
 type AWSCloud interface {
 	fi.Cloud
-	CloudFormation() *cloudformation.CloudFormation
 	Session() (*session.Session, error)
 	EC2() ec2iface.EC2API
 	IAM() iamiface.IAMAPI
@@ -168,6 +163,7 @@ type AWSCloud interface {
 	DescribeELBTags(loadBalancerNames []string) (map[string][]*elb.Tag, error)
 	FindELBV2ByNameTag(findNameTag string) (*elbv2.LoadBalancer, error)
 	DescribeELBV2Tags(loadBalancerNames []string) (map[string][]*elbv2.Tag, error)
+	FindELBV2NetworkInterfacesByName(vpcID string, loadBalancerName string) ([]*ec2.NetworkInterface, error)
 
 	// DescribeInstance is a helper that queries for the specified instance by id
 	DescribeInstance(instanceID string) (*ec2.Instance, error)
@@ -197,7 +193,6 @@ type AWSCloud interface {
 }
 
 type awsCloudImplementation struct {
-	cf          *cloudformation.CloudFormation
 	ec2         *ec2.EC2
 	iam         *iam.IAM
 	elb         *elb.ELB
@@ -261,14 +256,6 @@ func NewAWSCloud(region string, tags map[string]string) (AWSCloud, error) {
 		config = config.WithCredentialsChainVerboseErrors(true)
 		config = request.WithRetryer(config, newLoggingRetryer(ClientMaxRetries))
 
-		// We have the updated aws sdk from 1.9, but don't have https://github.com/kubernetes/kubernetes/pull/55307
-		// Set the SleepDelay function to work around this
-		// TODO: Remove once we update to k8s >= 1.9 (or a version of the retry delayer than includes this)
-		config.SleepDelay = func(d time.Duration) {
-			klog.V(6).Infof("aws request sleeping for %v", d)
-			time.Sleep(d)
-		}
-
 		requestLogger := newRequestLogger(2)
 
 		sess, err := session.NewSessionWithOptions(session.Options{
@@ -278,17 +265,6 @@ func NewAWSCloud(region string, tags map[string]string) (AWSCloud, error) {
 		if err != nil {
 			return c, err
 		}
-		c.cf = cloudformation.New(sess, config)
-		c.cf.Handlers.Send.PushFront(requestLogger)
-		c.addHandlers(region, &c.cf.Handlers)
-
-		sess, err = session.NewSessionWithOptions(session.Options{
-			Config:            *config,
-			SharedConfigState: session.SharedConfigEnable,
-		})
-		if err != nil {
-			return c, err
-		}
 		c.ec2 = ec2.New(sess, config)
 		c.ec2.Handlers.Send.PushFront(requestLogger)
 		c.addHandlers(region, &c.ec2.Handlers)
@@ -1235,13 +1211,22 @@ func buildCloudInstance(i *autoscaling.Instance, instances map[string]*ec2.Insta
 
 func addCloudInstanceData(cm *cloudinstances.CloudInstance, instance *ec2.Instance) {
 	cm.MachineType = aws.StringValue(instance.InstanceType)
+	isControlPlane := false
 	for _, tag := range instance.Tags {
 		key := aws.StringValue(tag.Key)
 		if !strings.HasPrefix(key, TagNameRolePrefix) {
 			continue
 		}
 		role := strings.TrimPrefix(key, TagNameRolePrefix)
-		cm.Roles = append(cm.Roles, role)
+		if role == "master" || role == "control-plane" {
+			isControlPlane = true
+		} else {
+			cm.Roles = append(cm.Roles, role)
+			cm.PrivateIP = aws.StringValue(instance.PrivateIpAddress)
+		}
+	}
+	if isControlPlane {
+		cm.Roles = append(cm.Roles, "control-plane")
 		cm.PrivateIP = aws.StringValue(instance.PrivateIpAddress)
 	}
 }
@@ -1892,6 +1877,35 @@ func findELBV2ByNameTag(c AWSCloud, findNameTag string) (*elbv2.LoadBalancer, er
 	return found[0], nil
 }
 
+func (c *awsCloudImplementation) FindELBV2NetworkInterfacesByName(vpcID string, loadBalancerName string) ([]*ec2.NetworkInterface, error) {
+	return findELBV2NetworkInterfaces(c, vpcID, loadBalancerName)
+}
+
+func findELBV2NetworkInterfaces(c AWSCloud, vpcID, lbName string) ([]*ec2.NetworkInterface, error) {
+	klog.V(2).Infof("Listing all NLB network interfaces")
+
+	request := &ec2.DescribeNetworkInterfacesInput{
+		Filters: []*ec2.Filter{
+			NewEC2Filter("vpc-id", vpcID),
+			NewEC2Filter("interface-type", "network_load_balancer"),
+		},
+	}
+
+	response, err := c.EC2().DescribeNetworkInterfaces(request)
+	if err != nil {
+		return nil, fmt.Errorf("error describing network interfaces: %w", err)
+	}
+
+	var found []*ec2.NetworkInterface
+	for _, ni := range response.NetworkInterfaces {
+		if strings.HasPrefix(aws.StringValue(ni.Description), "ELB net/"+lbName+"/") {
+			found = append(found, ni)
+		}
+	}
+
+	return found, nil
+}
+
 func (c *awsCloudImplementation) DescribeELBV2Tags(loadBalancerArns []string) (map[string][]*elbv2.Tag, error) {
 	return describeELBV2Tags(c, loadBalancerArns)
 }
@@ -2051,20 +2065,12 @@ func resolveImage(ssmClient ssmiface.SSMAPI, ec2Client ec2iface.EC2API, name str
 			switch owner {
 			case "amazon", "amazon.com":
 				owner = WellKnownAccountAmazonLinux2
-			case "centos":
-				owner = WellKnownAccountCentOS
-			case "coreos", "coreos.com":
-				owner = WellKnownAccountCoreOS
-			case "debian9":
-				owner = WellKnownAccountDebian9
 			case "debian10":
 				owner = WellKnownAccountDebian
 			case "debian11":
 				owner = WellKnownAccountDebian
 			case "flatcar":
 				owner = WellKnownAccountFlatcar
-			case "kopeio", "kope.io":
-				owner = WellKnownAccountKopeio
 			case "redhat", "redhat.com":
 				owner = WellKnownAccountRedhat
 			case "ubuntu":
@@ -2078,23 +2084,28 @@ func resolveImage(ssmClient ssmiface.SSMAPI, ec2Client ec2iface.EC2API, name str
 		}
 	}
 
-	response, err := ec2Client.DescribeImages(request)
+	var image *ec2.Image
+	err := ec2Client.DescribeImagesPagesWithContext(context.TODO(), request, func(output *ec2.DescribeImagesOutput, b bool) bool {
+		for _, v := range output.Images {
+			if image == nil {
+				image = v
+			} else {
+				itime, _ := time.Parse(time.RFC3339, *image.CreationDate)
+				vtime, _ := time.Parse(time.RFC3339, *v.CreationDate)
+				if vtime.After(itime) {
+					image = v
+				}
+			}
+		}
+		return true
+	})
 	if err != nil {
 		return nil, fmt.Errorf("error listing images: %v", err)
 	}
-	if response == nil || len(response.Images) == 0 {
+	if image == nil {
 		return nil, fmt.Errorf("could not find Image for %q", name)
 	}
 
-	image := response.Images[0]
-	for _, v := range response.Images {
-		itime, _ := time.Parse(time.RFC3339, *image.CreationDate)
-		vtime, _ := time.Parse(time.RFC3339, *v.CreationDate)
-		if vtime.After(itime) {
-			image = v
-		}
-	}
-
 	klog.V(4).Infof("Resolved image %q", aws.StringValue(image.ImageId))
 	return image, nil
 }
@@ -2156,10 +2167,6 @@ func (c *awsCloudImplementation) DNS() (dnsprovider.Interface, error) {
 	return provider, nil
 }
 
-func (c *awsCloudImplementation) CloudFormation() *cloudformation.CloudFormation {
-	return c.cf
-}
-
 func (c *awsCloudImplementation) EC2() ec2iface.EC2API {
 	return c.ec2
 }
@@ -2261,7 +2268,7 @@ func getApiIngressStatus(c AWSCloud, cluster *kops.Cluster) ([]fi.ApiIngressStat
 
 func findDNSName(c AWSCloud, cluster *kops.Cluster) (string, error) {
 	name := "api." + cluster.Name
-	if cluster.Spec.API == nil || cluster.Spec.API.LoadBalancer == nil {
+	if cluster.Spec.API.LoadBalancer == nil {
 		return "", nil
 	}
 	if cluster.Spec.API.LoadBalancer.Class == kops.LoadBalancerClassClassic {
@@ -2285,18 +2292,23 @@ func (c *awsCloudImplementation) DefaultInstanceType(cluster *kops.Cluster, ig *
 	var candidates []string
 
 	switch ig.Spec.Role {
-	case kops.InstanceGroupRoleMaster, kops.InstanceGroupRoleNode, kops.InstanceGroupRoleAPIServer:
+	case kops.InstanceGroupRoleControlPlane, kops.InstanceGroupRoleNode, kops.InstanceGroupRoleAPIServer:
 		// t3.medium is the cheapest instance with 4GB of mem, unlimited by default, fast and has decent network
 		// c5.large and c4.large are a good second option in case t3.medium is not available in the AZ
-		candidates = []string{"t3.medium", "c5.large", "c4.large"}
+		candidates = []string{"t3.medium", "c5.large", "c4.large", "t4g.medium"}
 
 	case kops.InstanceGroupRoleBastion:
-		candidates = []string{"t3.micro", "t2.micro"}
+		candidates = []string{"t3.micro", "t2.micro", "t4g.micro"}
 
 	default:
 		return "", fmt.Errorf("unhandled role %q", ig.Spec.Role)
 	}
 
+	imageArch := "x86_64"
+	if imageInfo, err := c.ResolveImage(ig.Spec.Image); err == nil {
+		imageArch = fi.ValueOf(imageInfo.Architecture)
+	}
+
 	// Find the AZs the InstanceGroup targets
 	igZones, err := model.FindZonesForInstanceGroup(cluster, ig)
 	if err != nil {
@@ -2306,6 +2318,16 @@ func (c *awsCloudImplementation) DefaultInstanceType(cluster *kops.Cluster, ig *
 
 	// TODO: Validate that instance type exists in all AZs, but skip AZs that don't support any VPC stuff
 	for _, instanceType := range candidates {
+		if strings.HasPrefix(instanceType, "t4g") {
+			if imageArch != "arm64" {
+				continue
+			}
+		} else {
+			if imageArch == "arm64" {
+				continue
+			}
+		}
+
 		zones, err := c.zonesWithInstanceType(instanceType)
 		if err != nil {
 			return "", err
@@ -2371,10 +2393,10 @@ func describeInstanceType(c AWSCloud, instanceType string) (*ec2.InstanceTypeInf
 	}
 	resp, err := c.EC2().DescribeInstanceTypes(req)
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("describing instance type %q in region %q: %w", instanceType, c.Region(), err)
 	}
 	if len(resp.InstanceTypes) != 1 {
-		return nil, fmt.Errorf("invalid instance type specified: %v", instanceType)
+		return nil, fmt.Errorf("instance type %q not found in region %q", instanceType, c.Region())
 	}
 	return resp.InstanceTypes[0], nil
 }
@@ -2434,7 +2456,9 @@ func GetInstanceCertificateNames(instances *ec2.DescribeInstancesOutput, useInst
 		addrs = append(addrs, *instance.InstanceId)
 	}
 
-	addrs = append(addrs, *instance.PrivateDnsName)
+	if instance.PrivateDnsName != nil {
+		addrs = append(addrs, *instance.PrivateDnsName)
+	}
 
 	// We only use data for the first interface, and only the first IP
 	for _, iface := range instance.NetworkInterfaces {
@@ -2444,7 +2468,9 @@ func GetInstanceCertificateNames(instances *ec2.DescribeInstancesOutput, useInst
 		if *iface.Attachment.DeviceIndex != 0 {
 			continue
 		}
-		addrs = append(addrs, *iface.PrivateIpAddress)
+		if iface.PrivateIpAddress != nil {
+			addrs = append(addrs, *iface.PrivateIpAddress)
+		}
 		if iface.Ipv6Addresses != nil && len(iface.Ipv6Addresses) > 0 {
 			addrs = append(addrs, *iface.Ipv6Addresses[0].Ipv6Address)
 		}
