@@ -58,7 +58,7 @@ type BootstrapChannelBuilder struct {
 	assetBuilder  *assets.AssetBuilder
 }
 
-var _ fi.ModelBuilder = &BootstrapChannelBuilder{}
+var _ fi.CloudupModelBuilder = &BootstrapChannelBuilder{}
 
 // networkSelector is the labels set on networking addons
 //
@@ -99,7 +99,7 @@ func NewBootstrapChannelBuilder(modelContext *model.KopsModelContext,
 }
 
 // Build is responsible for adding the addons to the channel
-func (b *BootstrapChannelBuilder) Build(c *fi.ModelBuilderContext) error {
+func (b *BootstrapChannelBuilder) Build(c *fi.CloudupModelBuilderContext) error {
 	addons, serviceAccounts, err := b.buildAddons(c)
 	if err != nil {
 		return err
@@ -155,8 +155,8 @@ func (b *BootstrapChannelBuilder) Build(c *fi.ModelBuilderContext) error {
 		c.AddTask(&fitasks.ManagedFile{
 			Contents:  fi.NewBytesResource(manifestBytes),
 			Lifecycle: b.Lifecycle,
-			Location:  fi.String(manifestPath),
-			Name:      fi.String(name),
+			Location:  fi.PtrTo(manifestPath),
+			Name:      fi.PtrTo(name),
 		})
 	}
 
@@ -202,8 +202,8 @@ func (b *BootstrapChannelBuilder) Build(c *fi.ModelBuilderContext) error {
 			c.AddTask(&fitasks.ManagedFile{
 				Contents:  fi.NewBytesResource(manifestBytes),
 				Lifecycle: b.Lifecycle,
-				Location:  fi.String(manifestPath),
-				Name:      fi.String(name),
+				Location:  fi.PtrTo(manifestPath),
+				Name:      fi.PtrTo(name),
 			})
 
 			addon := addons.Add(&a.Spec)
@@ -235,9 +235,9 @@ func (b *BootstrapChannelBuilder) Build(c *fi.ModelBuilderContext) error {
 		location := key + "/default.yaml"
 
 		a := &channelsapi.AddonSpec{
-			Name:     fi.String(key),
+			Name:     fi.PtrTo(key),
 			Selector: map[string]string{"k8s-addon": key},
-			Manifest: fi.String(location),
+			Manifest: fi.PtrTo(location),
 		}
 
 		name := b.Cluster.ObjectMeta.Name + "-addons-" + key
@@ -262,8 +262,8 @@ func (b *BootstrapChannelBuilder) Build(c *fi.ModelBuilderContext) error {
 		c.AddTask(&fitasks.ManagedFile{
 			Contents:  fi.NewBytesResource(manifestBytes),
 			Lifecycle: b.Lifecycle,
-			Location:  fi.String(manifestPath),
-			Name:      fi.String(name),
+			Location:  fi.PtrTo(manifestPath),
+			Name:      fi.PtrTo(name),
 		})
 
 		addons.Add(a)
@@ -294,8 +294,8 @@ func (b *BootstrapChannelBuilder) Build(c *fi.ModelBuilderContext) error {
 	c.AddTask(&fitasks.ManagedFile{
 		Contents:  fi.NewBytesResource(addonsYAML),
 		Lifecycle: b.Lifecycle,
-		Location:  fi.String("addons/bootstrap-channel.yaml"),
-		Name:      fi.String(name),
+		Location:  fi.PtrTo("addons/bootstrap-channel.yaml"),
+		Name:      fi.PtrTo(name),
 	})
 
 	return nil
@@ -324,7 +324,7 @@ type Addon struct {
 	BuildPrune bool
 }
 
-func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*AddonList, map[string]iam.Subject, error) {
+func (b *BootstrapChannelBuilder) buildAddons(c *fi.CloudupModelBuilderContext) (*AddonList, map[string]iam.Subject, error) {
 	addons := &AddonList{}
 
 	serviceAccountRoles := []iam.Subject{}
@@ -337,10 +337,10 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			id := "k8s-1.16"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:               fi.String(key),
+				Name:               fi.PtrTo(key),
 				Selector:           map[string]string{"k8s-addon": key},
-				Manifest:           fi.String(location),
-				NeedsRollingUpdate: "control-plane",
+				Manifest:           fi.PtrTo(location),
+				NeedsRollingUpdate: channelsapi.NeedsRollingUpdateControlPlane,
 				Id:                 id,
 			})
 		}
@@ -355,9 +355,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			id := "k8s-1.12"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: map[string]string{"k8s-addon": key},
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 		}
@@ -378,9 +378,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 				id := "k8s-1.12"
 
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
+					Name:     fi.PtrTo(key),
 					Selector: map[string]string{"k8s-addon": key},
-					Manifest: fi.String(location),
+					Manifest: fi.PtrTo(location),
 					Id:       id,
 				})
 			}
@@ -396,9 +396,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 				id := "k8s-1.12"
 
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
+					Name:     fi.PtrTo(key),
 					Selector: map[string]string{"k8s-addon": key},
-					Manifest: fi.String(location),
+					Manifest: fi.PtrTo(location),
 					Id:       id,
 				})
 			}
@@ -427,9 +427,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 				id := "k8s-1.8"
 
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
+					Name:     fi.PtrTo(key),
 					Selector: map[string]string{"k8s-addon": key},
-					Manifest: fi.String(location),
+					Manifest: fi.PtrTo(location),
 					Id:       id,
 				})
 			}
@@ -447,9 +447,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			id := "k8s-1.9"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: map[string]string{"k8s-addon": key},
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 		}
@@ -466,9 +466,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			id := "k8s-1.23"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: map[string]string{"k8s-addon": key},
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 		}
@@ -480,56 +480,58 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 		location := key + "/v" + version + ".yaml"
 
 		addons.Add(&channelsapi.AddonSpec{
-			Name:     fi.String(key),
+			Name:     fi.PtrTo(key),
 			Selector: map[string]string{"k8s-addon": key},
-			Manifest: fi.String(location),
+			Manifest: fi.PtrTo(location),
 		})
 	}
 
-	if b.Cluster.Spec.ExternalDNS == nil || b.Cluster.Spec.ExternalDNS.Provider == kops.ExternalDNSProviderDNSController {
-		{
-			key := "dns-controller.addons.k8s.io"
-			location := key + "/k8s-1.12.yaml"
-			id := "k8s-1.12"
-
-			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
-				Selector: map[string]string{"k8s-addon": key},
-				Manifest: fi.String(location),
-				Id:       id,
-			})
-		}
-
-		// Generate dns-controller ServiceAccount IAM permissions.
-		// Gossip clsuters do not require any cloud permissions.
-		if b.UseServiceAccountExternalPermissions() && !b.IsGossip() {
-			serviceAccountRoles = append(serviceAccountRoles, &dnscontroller.ServiceAccount{})
-		}
-	} else if b.Cluster.Spec.ExternalDNS.Provider == kops.ExternalDNSProviderExternalDNS {
-		{
-			key := "external-dns.addons.k8s.io"
-
+	if !b.Cluster.UsesNoneDNS() {
+		if b.Cluster.Spec.ExternalDNS == nil || b.Cluster.Spec.ExternalDNS.Provider == kops.ExternalDNSProviderDNSController {
 			{
-				location := key + "/k8s-1.19.yaml"
-				id := "k8s-1.19"
+				key := "dns-controller.addons.k8s.io"
+				location := key + "/k8s-1.12.yaml"
+				id := "k8s-1.12"
 
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
+					Name:     fi.PtrTo(key),
 					Selector: map[string]string{"k8s-addon": key},
-					Manifest: fi.String(location),
+					Manifest: fi.PtrTo(location),
 					Id:       id,
 				})
 			}
 
-			if b.UseServiceAccountExternalPermissions() {
-				serviceAccountRoles = append(serviceAccountRoles, &externaldns.ServiceAccount{})
+			// Generate dns-controller ServiceAccount IAM permissions.
+			// Gossip clsuters do not require any cloud permissions.
+			if b.UseServiceAccountExternalPermissions() && !b.Cluster.IsGossip() {
+				serviceAccountRoles = append(serviceAccountRoles, &dnscontroller.ServiceAccount{})
+			}
+		} else if b.Cluster.Spec.ExternalDNS.Provider == kops.ExternalDNSProviderExternalDNS {
+			{
+				key := "external-dns.addons.k8s.io"
+
+				{
+					location := key + "/k8s-1.19.yaml"
+					id := "k8s-1.19"
+
+					addons.Add(&channelsapi.AddonSpec{
+						Name:     fi.PtrTo(key),
+						Selector: map[string]string{"k8s-addon": key},
+						Manifest: fi.PtrTo(location),
+						Id:       id,
+					})
+				}
+
+				if b.UseServiceAccountExternalPermissions() {
+					serviceAccountRoles = append(serviceAccountRoles, &externaldns.ServiceAccount{})
+				}
 			}
 		}
 	}
 
 	// @check if the node-local-dns is enabled
 	NodeLocalDNS := b.Cluster.Spec.KubeDNS.NodeLocalDNS
-	if kubeDNS.Provider == "CoreDNS" && NodeLocalDNS != nil && fi.BoolValue(NodeLocalDNS.Enabled) {
+	if kubeDNS.Provider == "CoreDNS" && NodeLocalDNS != nil && fi.ValueOf(NodeLocalDNS.Enabled) {
 		{
 			key := "nodelocaldns.addons.k8s.io"
 
@@ -538,16 +540,17 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 				id := "k8s-1.12"
 
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
-					Selector: map[string]string{"k8s-addon": key},
-					Manifest: fi.String(location),
-					Id:       id,
+					Name:               fi.PtrTo(key),
+					Selector:           map[string]string{"k8s-addon": key},
+					Manifest:           fi.PtrTo(location),
+					NeedsRollingUpdate: channelsapi.NeedsRollingUpdateAll,
+					Id:                 id,
 				})
 			}
 		}
 	}
 
-	if b.Cluster.Spec.ClusterAutoscaler != nil && fi.BoolValue(b.Cluster.Spec.ClusterAutoscaler.Enabled) {
+	if b.Cluster.Spec.ClusterAutoscaler != nil && fi.ValueOf(b.Cluster.Spec.ClusterAutoscaler.Enabled) {
 		{
 			key := "cluster-autoscaler.addons.k8s.io"
 
@@ -556,9 +559,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 				id := "k8s-1.15"
 
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
+					Name:     fi.PtrTo(key),
 					Selector: map[string]string{"k8s-addon": key},
-					Manifest: fi.String(location),
+					Manifest: fi.PtrTo(location),
 					Id:       id,
 				})
 			}
@@ -570,7 +573,7 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 
 	}
 
-	if b.Cluster.Spec.MetricsServer != nil && fi.BoolValue(b.Cluster.Spec.MetricsServer.Enabled) {
+	if b.Cluster.Spec.MetricsServer != nil && fi.ValueOf(b.Cluster.Spec.MetricsServer.Enabled) {
 		{
 			key := "metrics-server.addons.k8s.io"
 
@@ -579,17 +582,17 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 				id := "k8s-1.11"
 
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
+					Name:     fi.PtrTo(key),
 					Selector: map[string]string{"k8s-app": "metrics-server"},
-					Manifest: fi.String(location),
+					Manifest: fi.PtrTo(location),
 					Id:       id,
-					NeedsPKI: !fi.BoolValue(b.Cluster.Spec.MetricsServer.Insecure),
+					NeedsPKI: !fi.ValueOf(b.Cluster.Spec.MetricsServer.Insecure),
 				})
 			}
 		}
 	}
 
-	if b.Cluster.Spec.CertManager != nil && fi.BoolValue(b.Cluster.Spec.CertManager.Enabled) && (b.Cluster.Spec.CertManager.Managed == nil || fi.BoolValue(b.Cluster.Spec.CertManager.Managed)) {
+	if b.Cluster.Spec.CertManager != nil && fi.ValueOf(b.Cluster.Spec.CertManager.Enabled) && (b.Cluster.Spec.CertManager.Managed == nil || fi.ValueOf(b.Cluster.Spec.CertManager.Managed)) {
 		{
 			key := "certmanager.io"
 
@@ -597,11 +600,12 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 				location := key + "/k8s-1.16.yaml"
 				id := "k8s-1.16"
 
-				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
-					Manifest: fi.String(location),
+				addon := addons.Add(&channelsapi.AddonSpec{
+					Name:     fi.PtrTo(key),
+					Manifest: fi.PtrTo(location),
 					Id:       id,
 				})
+				addon.BuildPrune = true
 			}
 		}
 
@@ -610,33 +614,35 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 		}
 	}
 
-	nth := b.Cluster.Spec.NodeTerminationHandler
+	if b.Cluster.Spec.CloudProvider.AWS != nil {
+		nth := b.Cluster.Spec.CloudProvider.AWS.NodeTerminationHandler
 
-	if nth != nil && fi.BoolValue(nth.Enabled) {
+		if nth != nil && fi.ValueOf(nth.Enabled) {
 
-		key := "node-termination-handler.aws"
+			key := "node-termination-handler.aws"
 
-		{
-			location := key + "/k8s-1.11.yaml"
-			id := "k8s-1.11"
+			{
+				location := key + "/k8s-1.11.yaml"
+				id := "k8s-1.11"
 
-			addon := addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
-				Selector: map[string]string{"k8s-addon": key},
-				Manifest: fi.String(location),
-				Id:       id,
-			})
-			addon.BuildPrune = true
-		}
+				addon := addons.Add(&channelsapi.AddonSpec{
+					Name:     fi.PtrTo(key),
+					Selector: map[string]string{"k8s-addon": key},
+					Manifest: fi.PtrTo(location),
+					Id:       id,
+				})
+				addon.BuildPrune = true
+			}
 
-		if b.UseServiceAccountExternalPermissions() {
-			serviceAccountRoles = append(serviceAccountRoles, &nodeterminationhandler.ServiceAccount{})
+			if b.UseServiceAccountExternalPermissions() {
+				serviceAccountRoles = append(serviceAccountRoles, &nodeterminationhandler.ServiceAccount{})
+			}
 		}
 	}
 
 	npd := b.Cluster.Spec.NodeProblemDetector
 
-	if npd != nil && fi.BoolValue(npd.Enabled) {
+	if npd != nil && fi.ValueOf(npd.Enabled) {
 
 		key := "node-problem-detector.addons.k8s.io"
 
@@ -645,9 +651,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			id := "k8s-1.17"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: map[string]string{"k8s-addon": key},
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 		}
@@ -656,13 +662,13 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 	nvidia := b.Cluster.Spec.Containerd.NvidiaGPU
 	igNvidia := false
 	for _, ig := range b.KopsModelContext.InstanceGroups {
-		if ig.Spec.Containerd != nil && ig.Spec.Containerd.NvidiaGPU != nil && fi.BoolValue(ig.Spec.Containerd.NvidiaGPU.Enabled) {
+		if ig.Spec.Containerd != nil && ig.Spec.Containerd.NvidiaGPU != nil && fi.ValueOf(ig.Spec.Containerd.NvidiaGPU.Enabled) {
 			igNvidia = true
 			break
 		}
 	}
 
-	if nvidia != nil && fi.BoolValue(nvidia.Enabled) || igNvidia {
+	if nvidia != nil && fi.ValueOf(nvidia.Enabled) || igNvidia {
 
 		key := "nvidia.addons.k8s.io"
 
@@ -671,56 +677,58 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			id := "k8s-1.16"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: map[string]string{"k8s-addon": key},
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 		}
 	}
 
-	if b.Cluster.Spec.AWSLoadBalancerController != nil && fi.BoolValue(b.Cluster.Spec.AWSLoadBalancerController.Enabled) {
+	if b.Cluster.Spec.CloudProvider.AWS != nil {
+		if b.Cluster.Spec.CloudProvider.AWS.LoadBalancerController != nil && fi.ValueOf(b.Cluster.Spec.CloudProvider.AWS.LoadBalancerController.Enabled) {
 
-		key := "aws-load-balancer-controller.addons.k8s.io"
+			key := "aws-load-balancer-controller.addons.k8s.io"
 
-		location := key + "/k8s-1.19.yaml"
-		id := "k8s-1.19"
-
-		addons.Add(&channelsapi.AddonSpec{
-			Name:     fi.String(key),
-			Selector: map[string]string{"k8s-addon": key},
-			Manifest: fi.String(location),
-			Id:       id,
-			NeedsPKI: true,
-		})
-
-		// Generate aws-load-balancer-controller ServiceAccount IAM permissions
-		if b.UseServiceAccountExternalPermissions() {
-			serviceAccountRoles = append(serviceAccountRoles, &awsloadbalancercontroller.ServiceAccount{})
-		}
-	}
-
-	if b.Cluster.Spec.PodIdentityWebhook != nil && fi.BoolValue(&b.Cluster.Spec.PodIdentityWebhook.Enabled) {
-
-		key := "eks-pod-identity-webhook.addons.k8s.io"
-
-		{
-			id := "k8s-1.16"
-			location := key + "/" + id + ".yaml"
+			location := key + "/k8s-1.19.yaml"
+			id := "k8s-1.19"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name: fi.String(key),
-				Selector: map[string]string{
-					"k8s-addon": key,
-				},
-				Manifest: fi.String(location),
+				Name:     fi.PtrTo(key),
+				Selector: map[string]string{"k8s-addon": key},
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 				NeedsPKI: true,
 			})
+
+			// Generate aws-load-balancer-controller ServiceAccount IAM permissions
+			if b.UseServiceAccountExternalPermissions() {
+				serviceAccountRoles = append(serviceAccountRoles, &awsloadbalancercontroller.ServiceAccount{})
+			}
+		}
+
+		if b.Cluster.Spec.CloudProvider.AWS.PodIdentityWebhook != nil && fi.ValueOf(&b.Cluster.Spec.CloudProvider.AWS.PodIdentityWebhook.Enabled) {
+
+			key := "eks-pod-identity-webhook.addons.k8s.io"
+
+			{
+				id := "k8s-1.16"
+				location := key + "/" + id + ".yaml"
+
+				addons.Add(&channelsapi.AddonSpec{
+					Name: fi.PtrTo(key),
+					Selector: map[string]string{
+						"k8s-addon": key,
+					},
+					Manifest: fi.PtrTo(location),
+					Id:       id,
+					NeedsPKI: true,
+				})
+			}
 		}
 	}
 
-	if fi.BoolValue(b.Cluster.Spec.CloudConfig.ManageStorageClasses) {
+	if fi.ValueOf(b.Cluster.Spec.CloudConfig.ManageStorageClasses) {
 		if b.Cluster.Spec.GetCloudProvider() == kops.CloudProviderAWS {
 			key := "storage-aws.addons.k8s.io"
 
@@ -729,14 +737,13 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 				location := key + "/" + id + ".yaml"
 
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
+					Name:     fi.PtrTo(key),
 					Selector: map[string]string{"k8s-addon": key},
-					Manifest: fi.String(location),
+					Manifest: fi.PtrTo(location),
 					Id:       id,
 				})
 			}
 		}
-
 	}
 
 	if b.Cluster.Spec.GetCloudProvider() == kops.CloudProviderDO {
@@ -747,9 +754,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: map[string]string{"k8s-addon": key},
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 		}
@@ -761,9 +768,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: map[string]string{"k8s-addon": key},
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 		}
@@ -776,9 +783,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: map[string]string{"k8s-addon": key},
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 		}
@@ -788,16 +795,16 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: map[string]string{"k8s-addon": key},
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 		}
 	}
 
 	if b.Cluster.Spec.GetCloudProvider() == kops.CloudProviderGCE {
-		if fi.BoolValue(b.Cluster.Spec.CloudConfig.ManageStorageClasses) {
+		if fi.ValueOf(b.Cluster.Spec.CloudConfig.ManageStorageClasses) {
 			key := "storage-gce.addons.k8s.io"
 
 			{
@@ -805,21 +812,21 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 				location := key + "/" + id + ".yaml"
 
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
+					Name:     fi.PtrTo(key),
 					Selector: map[string]string{"k8s-addon": key},
-					Manifest: fi.String(location),
+					Manifest: fi.PtrTo(location),
 					Id:       id,
 				})
 			}
 		}
-		if b.Cluster.Spec.CloudConfig != nil && b.Cluster.Spec.CloudConfig.GCPPDCSIDriver != nil && fi.BoolValue(b.Cluster.Spec.CloudConfig.GCPPDCSIDriver.Enabled) {
+		if b.Cluster.Spec.CloudProvider.GCE != nil && b.Cluster.Spec.CloudProvider.GCE.PDCSIDriver != nil && fi.ValueOf(b.Cluster.Spec.CloudProvider.GCE.PDCSIDriver.Enabled) {
 			key := "gcp-pd-csi-driver.addons.k8s.io"
 			{
 				id := "k8s-1.23"
 				location := key + "/" + id + ".yaml"
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
-					Manifest: fi.String(location),
+					Name:     fi.PtrTo(key),
+					Manifest: fi.PtrTo(location),
 					Selector: map[string]string{"k8s-addon": key},
 					Id:       id,
 				})
@@ -835,9 +842,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: map[string]string{"k8s-addon": key},
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 		}
@@ -853,9 +860,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: map[string]string{"k8s-addon": key},
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 		}
@@ -867,8 +874,8 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 					id := "k8s-1.23"
 					location := key + "/" + id + ".yaml"
 					addon := addons.Add(&channelsapi.AddonSpec{
-						Name:     fi.String(key),
-						Manifest: fi.String(location),
+						Name:     fi.PtrTo(key),
+						Manifest: fi.PtrTo(location),
 						Selector: map[string]string{"k8s-addon": key},
 						Id:       id,
 					})
@@ -886,9 +893,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			id := "k8s-1.12"
 
 			addon := addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: networkingSelector(),
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 
@@ -904,9 +911,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			id := "k8s-1.12"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: networkingSelector(),
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 		}
@@ -920,9 +927,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addon := addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: networkingSelector(),
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 			addon.BuildPrune = true
@@ -931,9 +938,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addon := addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: networkingSelector(),
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 			addon.BuildPrune = true
@@ -948,9 +955,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addon := addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: networkingSelector(),
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 			addon.BuildPrune = true
@@ -959,9 +966,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addon := addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: networkingSelector(),
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 			addon.BuildPrune = true
@@ -970,9 +977,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addon := addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: networkingSelector(),
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 			addon.BuildPrune = true
@@ -987,9 +994,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addon := addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: networkingSelector(),
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 			addon.BuildPrune = true
@@ -998,9 +1005,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addon := addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: networkingSelector(),
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 			addon.BuildPrune = true
@@ -1009,16 +1016,16 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addon := addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: networkingSelector(),
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 			addon.BuildPrune = true
 		}
 	}
 
-	if b.Cluster.Spec.Networking.Kuberouter != nil {
+	if b.Cluster.Spec.Networking.KubeRouter != nil {
 		key := "networking.kuberouter"
 
 		{
@@ -1026,9 +1033,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			id := "k8s-1.12"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
+				Name:     fi.PtrTo(key),
 				Selector: networkingSelector(),
-				Manifest: fi.String(location),
+				Manifest: fi.PtrTo(location),
 				Id:       id,
 			})
 		}
@@ -1047,11 +1054,11 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addons.Add(&channelsapi.AddonSpec{
-				Name:               fi.String(key),
+				Name:               fi.PtrTo(key),
 				Selector:           networkingSelector(),
-				Manifest:           fi.String(location),
+				Manifest:           fi.PtrTo(location),
 				Id:                 id,
-				NeedsRollingUpdate: "all",
+				NeedsRollingUpdate: channelsapi.NeedsRollingUpdateAll,
 			})
 		}
 	}
@@ -1072,9 +1079,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 				id := "k8s-1.12"
 
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
+					Name:     fi.PtrTo(key),
 					Selector: authenticationSelector,
-					Manifest: fi.String(location),
+					Manifest: fi.PtrTo(location),
 					Id:       id,
 				})
 			}
@@ -1087,9 +1094,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 				id := "k8s-1.12"
 
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
+					Name:     fi.PtrTo(key),
 					Selector: authenticationSelector,
-					Manifest: fi.String(location),
+					Manifest: fi.PtrTo(location),
 					Id:       id,
 				})
 			}
@@ -1104,8 +1111,8 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 			location := key + "/" + id + ".yaml"
 
 			addon := addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
-				Manifest: fi.String(location),
+				Name:     fi.PtrTo(key),
+				Manifest: fi.PtrTo(location),
 				Selector: map[string]string{"k8s-addon": key},
 				Id:       id,
 			})
@@ -1121,8 +1128,8 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 				id := "k8s-1.13-ccm"
 
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
-					Manifest: fi.String(location),
+					Name:     fi.PtrTo(key),
+					Manifest: fi.PtrTo(location),
 					Selector: map[string]string{"k8s-addon": key},
 					Id:       id,
 				})
@@ -1139,8 +1146,8 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 				id := "k8s-1.18"
 				location := key + "/" + id + ".yaml"
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
-					Manifest: fi.String(location),
+					Name:     fi.PtrTo(key),
+					Manifest: fi.PtrTo(location),
 					Selector: map[string]string{"k8s-addon": key},
 					Id:       id,
 				})
@@ -1149,17 +1156,17 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 				serviceAccountRoles = append(serviceAccountRoles, &awscloudcontrollermanager.ServiceAccount{})
 			}
 		}
-		if b.Cluster.Spec.CloudConfig != nil && b.Cluster.Spec.CloudConfig.AWSEBSCSIDriver != nil &&
-			fi.BoolValue(b.Cluster.Spec.CloudConfig.AWSEBSCSIDriver.Enabled) &&
-			(b.Cluster.Spec.CloudConfig.AWSEBSCSIDriver.Managed == nil || fi.BoolValue(b.Cluster.Spec.CloudConfig.AWSEBSCSIDriver.Managed)) {
+		if b.Cluster.Spec.CloudProvider.AWS != nil && b.Cluster.Spec.CloudProvider.AWS.EBSCSIDriver != nil &&
+			fi.ValueOf(b.Cluster.Spec.CloudProvider.AWS.EBSCSIDriver.Enabled) &&
+			(b.Cluster.Spec.CloudProvider.AWS.EBSCSIDriver.Managed == nil || fi.ValueOf(b.Cluster.Spec.CloudProvider.AWS.EBSCSIDriver.Managed)) {
 			key := "aws-ebs-csi-driver.addons.k8s.io"
 
 			{
 				id := "k8s-1.17"
 				location := key + "/" + id + ".yaml"
 				addons.Add(&channelsapi.AddonSpec{
-					Name:     fi.String(key),
-					Manifest: fi.String(location),
+					Name:     fi.PtrTo(key),
+					Manifest: fi.PtrTo(location),
 					Selector: map[string]string{"k8s-addon": key},
 					Id:       id,
 				})
@@ -1170,30 +1177,30 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 		}
 	}
 
-	if b.Cluster.Spec.SnapshotController != nil && fi.BoolValue(b.Cluster.Spec.SnapshotController.Enabled) {
+	if b.Cluster.Spec.SnapshotController != nil && fi.ValueOf(b.Cluster.Spec.SnapshotController.Enabled) {
 		key := "snapshot-controller.addons.k8s.io"
 
 		{
 			id := "k8s-1.20"
 			location := key + "/" + id + ".yaml"
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
-				Manifest: fi.String(location),
+				Name:     fi.PtrTo(key),
+				Manifest: fi.PtrTo(location),
 				Selector: map[string]string{"k8s-addon": key},
 				NeedsPKI: true,
 				Id:       id,
 			})
 		}
 	}
-	if b.Cluster.Spec.Karpenter != nil && fi.BoolValue(&b.Cluster.Spec.Karpenter.Enabled) {
+	if b.Cluster.Spec.Karpenter != nil && fi.ValueOf(&b.Cluster.Spec.Karpenter.Enabled) {
 		key := "karpenter.sh"
 
 		{
 			id := "k8s-1.19"
 			location := key + "/" + id + ".yaml"
 			addons.Add(&channelsapi.AddonSpec{
-				Name:     fi.String(key),
-				Manifest: fi.String(location),
+				Name:     fi.PtrTo(key),
+				Manifest: fi.PtrTo(location),
 				Selector: map[string]string{"k8s-addon": key},
 				Id:       id,
 			})
@@ -1209,9 +1216,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*Addon
 		location := key + "/v" + version + ".yaml"
 
 		addons.Add(&channelsapi.AddonSpec{
-			Name:     fi.String(key),
+			Name:     fi.PtrTo(key),
 			Selector: map[string]string{"k8s-addon": key},
-			Manifest: fi.String(location),
+			Manifest: fi.PtrTo(location),
 		})
 	}
 
