@@ -42,7 +42,6 @@ import (
 const (
 	TagKubernetesClusterIndex        = "k8s-index"
 	TagNameEtcdClusterPrefix         = "etcdCluster-"
-	TagNameRolePrefix                = "k8s.io/role/"
 	TagKubernetesClusterNamePrefix   = "KubernetesCluster"
 	TagKubernetesClusterMasterPrefix = "KubernetesCluster-Master"
 	TagKubernetesInstanceGroup       = "kops-instancegroup"
@@ -282,28 +281,26 @@ func (c *doCloudImplementation) GetVPCUUID(networkCIDR string, vpcName string) (
 func (c *doCloudImplementation) GetApiIngressStatus(cluster *kops.Cluster) ([]fi.ApiIngressStatus, error) {
 	var ingresses []fi.ApiIngressStatus
 	done, err := vfs.RetryWithBackoff(readBackoff, func() (bool, error) {
-		if cluster.Spec.MasterPublicName != "" {
-			// Note that this must match Digital Ocean's lb name
-			klog.V(2).Infof("Querying DO to find Loadbalancers for API (%q)", cluster.Name)
+		// Note that this must match Digital Ocean's lb name
+		klog.V(2).Infof("Querying DO to find Loadbalancers for API (%q)", cluster.Name)
 
-			loadBalancers, err := c.GetAllLoadBalancers()
-			if err != nil {
-				return false, fmt.Errorf("LoadBalancers.List returned error: %v", err)
-			}
+		loadBalancers, err := c.GetAllLoadBalancers()
+		if err != nil {
+			return false, fmt.Errorf("LoadBalancers.List returned error: %v", err)
+		}
 
-			lbName := "api-" + strings.Replace(cluster.Name, ".", "-", -1)
+		lbName := "api-" + strings.Replace(cluster.Name, ".", "-", -1)
 
-			for _, lb := range loadBalancers {
-				if lb.Name == lbName {
-					klog.V(10).Infof("Matching LB name found for API (%q)", cluster.Name)
+		for _, lb := range loadBalancers {
+			if lb.Name == lbName {
+				klog.V(10).Infof("Matching LB name found for API (%q)", cluster.Name)
 
-					if lb.Status != "active" {
-						return false, fmt.Errorf("load-balancer is not yet active (current status: %s)", lb.Status)
-					}
-
-					address := lb.IP
-					ingresses = append(ingresses, fi.ApiIngressStatus{IP: address})
+				if lb.Status != "active" {
+					return false, fmt.Errorf("load-balancer is not yet active (current status: %s)", lb.Status)
 				}
+
+				address := lb.IP
+				ingresses = append(ingresses, fi.ApiIngressStatus{IP: address})
 			}
 		}
 		return true, nil
@@ -501,7 +498,7 @@ func matchInstanceGroup(name string, clusterName string, instancegroups []*kops.
 		var groupName string
 
 		switch g.Spec.Role {
-		case kops.InstanceGroupRoleMaster, kops.InstanceGroupRoleNode:
+		case kops.InstanceGroupRoleControlPlane, kops.InstanceGroupRoleNode:
 			groupName = clusterName + "-" + g.ObjectMeta.Name
 		default:
 			klog.Warningf("Ignoring InstanceGroup of unknown role %q", g.Spec.Role)
@@ -524,9 +521,9 @@ func buildCloudInstanceGroup(c *doCloudImplementation, ig *kops.InstanceGroup, g
 		HumanName:     g.InstanceGroupName,
 		InstanceGroup: ig,
 		Raw:           g,
-		MinSize:       int(fi.Int32Value(ig.Spec.MinSize)),
-		TargetSize:    int(fi.Int32Value(ig.Spec.MinSize)),
-		MaxSize:       int(fi.Int32Value(ig.Spec.MaxSize)),
+		MinSize:       int(fi.ValueOf(ig.Spec.MinSize)),
+		TargetSize:    int(fi.ValueOf(ig.Spec.MinSize)),
+		MaxSize:       int(fi.ValueOf(ig.Spec.MaxSize)),
 	}
 
 	for _, member := range g.Members {
