@@ -47,7 +47,7 @@ type LoadBalancer struct {
 var _ fi.CompareWithID = &LoadBalancer{}
 
 func (v *LoadBalancer) CompareWithID() *string {
-	return fi.String(strconv.Itoa(fi.IntValue(v.ID)))
+	return fi.PtrTo(strconv.Itoa(fi.ValueOf(v.ID)))
 }
 
 var _ fi.HasAddress = &LoadBalancer{}
@@ -56,14 +56,14 @@ func (e *LoadBalancer) IsForAPIServer() bool {
 	return true
 }
 
-func (v *LoadBalancer) FindAddresses(c *fi.Context) ([]string, error) {
+func (v *LoadBalancer) FindAddresses(c *fi.CloudupContext) ([]string, error) {
 	// TODO(hakman): Use mock to handle this more gracefully
-	if strings.HasPrefix(c.ClusterConfigBase.Path(), "memfs://tests/") {
+	if strings.HasPrefix(c.T.ClusterConfigBase.Path(), "memfs://tests/") {
 		return nil, nil
 	}
 
 	ctx := context.TODO()
-	cloud := c.Cloud.(hetzner.HetznerCloud)
+	cloud := c.T.Cloud.(hetzner.HetznerCloud)
 	client := cloud.LoadBalancerClient()
 
 	// TODO(hakman): Find using label selector
@@ -73,15 +73,15 @@ func (v *LoadBalancer) FindAddresses(c *fi.Context) ([]string, error) {
 	}
 
 	for _, loadbalancer := range loadbalancers {
-		if loadbalancer.Name == fi.StringValue(v.Name) {
+		if loadbalancer.Name == fi.ValueOf(v.Name) {
 			var addresses []string
 			if loadbalancer.PublicNet.IPv4.IP == nil {
-				return nil, fmt.Errorf("failed to find load-balancer %q public address", fi.StringValue(v.Name))
+				return nil, fmt.Errorf("failed to find load-balancer %q public address", fi.ValueOf(v.Name))
 			}
 			addresses = append(addresses, loadbalancer.PublicNet.IPv4.IP.String())
 			for _, privateNetwork := range loadbalancer.PrivateNet {
 				if privateNetwork.IP == nil {
-					return nil, fmt.Errorf("failed to find load-balancer %q private address", fi.StringValue(v.Name))
+					return nil, fmt.Errorf("failed to find load-balancer %q private address", fi.ValueOf(v.Name))
 				}
 				addresses = append(addresses, privateNetwork.IP.String())
 			}
@@ -92,9 +92,9 @@ func (v *LoadBalancer) FindAddresses(c *fi.Context) ([]string, error) {
 	return nil, nil
 }
 
-func (v *LoadBalancer) Find(c *fi.Context) (*LoadBalancer, error) {
+func (v *LoadBalancer) Find(c *fi.CloudupContext) (*LoadBalancer, error) {
 	ctx := context.TODO()
-	cloud := c.Cloud.(hetzner.HetznerCloud)
+	cloud := c.T.Cloud.(hetzner.HetznerCloud)
 	client := cloud.LoadBalancerClient()
 
 	// TODO(hakman): Find using label selector
@@ -104,11 +104,11 @@ func (v *LoadBalancer) Find(c *fi.Context) (*LoadBalancer, error) {
 	}
 
 	for _, loadbalancer := range loadbalancers {
-		if loadbalancer.Name == fi.StringValue(v.Name) {
+		if loadbalancer.Name == fi.ValueOf(v.Name) {
 			matches := &LoadBalancer{
 				Lifecycle: v.Lifecycle,
-				Name:      fi.String(loadbalancer.Name),
-				ID:        fi.Int(loadbalancer.ID),
+				Name:      fi.PtrTo(loadbalancer.Name),
+				ID:        fi.PtrTo(loadbalancer.ID),
 				Labels:    loadbalancer.Labels,
 			}
 
@@ -122,8 +122,8 @@ func (v *LoadBalancer) Find(c *fi.Context) (*LoadBalancer, error) {
 			for _, service := range loadbalancer.Services {
 				loadbalancerService := LoadBalancerService{
 					Protocol:        string(service.Protocol),
-					ListenerPort:    fi.Int(service.ListenPort),
-					DestinationPort: fi.Int(service.DestinationPort),
+					ListenerPort:    fi.PtrTo(service.ListenPort),
+					DestinationPort: fi.PtrTo(service.DestinationPort),
 				}
 				matches.Services = append(matches.Services, &loadbalancerService)
 			}
@@ -145,8 +145,8 @@ func (v *LoadBalancer) Find(c *fi.Context) (*LoadBalancer, error) {
 	return nil, nil
 }
 
-func (v *LoadBalancer) Run(c *fi.Context) error {
-	return fi.DefaultDeltaRunMethod(v, c)
+func (v *LoadBalancer) Run(c *fi.CloudupContext) error {
+	return fi.CloudupDefaultDeltaRunMethod(v, c)
 }
 
 func (_ *LoadBalancer) CheckChanges(a, e, changes *LoadBalancer) error {
@@ -196,16 +196,16 @@ func (_ *LoadBalancer) RenderHetzner(t *hetzner.HetznerAPITarget, a, e, changes
 
 	if a == nil {
 		if e.Network == nil {
-			return fmt.Errorf("failed to find network for loadbalancer %q", fi.StringValue(e.Name))
+			return fmt.Errorf("failed to find network for loadbalancer %q", fi.ValueOf(e.Name))
 		}
 
-		networkID, err := strconv.Atoi(fi.StringValue(e.Network.ID))
+		networkID, err := strconv.Atoi(fi.ValueOf(e.Network.ID))
 		if err != nil {
-			return fmt.Errorf("failed to convert network ID %q to int: %w", fi.StringValue(e.Network.ID), err)
+			return fmt.Errorf("failed to convert network ID %q to int: %w", fi.ValueOf(e.Network.ID), err)
 		}
 
 		opts := hcloud.LoadBalancerCreateOpts{
-			Name: fi.StringValue(e.Name),
+			Name: fi.ValueOf(e.Name),
 			LoadBalancerType: &hcloud.LoadBalancerType{
 				Name: e.Type,
 			},
@@ -222,7 +222,7 @@ func (_ *LoadBalancer) RenderHetzner(t *hetzner.HetznerAPITarget, a, e, changes
 					LabelSelector: hcloud.LoadBalancerCreateOptsTargetLabelSelector{
 						Selector: e.Target,
 					},
-					UsePrivateIP: fi.Bool(true),
+					UsePrivateIP: fi.PtrTo(true),
 				},
 			},
 			Network: &hcloud.Network{
@@ -249,7 +249,7 @@ func (_ *LoadBalancer) RenderHetzner(t *hetzner.HetznerAPITarget, a, e, changes
 
 	} else {
 		var err error
-		loadbalancer, _, err := client.Get(ctx, strconv.Itoa(fi.IntValue(a.ID)))
+		loadbalancer, _, err := client.Get(ctx, strconv.Itoa(fi.ValueOf(a.ID)))
 		if err != nil {
 			return err
 		}
@@ -257,7 +257,7 @@ func (_ *LoadBalancer) RenderHetzner(t *hetzner.HetznerAPITarget, a, e, changes
 		// Update the labels
 		if changes.Name != nil || len(changes.Labels) != 0 {
 			_, _, err := client.Update(ctx, loadbalancer, hcloud.LoadBalancerUpdateOpts{
-				Name:   fi.StringValue(e.Name),
+				Name:   fi.ValueOf(e.Name),
 				Labels: e.Labels,
 			})
 			if err != nil {
@@ -289,7 +289,7 @@ func (_ *LoadBalancer) RenderHetzner(t *hetzner.HetznerAPITarget, a, e, changes
 		if a.Target == "" {
 			action, _, err := client.AddLabelSelectorTarget(ctx, loadbalancer, hcloud.LoadBalancerAddLabelSelectorTargetOpts{
 				Selector:     e.Target,
-				UsePrivateIP: fi.Bool(true),
+				UsePrivateIP: fi.PtrTo(true),
 			})
 			if err != nil {
 				return err
@@ -311,9 +311,9 @@ type LoadBalancerService struct {
 	DestinationPort *int
 }
 
-var _ fi.HasDependencies = &LoadBalancerService{}
+var _ fi.CloudupHasDependencies = &LoadBalancerService{}
 
-func (e *LoadBalancerService) GetDependencies(tasks map[string]fi.Task) []fi.Task {
+func (e *LoadBalancerService) GetDependencies(tasks map[string]fi.CloudupTask) []fi.CloudupTask {
 	return nil
 }
 
@@ -375,12 +375,12 @@ func (_ *LoadBalancer) RenderTerraform(t *terraform.TerraformTarget, a, e, chang
 	for _, service := range e.Services {
 		tf := &terraformLoadBalancerService{
 			LoadBalancerID:  e.TerraformLink(),
-			Protocol:        fi.String(service.Protocol),
+			Protocol:        fi.PtrTo(service.Protocol),
 			ListenPort:      service.ListenerPort,
 			DestinationPort: service.DestinationPort,
 		}
 
-		err := t.RenderResource("hcloud_load_balancer_service", *e.Name, tf)
+		err := t.RenderResource("hcloud_load_balancer_service", fmt.Sprintf("%s-%s-%d", *e.Name, service.Protocol, *service.ListenerPort), tf)
 		if err != nil {
 			return err
 		}
@@ -389,9 +389,9 @@ func (_ *LoadBalancer) RenderTerraform(t *terraform.TerraformTarget, a, e, chang
 	{
 		tf := &terraformLoadBalancerTarget{
 			LoadBalancerID: e.TerraformLink(),
-			Type:           fi.String(string(hcloud.LoadBalancerTargetTypeLabelSelector)),
-			LabelSelector:  fi.String(e.Target),
-			UsePrivateIP:   fi.Bool(true),
+			Type:           fi.PtrTo(string(hcloud.LoadBalancerTargetTypeLabelSelector)),
+			LabelSelector:  fi.PtrTo(e.Target),
+			UsePrivateIP:   fi.PtrTo(true),
 		}
 
 		err := t.RenderResource("hcloud_load_balancer_target", *e.Name, tf)
