@@ -39,11 +39,12 @@ type Port struct {
 	AdditionalSecurityGroups []string
 	Lifecycle                fi.Lifecycle
 	Tags                     []string
+	ForAPIServer             bool
 }
 
 // GetDependencies returns the dependencies of the Port task
-func (e *Port) GetDependencies(tasks map[string]fi.Task) []fi.Task {
-	var deps []fi.Task
+func (e *Port) GetDependencies(tasks map[string]fi.CloudupTask) []fi.CloudupTask {
+	var deps []fi.CloudupTask
 	for _, task := range tasks {
 		if _, ok := task.(*Subnet); ok {
 			deps = append(deps, task)
@@ -64,6 +65,26 @@ func (s *Port) CompareWithID() *string {
 	return s.ID
 }
 
+func (s *Port) FindAddresses(context *fi.CloudupContext) ([]string, error) {
+	cloud := context.T.Cloud.(openstack.OpenstackCloud)
+	if s.ID == nil {
+		return nil, nil
+	}
+	port, err := cloud.GetPort(fi.ValueOf(s.ID))
+	if err != nil {
+		return nil, err
+	}
+	addrs := []string{}
+	for _, addr := range port.FixedIPs {
+		addrs = append(addrs, addr.IPAddress)
+	}
+	return addrs, nil
+}
+
+func (s *Port) IsForAPIServer() bool {
+	return s.ForAPIServer
+}
+
 func newPortTaskFromCloud(cloud openstack.OpenstackCloud, lifecycle fi.Lifecycle, port *ports.Port, find *Port) (*Port, error) {
 	additionalSecurityGroupIDs := map[string]struct{}{}
 	if find != nil {
@@ -87,7 +108,7 @@ func newPortTaskFromCloud(cloud openstack.OpenstackCloud, lifecycle fi.Lifecycle
 			continue
 		}
 		sgs = append(sgs, &SecurityGroup{
-			ID:        fi.String(sgid),
+			ID:        fi.PtrTo(sgid),
 			Lifecycle: lifecycle,
 		})
 	}
@@ -98,7 +119,7 @@ func newPortTaskFromCloud(cloud openstack.OpenstackCloud, lifecycle fi.Lifecycle
 	subnets := make([]*Subnet, len(port.FixedIPs))
 	for i, subn := range port.FixedIPs {
 		subnets[i] = &Subnet{
-			ID:        fi.String(subn.SubnetID),
+			ID:        fi.PtrTo(subn.SubnetID),
 			Lifecycle: lifecycle,
 		}
 	}
@@ -121,7 +142,7 @@ func newPortTaskFromCloud(cloud openstack.OpenstackCloud, lifecycle fi.Lifecycle
 		if !strings.HasPrefix(t, prefix) {
 			continue
 		}
-		cloudInstanceGroupName = fi.String("")
+		cloudInstanceGroupName = fi.PtrTo("")
 		scanString := fmt.Sprintf("%s%%s", prefix)
 		if _, err := fmt.Sscanf(t, scanString, cloudInstanceGroupName); err != nil {
 			klog.V(2).Infof("Error extracting instance group for Port with name: %q", port.Name)
@@ -129,10 +150,10 @@ func newPortTaskFromCloud(cloud openstack.OpenstackCloud, lifecycle fi.Lifecycle
 	}
 
 	actual := &Port{
-		ID:                fi.String(port.ID),
+		ID:                fi.PtrTo(port.ID),
 		InstanceGroupName: cloudInstanceGroupName,
-		Name:              fi.String(port.Name),
-		Network:           &Network{ID: fi.String(port.NetworkID)},
+		Name:              fi.PtrTo(port.Name),
+		Network:           &Network{ID: fi.PtrTo(port.NetworkID)},
 		SecurityGroups:    sgs,
 		Subnets:           subnets,
 		Lifecycle:         lifecycle,
@@ -142,14 +163,15 @@ func newPortTaskFromCloud(cloud openstack.OpenstackCloud, lifecycle fi.Lifecycle
 		find.ID = actual.ID
 		actual.InstanceGroupName = find.InstanceGroupName
 		actual.AdditionalSecurityGroups = find.AdditionalSecurityGroups
+		actual.ForAPIServer = find.ForAPIServer
 	}
 	return actual, nil
 }
 
-func (s *Port) Find(context *fi.Context) (*Port, error) {
-	cloud := context.Cloud.(openstack.OpenstackCloud)
+func (s *Port) Find(context *fi.CloudupContext) (*Port, error) {
+	cloud := context.T.Cloud.(openstack.OpenstackCloud)
 	opt := ports.ListOpts{
-		Name: fi.StringValue(s.Name),
+		Name: fi.ValueOf(s.Name),
 	}
 	rs, err := cloud.ListPorts(opt)
 	if err != nil {
@@ -158,7 +180,7 @@ func (s *Port) Find(context *fi.Context) (*Port, error) {
 	if rs == nil {
 		return nil, nil
 	} else if len(rs) != 1 {
-		return nil, fmt.Errorf("found multiple ports with name: %s", fi.StringValue(s.Name))
+		return nil, fmt.Errorf("found multiple ports with name: %s", fi.ValueOf(s.Name))
 	}
 
 	// sort for consistent comparison
@@ -167,8 +189,8 @@ func (s *Port) Find(context *fi.Context) (*Port, error) {
 	return newPortTaskFromCloud(cloud, s.Lifecycle, &rs[0], s)
 }
 
-func (s *Port) Run(context *fi.Context) error {
-	return fi.DefaultDeltaRunMethod(s, context)
+func (s *Port) Run(context *fi.CloudupContext) error {
+	return fi.CloudupDefaultDeltaRunMethod(s, context)
 }
 
 func (_ *Port) CheckChanges(a, e, changes *Port) error {
@@ -192,7 +214,7 @@ func (_ *Port) CheckChanges(a, e, changes *Port) error {
 
 func (*Port) RenderOpenstack(t *openstack.OpenstackAPITarget, a, e, changes *Port) error {
 	if a == nil {
-		klog.V(2).Infof("Creating Port with name: %q", fi.StringValue(e.Name))
+		klog.V(2).Infof("Creating Port with name: %q", fi.ValueOf(e.Name))
 
 		opt, err := portCreateOptsFromPortTask(t, a, e, changes)
 		if err != nil {
@@ -212,28 +234,28 @@ func (*Port) RenderOpenstack(t *openstack.OpenstackAPITarget, a, e, changes *Por
 				}
 			}
 		}
-		e.ID = fi.String(v.ID)
+		e.ID = fi.PtrTo(v.ID)
 		klog.V(2).Infof("Creating a new Openstack port, id=%s", v.ID)
 		return nil
 	}
 	if changes != nil && changes.Tags != nil {
-		klog.V(2).Infof("Updating tags for Port with name: %q", fi.StringValue(e.Name))
+		klog.V(2).Infof("Updating tags for Port with name: %q", fi.ValueOf(e.Name))
 		for _, tag := range e.Tags {
-			err := t.Cloud.AppendTag(openstack.ResourceTypePort, fi.StringValue(a.ID), tag)
+			err := t.Cloud.AppendTag(openstack.ResourceTypePort, fi.ValueOf(a.ID), tag)
 			if err != nil {
 				return fmt.Errorf("Error appending tag to port: %v", err)
 			}
 		}
 	}
 	e.ID = a.ID
-	klog.V(2).Infof("Using an existing Openstack port, id=%s", fi.StringValue(e.ID))
+	klog.V(2).Infof("Using an existing Openstack port, id=%s", fi.ValueOf(e.ID))
 	return nil
 }
 
 func portCreateOptsFromPortTask(t *openstack.OpenstackAPITarget, a, e, changes *Port) (ports.CreateOptsBuilder, error) {
 	sgs := make([]string, len(e.SecurityGroups)+len(e.AdditionalSecurityGroups))
 	for i, sg := range e.SecurityGroups {
-		sgs[i] = fi.StringValue(sg.ID)
+		sgs[i] = fi.ValueOf(sg.ID)
 	}
 	for i, sg := range e.AdditionalSecurityGroups {
 		opt := secgroup.ListOpts{
@@ -251,13 +273,13 @@ func portCreateOptsFromPortTask(t *openstack.OpenstackAPITarget, a, e, changes *
 	fixedIPs := make([]ports.IP, len(e.Subnets))
 	for i, subn := range e.Subnets {
 		fixedIPs[i] = ports.IP{
-			SubnetID: fi.StringValue(subn.ID),
+			SubnetID: fi.ValueOf(subn.ID),
 		}
 	}
 
 	return ports.CreateOpts{
-		Name:           fi.StringValue(e.Name),
-		NetworkID:      fi.StringValue(e.Network.ID),
+		Name:           fi.ValueOf(e.Name),
+		NetworkID:      fi.ValueOf(e.Network.ID),
 		SecurityGroups: &sgs,
 		FixedIPs:       fixedIPs,
 	}, nil
