@@ -20,6 +20,7 @@ import (
 	"fmt"
 	"strings"
 
+	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/klog/v2"
 
@@ -35,21 +36,25 @@ import (
 
 // Default Machine types for various types of instance group machine
 const (
-	defaultNodeMachineTypeGCE     = "n1-standard-2"
-	defaultNodeMachineTypeDO      = "s-2vcpu-4gb"
-	defaultNodeMachineTypeAzure   = "Standard_B2ms"
-	defaultNodeMachineTypeHetzner = "cx21"
+	defaultNodeMachineTypeGCE      = "e2-medium"
+	defaultNodeMachineTypeDO       = "s-2vcpu-4gb"
+	defaultNodeMachineTypeAzure    = "Standard_B2s"
+	defaultNodeMachineTypeHetzner  = "cx21"
+	defaultNodeMachineTypeScaleway = "DEV1-M"
 
-	defaultBastionMachineTypeGCE     = "f1-micro"
-	defaultBastionMachineTypeAzure   = "Standard_B2ms"
+	defaultBastionMachineTypeGCE     = "e2-micro"
+	defaultBastionMachineTypeAzure   = "Standard_B2s"
 	defaultBastionMachineTypeHetzner = "cx11"
 
-	defaultMasterMachineTypeGCE     = "n1-standard-1"
-	defaultMasterMachineTypeDO      = "s-2vcpu-4gb"
-	defaultMasterMachineTypeAzure   = "Standard_B2ms"
-	defaultMasterMachineTypeHetzner = "cx21"
+	defaultMasterMachineTypeGCE      = "e2-medium"
+	defaultMasterMachineTypeDO       = "s-2vcpu-4gb"
+	defaultMasterMachineTypeAzure    = "Standard_B2s"
+	defaultMasterMachineTypeHetzner  = "cx21"
+	defaultMasterMachineTypeScaleway = "DEV1-M"
 
-	defaultDONodeImage = "ubuntu-20-04-x64"
+	defaultDOImage       = "ubuntu-20-04-x64"
+	defaultHetznerImage  = "ubuntu-20.04"
+	defaultScalewayImage = "ubuntu_focal"
 )
 
 // TODO: this hardcoded list can be replaced with DescribeInstanceTypes' DedicatedHostsSupported field
@@ -78,19 +83,19 @@ func PopulateInstanceGroupSpec(cluster *kops.Cluster, input *kops.InstanceGroup,
 	igSpec := &ig.Spec
 
 	// TODO: Clean up
-	if ig.IsMaster() {
+	if ig.IsControlPlane() {
 		if ig.Spec.MachineType == "" {
 			ig.Spec.MachineType, err = defaultMachineType(cloud, cluster, ig)
 			if err != nil {
-				return nil, fmt.Errorf("error assigning default machine type for masters: %v", err)
+				return nil, fmt.Errorf("assigning default machine type for control-plane nodes: %v", err)
 			}
 
 		}
 		if ig.Spec.MinSize == nil {
-			ig.Spec.MinSize = fi.Int32(1)
+			ig.Spec.MinSize = fi.PtrTo(int32(1))
 		}
 		if ig.Spec.MaxSize == nil {
-			ig.Spec.MaxSize = fi.Int32(1)
+			ig.Spec.MaxSize = fi.PtrTo(int32(1))
 		}
 	} else if ig.Spec.Role == kops.InstanceGroupRoleBastion {
 		if ig.Spec.MachineType == "" {
@@ -100,10 +105,10 @@ func PopulateInstanceGroupSpec(cluster *kops.Cluster, input *kops.InstanceGroup,
 			}
 		}
 		if ig.Spec.MinSize == nil {
-			ig.Spec.MinSize = fi.Int32(1)
+			ig.Spec.MinSize = fi.PtrTo(int32(1))
 		}
 		if ig.Spec.MaxSize == nil {
-			ig.Spec.MaxSize = fi.Int32(1)
+			ig.Spec.MaxSize = fi.PtrTo(int32(1))
 		}
 	} else {
 		if ig.IsAPIServerOnly() && !featureflag.APIServerNodes.Enabled() {
@@ -116,10 +121,10 @@ func PopulateInstanceGroupSpec(cluster *kops.Cluster, input *kops.InstanceGroup,
 			}
 		}
 		if ig.Spec.MinSize == nil {
-			ig.Spec.MinSize = fi.Int32(2)
+			ig.Spec.MinSize = fi.PtrTo(int32(2))
 		}
 		if ig.Spec.MaxSize == nil {
-			ig.Spec.MaxSize = fi.Int32(2)
+			ig.Spec.MaxSize = fi.PtrTo(int32(2))
 		}
 	}
 
@@ -128,9 +133,9 @@ func PopulateInstanceGroupSpec(cluster *kops.Cluster, input *kops.InstanceGroup,
 		if err != nil {
 			return nil, fmt.Errorf("unable to determine machine architecture for InstanceGroup %q: %v", ig.ObjectMeta.Name, err)
 		}
-		ig.Spec.Image = defaultImage(cluster, channel, architecture)
-		if ig.Spec.Image == "" {
-			return nil, fmt.Errorf("unable to determine default image for InstanceGroup %s", ig.ObjectMeta.Name)
+		ig.Spec.Image, err = defaultImage(cluster, channel, architecture)
+		if err != nil {
+			return nil, fmt.Errorf("unable to determine default image for instance group %q: %v", ig.ObjectMeta.Name, err)
 		}
 	}
 
@@ -145,13 +150,13 @@ func PopulateInstanceGroupSpec(cluster *kops.Cluster, input *kops.InstanceGroup,
 		}
 	}
 
-	if ig.IsMaster() {
+	if ig.IsControlPlane() {
 		if len(ig.Spec.Subnets) == 0 {
-			return nil, fmt.Errorf("master InstanceGroup %s did not specify any Subnets", ig.ObjectMeta.Name)
+			return nil, fmt.Errorf("control-plane InstanceGroup %s did not specify any Subnets", ig.ObjectMeta.Name)
 		}
 	} else if ig.IsAPIServerOnly() && cluster.Spec.IsIPv6Only() {
 		if len(ig.Spec.Subnets) == 0 {
-			for _, subnet := range cluster.Spec.Subnets {
+			for _, subnet := range cluster.Spec.Networking.Subnets {
 				if subnet.Type != kops.SubnetTypePrivate && subnet.Type != kops.SubnetTypeUtility {
 					ig.Spec.Subnets = append(ig.Spec.Subnets, subnet.Name)
 				}
@@ -159,7 +164,7 @@ func PopulateInstanceGroupSpec(cluster *kops.Cluster, input *kops.InstanceGroup,
 		}
 	} else {
 		if len(ig.Spec.Subnets) == 0 {
-			for _, subnet := range cluster.Spec.Subnets {
+			for _, subnet := range cluster.Spec.Networking.Subnets {
 				if subnet.Type != kops.SubnetTypeDualStack && subnet.Type != kops.SubnetTypeUtility {
 					ig.Spec.Subnets = append(ig.Spec.Subnets, subnet.Name)
 				}
@@ -167,7 +172,7 @@ func PopulateInstanceGroupSpec(cluster *kops.Cluster, input *kops.InstanceGroup,
 		}
 
 		if len(ig.Spec.Subnets) == 0 {
-			for _, subnet := range cluster.Spec.Subnets {
+			for _, subnet := range cluster.Spec.Networking.Subnets {
 				if subnet.Type != kops.SubnetTypeUtility {
 					ig.Spec.Subnets = append(ig.Spec.Subnets, subnet.Name)
 				}
@@ -181,11 +186,11 @@ func PopulateInstanceGroupSpec(cluster *kops.Cluster, input *kops.InstanceGroup,
 
 	hasGPU := false
 	clusterNvidia := false
-	if cluster.Spec.Containerd != nil && cluster.Spec.Containerd.NvidiaGPU != nil && fi.BoolValue(cluster.Spec.Containerd.NvidiaGPU.Enabled) {
+	if cluster.Spec.Containerd != nil && cluster.Spec.Containerd.NvidiaGPU != nil && fi.ValueOf(cluster.Spec.Containerd.NvidiaGPU.Enabled) {
 		clusterNvidia = true
 	}
 	igNvidia := false
-	if ig.Spec.Containerd != nil && ig.Spec.Containerd.NvidiaGPU != nil && fi.BoolValue(ig.Spec.Containerd.NvidiaGPU.Enabled) {
+	if ig.Spec.Containerd != nil && ig.Spec.Containerd.NvidiaGPU != nil && fi.ValueOf(ig.Spec.Containerd.NvidiaGPU.Enabled) {
 		igNvidia = true
 	}
 
@@ -230,15 +235,15 @@ func PopulateInstanceGroupSpec(cluster *kops.Cluster, input *kops.InstanceGroup,
 
 	var igKubeletConfig *kops.KubeletConfigSpec
 	// Start with the cluster kubelet config
-	if ig.IsMaster() {
-		if cluster.Spec.MasterKubelet != nil {
-			igKubeletConfig = cluster.Spec.MasterKubelet.DeepCopy()
+	if ig.IsControlPlane() {
+		if cluster.Spec.ControlPlaneKubelet != nil {
+			igKubeletConfig = cluster.Spec.ControlPlaneKubelet.DeepCopy()
 		} else {
 			igKubeletConfig = &kops.KubeletConfigSpec{}
 		}
-		// A few settings in Kubelet override those in MasterKubelet. I'm not sure why.
+		// A few settings in Kubelet override those in ControlPlaneKubelet. I'm not sure why.
 		if cluster.Spec.Kubelet != nil && cluster.Spec.Kubelet.AnonymousAuth != nil && !*cluster.Spec.Kubelet.AnonymousAuth {
-			igKubeletConfig.AnonymousAuth = fi.Bool(false)
+			igKubeletConfig.AnonymousAuth = fi.PtrTo(false)
 		}
 	} else {
 		if cluster.Spec.Kubelet != nil {
@@ -252,7 +257,7 @@ func PopulateInstanceGroupSpec(cluster *kops.Cluster, input *kops.InstanceGroup,
 	// rolling update will still replace nodes when they change.
 	igKubeletConfig.NodeLabels = nodelabels.BuildNodeLabels(cluster, ig)
 
-	useSecureKubelet := fi.BoolValue(igKubeletConfig.AnonymousAuth)
+	useSecureKubelet := fi.ValueOf(igKubeletConfig.AnonymousAuth)
 
 	// While slices are overridden in most cases, taints are explicitly merged
 	taints := sets.NewString(igKubeletConfig.Taints...)
@@ -267,10 +272,25 @@ func PopulateInstanceGroupSpec(cluster *kops.Cluster, input *kops.InstanceGroup,
 		reflectutils.JSONMergeStruct(igKubeletConfig, ig.Spec.Kubelet)
 	}
 
+	{
+		if ig.IsControlPlane() {
+			// (Even though the value is empty, we still expect <Key>=<Value>:<Effect>)
+			if cluster.IsKubernetesLT("1.24") {
+				taints.Insert(nodelabels.RoleLabelMaster16 + "=:" + string(v1.TaintEffectNoSchedule))
+			} else {
+				taints.Insert(nodelabels.RoleLabelControlPlane20 + "=:" + string(v1.TaintEffectNoSchedule))
+			}
+		}
+		if ig.IsAPIServerOnly() {
+			// (Even though the value is empty, we still expect <Key>=<Value>:<Effect>)
+			taints.Insert(nodelabels.RoleLabelAPIServer16 + "=:" + string(v1.TaintEffectNoSchedule))
+		}
+	}
+
 	igKubeletConfig.Taints = taints.List()
 
 	if useSecureKubelet {
-		igKubeletConfig.AnonymousAuth = fi.Bool(false)
+		igKubeletConfig.AnonymousAuth = fi.PtrTo(false)
 	}
 
 	ig.Spec.Kubelet = igKubeletConfig
@@ -294,7 +314,7 @@ func defaultMachineType(cloud fi.Cloud, cluster *kops.Cluster, ig *kops.Instance
 
 	case kops.CloudProviderGCE:
 		switch ig.Spec.Role {
-		case kops.InstanceGroupRoleMaster:
+		case kops.InstanceGroupRoleControlPlane:
 			return defaultMasterMachineTypeGCE, nil
 
 		case kops.InstanceGroupRoleNode:
@@ -306,7 +326,7 @@ func defaultMachineType(cloud fi.Cloud, cluster *kops.Cluster, ig *kops.Instance
 
 	case kops.CloudProviderDO:
 		switch ig.Spec.Role {
-		case kops.InstanceGroupRoleMaster:
+		case kops.InstanceGroupRoleControlPlane:
 			return defaultMasterMachineTypeDO, nil
 
 		case kops.InstanceGroupRoleNode:
@@ -316,7 +336,7 @@ func defaultMachineType(cloud fi.Cloud, cluster *kops.Cluster, ig *kops.Instance
 
 	case kops.CloudProviderHetzner:
 		switch ig.Spec.Role {
-		case kops.InstanceGroupRoleMaster:
+		case kops.InstanceGroupRoleControlPlane:
 			return defaultMasterMachineTypeHetzner, nil
 
 		case kops.InstanceGroupRoleNode:
@@ -335,7 +355,7 @@ func defaultMachineType(cloud fi.Cloud, cluster *kops.Cluster, ig *kops.Instance
 
 	case kops.CloudProviderAzure:
 		switch ig.Spec.Role {
-		case kops.InstanceGroupRoleMaster:
+		case kops.InstanceGroupRoleControlPlane:
 			return defaultMasterMachineTypeAzure, nil
 
 		case kops.InstanceGroupRoleNode:
@@ -344,6 +364,15 @@ func defaultMachineType(cloud fi.Cloud, cluster *kops.Cluster, ig *kops.Instance
 		case kops.InstanceGroupRoleBastion:
 			return defaultBastionMachineTypeAzure, nil
 		}
+
+	case kops.CloudProviderScaleway:
+		switch ig.Spec.Role {
+		case kops.InstanceGroupRoleControlPlane:
+			return defaultMasterMachineTypeScaleway, nil
+
+		case kops.InstanceGroupRoleNode:
+			return defaultNodeMachineTypeScaleway, nil
+		}
 	}
 
 	klog.V(2).Infof("Cannot set default MachineType for CloudProvider=%q, Role=%q", cluster.Spec.GetCloudProvider(), ig.Spec.Role)
