@@ -45,9 +45,6 @@ import (
 	"k8s.io/apimachinery/pkg/api/resource"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/klog/v2"
-	"k8s.io/kops/pkg/flagbuilder"
-	"sigs.k8s.io/yaml"
-
 	kopscontrollerconfig "k8s.io/kops/cmd/kops-controller/pkg/config"
 	"k8s.io/kops/pkg/apis/kops"
 	apiModel "k8s.io/kops/pkg/apis/kops/model"
@@ -55,6 +52,7 @@ import (
 	"k8s.io/kops/pkg/apis/nodeup"
 	"k8s.io/kops/pkg/dns"
 	"k8s.io/kops/pkg/featureflag"
+	"k8s.io/kops/pkg/flagbuilder"
 	"k8s.io/kops/pkg/kubemanifest"
 	"k8s.io/kops/pkg/model"
 	"k8s.io/kops/pkg/model/components/kopscontroller"
@@ -65,7 +63,10 @@ import (
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
 	"k8s.io/kops/upup/pkg/fi/cloudup/gce"
 	gcetpm "k8s.io/kops/upup/pkg/fi/cloudup/gce/tpm"
+	"k8s.io/kops/upup/pkg/fi/cloudup/hetzner"
+	"k8s.io/kops/upup/pkg/fi/cloudup/openstack"
 	"k8s.io/kops/util/pkg/env"
+	"sigs.k8s.io/yaml"
 )
 
 // TemplateFunctions provides a collection of methods used throughout the templates
@@ -113,23 +114,28 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 	dest["GetCloudProvider"] = cluster.Spec.GetCloudProvider
 	dest["GetInstanceGroup"] = tf.GetInstanceGroup
 	dest["GetNodeInstanceGroups"] = tf.GetNodeInstanceGroups
+	dest["GetClusterAutoscalerNodeGroups"] = tf.GetClusterAutoscalerNodeGroups
 	dest["HasHighlyAvailableControlPlane"] = tf.HasHighlyAvailableControlPlane
 	dest["ControlPlaneControllerReplicas"] = tf.ControlPlaneControllerReplicas
 	dest["APIServerNodeRole"] = tf.APIServerNodeRole
+	dest["APIInternalName"] = tf.Cluster.APIInternalName
 
 	dest["CloudTags"] = tf.CloudTagsForInstanceGroup
 	dest["KubeDNS"] = func() *kops.KubeDNSConfig {
 		return cluster.Spec.KubeDNS
 	}
 
-	dest["GossipDomains"] = func() []string {
-		var names []string
-
-		if dns.IsGossipHostname(cluster.Spec.MasterInternalName) {
-			names = append(names, "k8s.local")
+	dest["GossipEnabled"] = func() bool {
+		if cluster.IsGossip() {
+			return true
 		}
-
-		return names
+		return false
+	}
+	dest["GossipName"] = func() bool {
+		if dns.IsGossipClusterName(cluster.Name) {
+			return true
+		}
+		return false
 	}
 
 	dest["NodeLocalDNSClusterIP"] = func() string {
@@ -172,12 +178,17 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 		return os.Getenv("HCLOUD_TOKEN")
 	}
 	dest["HCLOUD_NETWORK"] = func() string {
-		if cluster.Spec.NetworkID != "" {
-			return cluster.Spec.NetworkID
+		if cluster.Spec.Networking.NetworkID != "" {
+			return cluster.Spec.Networking.NetworkID
 		}
 		return cluster.Name
 	}
 
+	dest["OPENSTACK_CONF"] = func() string {
+		lines := openstack.MakeCloudConfig(cluster.Spec)
+		return "[global]\n" + strings.Join(lines, "\n") + "\n"
+	}
+
 	if featureflag.Spotinst.Enabled() {
 		if creds, err := spotinst.LoadCredentials(); err == nil {
 			dest["SpotinstToken"] = func() string { return creds.Token }
@@ -187,7 +198,7 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 		}
 	}
 
-	if cluster.Spec.Networking != nil && cluster.Spec.Networking.AmazonVPC != nil {
+	if cluster.Spec.Networking.AmazonVPC != nil {
 		c := cluster.Spec.Networking.AmazonVPC
 		dest["AmazonVpcEnvVars"] = func() map[string]string {
 			envVars := map[string]string{
@@ -232,7 +243,7 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 		}
 	}
 
-	if cluster.Spec.Networking != nil && cluster.Spec.Networking.Calico != nil {
+	if cluster.Spec.Networking.Calico != nil {
 		c := cluster.Spec.Networking.Calico
 		dest["CalicoIPv4PoolIPIPMode"] = func() string {
 			if c.EncapsulationMode != "ipip" {
@@ -257,7 +268,7 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 		}
 	}
 
-	if cluster.Spec.Networking != nil && cluster.Spec.Networking.Cilium != nil {
+	if cluster.Spec.Networking.Cilium != nil {
 		ciliumsecretString := ""
 		ciliumsecret, _ := secretStore.Secret("ciliumpassword")
 		if ciliumsecret != nil {
@@ -271,7 +282,7 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 		dest["CiliumSecret"] = func() string { return ciliumsecretString }
 	}
 
-	if cluster.Spec.Networking != nil && cluster.Spec.Networking.Flannel != nil {
+	if cluster.Spec.Networking.Flannel != nil {
 		flannelBackendType := cluster.Spec.Networking.Flannel.Backend
 		if flannelBackendType == "" {
 			klog.Warningf("Defaulting flannel backend to udp (not a recommended configuration)")
@@ -280,7 +291,7 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 		dest["FlannelBackendType"] = func() string { return flannelBackendType }
 	}
 
-	if cluster.Spec.Networking != nil && cluster.Spec.Networking.Weave != nil {
+	if cluster.Spec.Networking.Weave != nil {
 		weavesecretString := ""
 		weavesecret, _ := secretStore.Secret("weavepassword")
 		if weavesecret != nil {
@@ -309,7 +320,27 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 	dest["IsIPv6Only"] = tf.IsIPv6Only
 	dest["UseServiceAccountExternalPermissions"] = tf.UseServiceAccountExternalPermissions
 
-	if cluster.Spec.NodeTerminationHandler != nil {
+	if cluster.Spec.ClusterAutoscaler != nil {
+		dest["ClusterAutoscalerPriorities"] = func() map[string][]string {
+			priorities := make(map[string][]string)
+			if cluster.Spec.ClusterAutoscaler.CustomPriorityExpanderConfig != nil {
+				priorities = cluster.Spec.ClusterAutoscaler.CustomPriorityExpanderConfig
+			} else {
+				for name, spec := range tf.GetNodeInstanceGroups() {
+
+					if spec.Autoscale != nil {
+						priorities[fmt.Sprint(spec.AutoscalePriority)] = append(priorities[fmt.Sprint(spec.AutoscalePriority)], fmt.Sprintf("%s.%s", name, tf.ClusterName()))
+					}
+				}
+			}
+			return priorities
+		}
+		dest["CreateClusterAutoscalerPriorityConfig"] = func() bool {
+			return fi.ValueOf(cluster.Spec.ClusterAutoscaler.CreatePriorityExpenderConfig)
+		}
+	}
+
+	if cluster.Spec.CloudProvider.AWS != nil && cluster.Spec.CloudProvider.AWS.NodeTerminationHandler != nil {
 		dest["DefaultQueueName"] = func() string {
 			s := strings.Replace(tf.ClusterName(), ".", "-", -1)
 			domain := ".amazonaws.com/"
@@ -320,7 +351,7 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 			return url
 		}
 
-		dest["EnableSQSTerminationDraining"] = func() bool { return *cluster.Spec.NodeTerminationHandler.EnableSQSTerminationDraining }
+		dest["EnableSQSTerminationDraining"] = func() bool { return cluster.Spec.CloudProvider.AWS.NodeTerminationHandler.IsQueueMode() }
 	}
 
 	dest["ArchitectureOfAMI"] = tf.architectureOfAMI
@@ -339,12 +370,14 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 
 	dest["HasSnapshotController"] = func() bool {
 		sc := cluster.Spec.SnapshotController
-		return sc != nil && fi.BoolValue(sc.Enabled)
+		return sc != nil && fi.ValueOf(sc.Enabled)
 	}
 
 	dest["IsKubernetesGTE"] = tf.IsKubernetesGTE
 	dest["IsKubernetesLT"] = tf.IsKubernetesLT
 
+	dest["KopsFeatureEnabled"] = tf.kopsFeatureEnabled
+
 	return nil
 }
 
@@ -387,7 +420,7 @@ func (tf *TemplateFunctions) GetInstanceGroup(name string) (*kops.InstanceGroup,
 // deployOnWorkersIfExternalPermissons should be true if a controller runs on worker nodes when external IAM permissions is enabled for the cluster.
 // In this case it is assumed that it can run 2 replicas.
 func (tf *TemplateFunctions) ControlPlaneControllerReplicas(deployOnWorkersIfExternalPermissions bool) int {
-	if deployOnWorkersIfExternalPermissions && tf.Cluster.Spec.IAM != nil && fi.BoolValue(tf.Cluster.Spec.IAM.UseServiceAccountExternalPermissions) {
+	if deployOnWorkersIfExternalPermissions && tf.Cluster.Spec.IAM != nil && fi.ValueOf(tf.Cluster.Spec.IAM.UseServiceAccountExternalPermissions) {
 		return 2
 	}
 	if tf.HasHighlyAvailableControlPlane() {
@@ -409,7 +442,7 @@ func (tf *TemplateFunctions) APIServerNodeRole() string {
 func (tf *TemplateFunctions) HasHighlyAvailableControlPlane() bool {
 	cp := 0
 	for _, ig := range tf.InstanceGroups {
-		if ig.Spec.Role == kops.InstanceGroupRoleMaster {
+		if ig.Spec.Role == kops.InstanceGroupRoleControlPlane {
 			cp++
 			if cp > 1 {
 				return true
@@ -473,7 +506,7 @@ func (tf *TemplateFunctions) DNSControllerArgv() ([]string, error) {
 		// @check if the watch ingress is set
 		var watchIngress bool
 		if cluster.Spec.ExternalDNS.WatchIngress != nil {
-			watchIngress = fi.BoolValue(cluster.Spec.ExternalDNS.WatchIngress)
+			watchIngress = fi.ValueOf(cluster.Spec.ExternalDNS.WatchIngress)
 		}
 
 		if watchIngress {
@@ -486,7 +519,7 @@ func (tf *TemplateFunctions) DNSControllerArgv() ([]string, error) {
 		}
 	}
 
-	if dns.IsGossipHostname(cluster.Spec.MasterInternalName) {
+	if cluster.IsGossip() {
 		argv = append(argv, "--dns=gossip")
 
 		// Configuration specifically for the DNS controller gossip
@@ -580,8 +613,10 @@ func (tf *TemplateFunctions) KopsControllerConfig() (string, error) {
 	cluster := tf.Cluster
 
 	config := &kopscontrollerconfig.Options{
-		Cloud:      string(cluster.Spec.GetCloudProvider()),
-		ConfigBase: cluster.Spec.ConfigBase,
+		ClusterName: cluster.Name,
+		Cloud:       string(cluster.Spec.GetCloudProvider()),
+		ConfigBase:  cluster.Spec.ConfigBase,
+		SecretStore: cluster.Spec.SecretStore,
 	}
 
 	if featureflag.CacheNodeidentityInfo.Enabled() {
@@ -598,7 +633,7 @@ func (tf *TemplateFunctions) KopsControllerConfig() (string, error) {
 		if cluster.Spec.KubeProxy.Enabled == nil || *cluster.Spec.KubeProxy.Enabled {
 			certNames = append(certNames, "kube-proxy")
 		}
-		if cluster.Spec.Networking.Kuberouter != nil {
+		if cluster.Spec.Networking.KubeRouter != nil {
 			certNames = append(certNames, "kube-router")
 		}
 
@@ -657,6 +692,13 @@ func (tf *TemplateFunctions) KopsControllerConfig() (string, error) {
 				Region:      tf.Region,
 				MaxTimeSkew: 300,
 			}
+
+		case kops.CloudProviderHetzner:
+			config.Server.Provider.Hetzner = &hetzner.HetznerVerifierOptions{}
+
+		case kops.CloudProviderOpenstack:
+			config.Server.Provider.OpenStack = &openstack.OpenStackVerifierOptions{}
+
 		default:
 			return "", fmt.Errorf("unsupported cloud provider %s", cluster.Spec.GetCloudProvider())
 		}
@@ -666,7 +708,7 @@ func (tf *TemplateFunctions) KopsControllerConfig() (string, error) {
 		config.EnableCloudIPAM = true
 	}
 
-	if dns.IsGossipHostname(cluster.Spec.MasterInternalName) {
+	if cluster.IsGossip() {
 		config.Discovery = &kopscontrollerconfig.DiscoveryOptions{
 			Enabled: true,
 		}
@@ -705,7 +747,7 @@ func (tf *TemplateFunctions) ExternalDNSArgv() ([]string, error) {
 	case kops.CloudProviderAWS:
 		argv = append(argv, "--provider=aws")
 	case kops.CloudProviderGCE:
-		project := cluster.Spec.Project
+		project := cluster.Spec.CloudProvider.GCE.Project
 		argv = append(argv, "--provider=google")
 		argv = append(argv, "--google-project="+project)
 	default:
@@ -733,7 +775,7 @@ func (tf *TemplateFunctions) ProxyEnv() map[string]string {
 	cluster := tf.Cluster
 
 	envs := map[string]string{}
-	proxies := cluster.Spec.EgressProxy
+	proxies := cluster.Spec.Networking.EgressProxy
 	if proxies == nil {
 		return envs
 	}
@@ -774,9 +816,11 @@ func (tf *TemplateFunctions) OpenStackCCMTag() string {
 		if parsed.Minor == 23 {
 			tag = "v1.23.1"
 		} else if parsed.Minor == 24 {
-			tag = "v1.24.5"
+			tag = "v1.24.6"
 		} else if parsed.Minor == 25 {
-			tag = "v1.25.3"
+			tag = "v1.25.4"
+		} else if parsed.Minor == 26 {
+			tag = "v1.26.1"
 		} else {
 			// otherwise we use always .0 ccm image, if needed that can be overrided using clusterspec
 			tag = fmt.Sprintf("v%d.%d.0", parsed.Major, parsed.Minor)
@@ -794,9 +838,11 @@ func (tf *TemplateFunctions) OpenStackCSITag() string {
 		tag = "latest"
 	} else {
 		if parsed.Minor == 24 {
-			tag = "v1.24.5"
+			tag = "v1.24.6"
 		} else if parsed.Minor == 25 {
-			tag = "v1.25.3"
+			tag = "v1.25.4"
+		} else if parsed.Minor == 26 {
+			tag = "v1.26.1"
 		} else {
 			// otherwise we use always .0 csi image, if needed that can be overrided using cloud config spec
 			tag = fmt.Sprintf("v%d.%d.0", parsed.Major, parsed.Minor)
@@ -816,6 +862,37 @@ func (tf *TemplateFunctions) GetNodeInstanceGroups() map[string]kops.InstanceGro
 	return nodegroups
 }
 
+type ClusterAutoscalerNodeGroup struct {
+	AutoScale *bool
+	MinSize   int32
+	MaxSize   int32
+	Other     string
+}
+
+// GetClusterAutoscalerGroups returns a map containing ClusterAutoscaler info for each instance group of type Node.
+func (tf *TemplateFunctions) GetClusterAutoscalerNodeGroups() map[string]ClusterAutoscalerNodeGroup {
+	cluster := tf.Cluster
+	groups := make(map[string]ClusterAutoscalerNodeGroup)
+	for _, ig := range tf.KopsModelContext.InstanceGroups {
+		if ig.Spec.Role == kops.InstanceGroupRoleNode && (ig.Spec.Autoscale == nil || fi.ValueOf(ig.Spec.Autoscale)) {
+			group := ClusterAutoscalerNodeGroup{
+				AutoScale: ig.Spec.Autoscale,
+				MinSize:   fi.ValueOf(ig.Spec.MinSize),
+				MaxSize:   fi.ValueOf(ig.Spec.MaxSize),
+			}
+			if cluster.Spec.GetCloudProvider() == kops.CloudProviderGCE {
+				cloud := tf.cloud.(gce.GCECloud)
+				format := "https://www.googleapis.com/compute/v1/projects/%s/zones/%s/instanceGroups/%s"
+				group.Other = fmt.Sprintf(format, cloud.Project(), ig.Spec.Zones[0], gce.NameForInstanceGroupManager(cluster, ig, ig.Spec.Zones[0]))
+			} else {
+				group.Other = ig.Name + "." + cluster.Name
+			}
+			groups[ig.Name] = group
+		}
+	}
+	return groups
+}
+
 func (tf *TemplateFunctions) architectureOfAMI(amiID string) string {
 	image, _ := tf.cloud.(awsup.AWSCloud).ResolveImage(amiID)
 	switch *image.Architecture {
@@ -892,8 +969,8 @@ func karpenterInstanceTypes(cloud awsup.AWSCloud, ig kops.InstanceGroupSpec) ([]
 			ir := &ec2.InstanceRequirementsRequest{
 				VCpuCount:            &ec2.VCpuCountRangeRequest{},
 				MemoryMiB:            &ec2.MemoryMiBRequest{},
-				BurstablePerformance: fi.String("included"),
-				InstanceGenerations:  []*string{fi.String("current")},
+				BurstablePerformance: fi.PtrTo("included"),
+				InstanceGenerations:  []*string{fi.PtrTo("current")},
 			}
 			cpu := instanceRequirements.CPU
 			if cpu != nil {
@@ -912,7 +989,7 @@ func karpenterInstanceTypes(cloud awsup.AWSCloud, ig kops.InstanceGroupSpec) ([]
 						ir.VCpuCount.Min = &cpuMin
 					}
 				} else {
-					ir.VCpuCount.Min = fi.Int64(0)
+					ir.VCpuCount.Min = fi.PtrTo(int64(0))
 				}
 
 				memory := instanceRequirements.Memory
@@ -926,12 +1003,12 @@ func karpenterInstanceTypes(cloud awsup.AWSCloud, ig kops.InstanceGroupSpec) ([]
 						ir.MemoryMiB.Min = &memoryMin
 					}
 				} else {
-					ir.MemoryMiB.Min = fi.Int64(0)
+					ir.MemoryMiB.Min = fi.PtrTo(int64(0))
 				}
 
 				ir.AcceleratorCount = &ec2.AcceleratorCountRequest{
-					Min: fi.Int64(0),
-					Max: fi.Int64(0),
+					Min: fi.PtrTo(int64(0)),
+					Max: fi.PtrTo(int64(0)),
 				}
 
 				response, err := cloud.EC2().GetInstanceTypesFromInstanceRequirements(
@@ -958,3 +1035,11 @@ func karpenterInstanceTypes(cloud awsup.AWSCloud, ig kops.InstanceGroupSpec) ([]
 
 	return []string{ig.MachineType}, nil
 }
+
+func (tf *TemplateFunctions) kopsFeatureEnabled(featureName string) (bool, error) {
+	f, err := featureflag.Get(featureName)
+	if err != nil {
+		return false, err
+	}
+	return f.Enabled(), nil
+}
