@@ -43,17 +43,18 @@ type ManagedFile struct {
 
 	Contents fi.Resource
 
-	// Public controls whether the object is world-readable
-	Public *bool
+	// PublicACL controls whether the _object_ has an ACL which grants world-readable status.
+	// Note that the _bucket_ may itself have a grant for world-readable; that is separate.
+	PublicACL *bool
 }
 
-func (e *ManagedFile) Find(c *fi.Context) (*ManagedFile, error) {
+func (e *ManagedFile) Find(c *fi.CloudupContext) (*ManagedFile, error) {
 	managedFiles, err := getBasePath(c, e)
 	if err != nil {
 		return nil, err
 	}
 
-	location := fi.StringValue(e.Location)
+	location := fi.ValueOf(e.Location)
 	if location == "" {
 		return nil, nil
 	}
@@ -80,10 +81,10 @@ func (e *ManagedFile) Find(c *fi.Context) (*ManagedFile, error) {
 		if err != nil {
 			return nil, err
 		}
-		actual.Public = &public
+		actual.PublicACL = &public
 
-		if e.Public == nil {
-			e.Public = fi.Bool(false)
+		if e.PublicACL == nil {
+			e.PublicACL = fi.PtrTo(false)
 		}
 	}
 
@@ -92,10 +93,10 @@ func (e *ManagedFile) Find(c *fi.Context) (*ManagedFile, error) {
 		if err != nil {
 			return nil, err
 		}
-		actual.Public = &public
+		actual.PublicACL = &public
 
-		if e.Public == nil {
-			e.Public = fi.Bool(false)
+		if e.PublicACL == nil {
+			e.PublicACL = fi.PtrTo(false)
 		}
 	}
 
@@ -105,8 +106,8 @@ func (e *ManagedFile) Find(c *fi.Context) (*ManagedFile, error) {
 	return actual, nil
 }
 
-func (e *ManagedFile) Run(c *fi.Context) error {
-	return fi.DefaultDeltaRunMethod(e, c)
+func (e *ManagedFile) Run(c *fi.CloudupContext) error {
+	return fi.CloudupDefaultDeltaRunMethod(e, c)
 }
 
 func (s *ManagedFile) CheckChanges(a, e, changes *ManagedFile) error {
@@ -121,20 +122,22 @@ func (s *ManagedFile) CheckChanges(a, e, changes *ManagedFile) error {
 	return nil
 }
 
-func (e *ManagedFile) getACL(c *fi.Context, p vfs.Path) (vfs.ACL, error) {
+func (e *ManagedFile) getACL(c *fi.CloudupContext, p vfs.Path) (vfs.ACL, error) {
+	ctx := c.Context()
+
 	var acl vfs.ACL
-	if fi.BoolValue(e.Public) {
+	if fi.ValueOf(e.PublicACL) {
 		switch p := p.(type) {
 		case *vfs.S3Path:
 			acl = &vfs.S3Acl{
-				RequestACL: fi.String("public-read"),
+				RequestACL: fi.PtrTo("public-read"),
 			}
 		case *vfs.MemFSPath:
 			if !p.IsClusterReadable() {
 				return nil, fmt.Errorf("the %q path is intended for use in tests", p.Path())
 			}
 			acl = &vfs.S3Acl{
-				RequestACL: fi.String("public-read"),
+				RequestACL: fi.PtrTo("public-read"),
 			}
 		default:
 			return nil, fmt.Errorf("the %q path does not support public ACL", p.Path())
@@ -142,11 +145,11 @@ func (e *ManagedFile) getACL(c *fi.Context, p vfs.Path) (vfs.ACL, error) {
 		return acl, nil
 	}
 
-	return acls.GetACL(p, c.Cluster)
+	return acls.GetACL(ctx, p, c.T.Cluster)
 }
 
-func (_ *ManagedFile) Render(c *fi.Context, a, e, changes *ManagedFile) error {
-	location := fi.StringValue(e.Location)
+func (_ *ManagedFile) Render(c *fi.CloudupContext, a, e, changes *ManagedFile) error {
+	location := fi.ValueOf(e.Location)
 	if location == "" {
 		return fi.RequiredField("Location")
 	}
@@ -175,8 +178,8 @@ func (_ *ManagedFile) Render(c *fi.Context, a, e, changes *ManagedFile) error {
 	return nil
 }
 
-func getBasePath(c *fi.Context, e *ManagedFile) (vfs.Path, error) {
-	base := fi.StringValue(e.Base)
+func getBasePath(c *fi.CloudupContext, e *ManagedFile) (vfs.Path, error) {
+	base := fi.ValueOf(e.Base)
 	if base != "" {
 		p, err := vfs.Context.BuildVfsPath(base)
 		if err != nil {
@@ -185,16 +188,16 @@ func getBasePath(c *fi.Context, e *ManagedFile) (vfs.Path, error) {
 		return p, nil
 	}
 
-	return c.ClusterConfigBase, nil
+	return c.T.ClusterConfigBase, nil
 }
 
 // RenderTerraform is responsible for rendering the terraform json.
-func (f *ManagedFile) RenderTerraform(c *fi.Context, t *terraform.TerraformTarget, a, e, changes *ManagedFile) error {
+func (f *ManagedFile) RenderTerraform(c *fi.CloudupContext, t *terraform.TerraformTarget, a, e, changes *ManagedFile) error {
 	if !featureflag.TerraformManagedFiles.Enabled() {
 		return f.Render(c, a, e, changes)
 	}
 
-	location := fi.StringValue(e.Location)
+	location := fi.ValueOf(e.Location)
 	if location == "" {
 		return fi.RequiredField("Location")
 	}
